@@ -17,53 +17,40 @@
 
 from copy import deepcopy
 from cStringIO import StringIO
-import errno
-import os
-import shutil
-import sys
 from unittest import TestSuite
 from warnings import warn
 
 import bzrlib
-import bzrlib.bzrdir as bzrdir
+from bzrlib import (
+        bzrdir,
+        cache_utf8,
+        errors,
+        lockdir,
+        osutils,
+        revision,
+        transport,
+        tree,
+        ui,
+        urlutils,
+        )
 from bzrlib.config import TreeConfig
 from bzrlib.decorators import needs_read_lock, needs_write_lock
-from bzrlib.delta import compare_trees
 import bzrlib.errors as errors
-from bzrlib.errors import (BzrError, InvalidRevisionNumber, InvalidRevisionId,
-                           NoSuchRevision, HistoryMissing, NotBranchError,
-                           DivergedBranches, LockError,
-                           UninitializableFormat,
-                           UnlistableStore,
-                           UnlistableBranch, NoSuchFile, NotVersionedError,
-                           NoWorkingTree)
-import bzrlib.inventory as inventory
-from bzrlib.inventory import Inventory
+from bzrlib.errors import (BzrError, BzrCheckError, DivergedBranches, 
+                           HistoryMissing, InvalidRevisionId, 
+                           InvalidRevisionNumber, LockError, NoSuchFile, 
+                           NoSuchRevision, NoWorkingTree, NotVersionedError,
+                           NotBranchError, UninitializableFormat, 
+                           UnlistableStore, UnlistableBranch, 
+                           )
 from bzrlib.lockable_files import LockableFiles, TransportLock
-from bzrlib.lockdir import LockDir
-from bzrlib.osutils import (isdir, quotefn,
-                            rename, splitpath, sha_file,
-                            file_kind, abspath, normpath, pathjoin,
-                            safe_unicode,
-                            rmtree,
-                            )
-from bzrlib.textui import show_status
+from bzrlib.symbol_versioning import (deprecated_function,
+                                      deprecated_method,
+                                      DEPRECATED_PARAMETER,
+                                      deprecated_passed,
+                                      zero_eight, zero_nine,
+                                      )
 from bzrlib.trace import mutter, note
-from bzrlib.tree import EmptyTree, RevisionTree
-from bzrlib.repository import Repository
-from bzrlib.revision import (
-                             get_intervening_revisions,
-                             is_ancestor,
-                             NULL_REVISION,
-                             Revision,
-                             )
-from bzrlib.store import copy_all
-from bzrlib.symbol_versioning import *
-import bzrlib.transactions as transactions
-from bzrlib.transport import Transport, get_transport
-from bzrlib.tree import EmptyTree, RevisionTree
-import bzrlib.ui
-import bzrlib.xml5
 
 
 BZR_BRANCH_FORMAT_4 = "Bazaar-NG branch, format 0.0.4\n"
@@ -118,10 +105,10 @@ class Branch(object):
         
     @staticmethod
     def open(base, _unsupported=False):
-        """Open the repository rooted at base.
+        """Open the branch rooted at base.
 
-        For instance, if the repository is at URL/.bzr/repository,
-        Repository.open(URL) -> a Repository instance.
+        For instance, if the branch is at URL/.bzr/branch,
+        Branch.open(URL) -> a Branch instance.
         """
         control = bzrdir.BzrDir.open(base, _unsupported)
         return control.open_branch(_unsupported)
@@ -151,43 +138,44 @@ class Branch(object):
         """
         return bzrdir.BzrDir.create_standalone_workingtree(base).branch
 
+    @deprecated_function(zero_eight)
     def setup_caching(self, cache_root):
         """Subclasses that care about caching should override this, and set
         up cached stores located under cache_root.
+        
+        NOTE: This is unused.
         """
-        # seems to be unused, 2006-01-13 mbp
-        warn('%s is deprecated' % self.setup_caching)
-        self.cache_root = cache_root
+        pass
+
+    def get_config(self):
+        return bzrlib.config.BranchConfig(self)
 
     def _get_nick(self):
-        cfg = self.tree_config()
-        return cfg.get_option(u"nickname", default=self.base.split('/')[-2])
+        return self.get_config().get_nickname()
 
     def _set_nick(self, nick):
-        cfg = self.tree_config()
-        cfg.set_option(nick, "nickname")
-        assert cfg.get_option("nickname") == nick
+        self.get_config().set_user_option('nickname', nick)
 
     nick = property(_get_nick, _set_nick)
 
     def is_locked(self):
-        raise NotImplementedError('is_locked is abstract')
+        raise NotImplementedError(self.is_locked)
 
     def lock_write(self):
-        raise NotImplementedError('lock_write is abstract')
+        raise NotImplementedError(self.lock_write)
 
     def lock_read(self):
-        raise NotImplementedError('lock_read is abstract')
+        raise NotImplementedError(self.lock_read)
 
     def unlock(self):
-        raise NotImplementedError('unlock is abstract')
+        raise NotImplementedError(self.unlock)
 
     def peek_lock_mode(self):
         """Return lock mode for the Branch: 'r', 'w' or None"""
         raise NotImplementedError(self.peek_lock_mode)
 
     def get_physical_lock_status(self):
-        raise NotImplementedError('get_physical_lock_status is abstract')
+        raise NotImplementedError(self.get_physical_lock_status)
 
     def abspath(self, name):
         """Return absolute filename for something in the branch
@@ -195,7 +183,7 @@ class Branch(object):
         XXX: Robert Collins 20051017 what is this used for? why is it a branch
         method and not a tree method.
         """
-        raise NotImplementedError('abspath is abstract')
+        raise NotImplementedError(self.abspath)
 
     def bind(self, other):
         """Bind the local branch the other branch.
@@ -220,7 +208,7 @@ class Branch(object):
         if self.base == from_branch.base:
             return (0, [])
         if pb is None:
-            nested_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+            nested_pb = ui.ui_factory.nested_progress_bar()
             pb = nested_pb
         else:
             nested_pb = None
@@ -234,7 +222,7 @@ class Branch(object):
                     last_revision = from_history[-1]
                 else:
                     # no history in the source branch
-                    last_revision = NULL_REVISION
+                    last_revision = revision.NULL_REVISION
             return self.repository.fetch(from_branch.repository,
                                          revision_id=last_revision,
                                          pb=nested_pb)
@@ -250,6 +238,26 @@ class Branch(object):
         branch.
         """
         return None
+    
+    def get_commit_builder(self, parents, config=None, timestamp=None, 
+                           timezone=None, committer=None, revprops=None, 
+                           revision_id=None):
+        """Obtain a CommitBuilder for this branch.
+        
+        :param parents: Revision ids of the parents of the new revision.
+        :param config: Optional configuration to use.
+        :param timestamp: Optional timestamp recorded for commit.
+        :param timezone: Optional timezone for timestamp.
+        :param committer: Optional committer to set for commit.
+        :param revprops: Optional dictionary of revision properties.
+        :param revision_id: Optional revision id.
+        """
+
+        if config is None:
+            config = self.get_config()
+        
+        return self.repository.get_commit_builder(self, parents, config, 
+            timestamp, timezone, committer, revprops, revision_id)
 
     def get_master_branch(self):
         """Return the branch we are bound to.
@@ -258,23 +266,35 @@ class Branch(object):
         """
         return None
 
+    def get_revision_delta(self, revno):
+        """Return the delta for one revision.
+
+        The delta is relative to its mainline predecessor, or the
+        empty tree for revision 1.
+        """
+        assert isinstance(revno, int)
+        rh = self.revision_history()
+        if not (1 <= revno <= len(rh)):
+            raise InvalidRevisionNumber(revno)
+        return self.repository.get_revision_delta(rh[revno-1])
+
     def get_root_id(self):
         """Return the id of this branches root"""
-        raise NotImplementedError('get_root_id is abstract')
+        raise NotImplementedError(self.get_root_id)
 
     def print_file(self, file, revision_id):
         """Print `file` to stdout."""
-        raise NotImplementedError('print_file is abstract')
+        raise NotImplementedError(self.print_file)
 
     def append_revision(self, *revision_ids):
-        raise NotImplementedError('append_revision is abstract')
+        raise NotImplementedError(self.append_revision)
 
     def set_revision_history(self, rev_history):
-        raise NotImplementedError('set_revision_history is abstract')
+        raise NotImplementedError(self.set_revision_history)
 
     def revision_history(self):
         """Return sequence of revision hashes on to this branch."""
-        raise NotImplementedError('revision_history is abstract')
+        raise NotImplementedError(self.revision_history)
 
     def revno(self):
         """Return current revision number for this branch.
@@ -289,7 +309,7 @@ class Branch(object):
         raise errors.UpgradeRequired(self.base)
 
     def last_revision(self):
-        """Return last patch hash, or None if no history."""
+        """Return last revision id, or None"""
         ph = self.revision_history()
         if ph:
             return ph[-1]
@@ -301,32 +321,6 @@ class Branch(object):
         
         If self and other have not diverged, return a list of the revisions
         present in other, but missing from self.
-
-        >>> from bzrlib.workingtree import WorkingTree
-        >>> bzrlib.trace.silent = True
-        >>> d1 = bzrdir.ScratchDir()
-        >>> br1 = d1.open_branch()
-        >>> wt1 = d1.open_workingtree()
-        >>> d2 = bzrdir.ScratchDir()
-        >>> br2 = d2.open_branch()
-        >>> wt2 = d2.open_workingtree()
-        >>> br1.missing_revisions(br2)
-        []
-        >>> wt2.commit("lala!", rev_id="REVISION-ID-1")
-        >>> br1.missing_revisions(br2)
-        [u'REVISION-ID-1']
-        >>> br2.missing_revisions(br1)
-        []
-        >>> wt1.commit("lala!", rev_id="REVISION-ID-1")
-        >>> br1.missing_revisions(br2)
-        []
-        >>> wt2.commit("lala!", rev_id="REVISION-ID-2A")
-        >>> br1.missing_revisions(br2)
-        [u'REVISION-ID-2A']
-        >>> wt1.commit("lala!", rev_id="REVISION-ID-2B")
-        >>> br1.missing_revisions(br2)
-        Traceback (most recent call last):
-        DivergedBranches: These branches have diverged.  Try merge.
         """
         self_history = self.revision_history()
         self_len = len(self_history)
@@ -342,7 +336,7 @@ class Branch(object):
         else:
             assert isinstance(stop_revision, int)
             if stop_revision > other_len:
-                raise bzrlib.errors.NoSuchRevision(self, stop_revision)
+                raise errors.NoSuchRevision(self, stop_revision)
         return other_history[self_len:stop_revision]
 
     def update_revisions(self, other, stop_revision=None):
@@ -352,7 +346,7 @@ class Branch(object):
         :param stop_revision: Updated until the given revision
         :return: None
         """
-        raise NotImplementedError('update_revisions is abstract')
+        raise NotImplementedError(self.update_revisions)
 
     def revision_id_to_revno(self, revision_id):
         """Given a revision id, return its revno"""
@@ -370,18 +364,15 @@ class Branch(object):
             return None
         if history is None:
             history = self.revision_history()
-        elif revno <= 0 or revno > len(history):
+        if revno <= 0 or revno > len(history):
             raise bzrlib.errors.NoSuchRevision(self, revno)
         return history[revno - 1]
 
     def pull(self, source, overwrite=False, stop_revision=None):
-        raise NotImplementedError('pull is abstract')
+        raise NotImplementedError(self.pull)
 
     def basis_tree(self):
-        """Return `Tree` object for last revision.
-
-        If there are no revisions yet, return an `EmptyTree`.
-        """
+        """Return `Tree` object for last revision."""
         return self.repository.revision_tree(self.last_revision())
 
     def rename_one(self, from_rel, to_rel):
@@ -389,7 +380,7 @@ class Branch(object):
 
         This can change the directory or the filename or both.
         """
-        raise NotImplementedError('rename_one is abstract')
+        raise NotImplementedError(self.rename_one)
 
     def move(self, from_paths, to_name):
         """Rename files.
@@ -405,7 +396,7 @@ class Branch(object):
         This returns a list of (from_path, to_path) pairs for each
         entry that is moved.
         """
-        raise NotImplementedError('move is abstract')
+        raise NotImplementedError(self.move)
 
     def get_parent(self):
         """Return the parent location of the branch.
@@ -414,18 +405,36 @@ class Branch(object):
         pattern is that the user can override it by specifying a
         location.
         """
-        raise NotImplementedError('get_parent is abstract')
+        raise NotImplementedError(self.get_parent)
+
+    def get_submit_branch(self):
+        """Return the submit location of the branch.
+
+        This is the default location for bundle.  The usual
+        pattern is that the user can override it by specifying a
+        location.
+        """
+        return self.get_config().get_user_option('submit_branch')
+
+    def set_submit_branch(self, location):
+        """Return the submit location of the branch.
+
+        This is the default location for bundle.  The usual
+        pattern is that the user can override it by specifying a
+        location.
+        """
+        self.get_config().set_user_option('submit_branch', location)
 
     def get_push_location(self):
         """Return the None or the location to push this branch to."""
-        raise NotImplementedError('get_push_location is abstract')
+        raise NotImplementedError(self.get_push_location)
 
     def set_push_location(self, location):
         """Set a new push location for this branch."""
-        raise NotImplementedError('set_push_location is abstract')
+        raise NotImplementedError(self.set_push_location)
 
     def set_parent(self, url):
-        raise NotImplementedError('set_parent is abstract')
+        raise NotImplementedError(self.set_parent)
 
     @needs_write_lock
     def update(self):
@@ -458,7 +467,7 @@ class Branch(object):
         revision_id: if not None, the revision history in the new branch will
                      be truncated to end with revision_id.
         """
-        # for API compatability, until 0.8 releases we provide the old api:
+        # for API compatibility, until 0.8 releases we provide the old api:
         # def clone(self, to_location, revision=None, basis_branch=None, to_branch_format=None):
         # after 0.8 releases, the *args and **kwargs should be changed:
         # def clone(self, to_bzrdir, revision_id=None):
@@ -466,7 +475,7 @@ class Branch(object):
             kwargs.get('revision', None) or
             kwargs.get('basis_branch', None) or
             (len(args) and isinstance(args[0], basestring))):
-            # backwards compatability api:
+            # backwards compatibility api:
             warn("Branch.clone() has been deprecated for BzrDir.clone() from"
                  " bzrlib 0.8.", DeprecationWarning, stacklevel=3)
             # get basis_branch
@@ -534,9 +543,84 @@ class Branch(object):
                 rev = self.repository.get_revision(revision_id)
                 new_history = rev.get_history(self.repository)[1:]
         destination.set_revision_history(new_history)
-        parent = self.get_parent()
-        if parent:
-            destination.set_parent(parent)
+        try:
+            parent = self.get_parent()
+        except errors.InaccessibleParent, e:
+            mutter('parent was not accessible to copy: %s', e)
+        else:
+            if parent:
+                destination.set_parent(parent)
+
+    @needs_read_lock
+    def check(self):
+        """Check consistency of the branch.
+
+        In particular this checks that revisions given in the revision-history
+        do actually match up in the revision graph, and that they're all 
+        present in the repository.
+        
+        Callers will typically also want to check the repository.
+
+        :return: A BranchCheckResult.
+        """
+        mainline_parent_id = None
+        for revision_id in self.revision_history():
+            try:
+                revision = self.repository.get_revision(revision_id)
+            except errors.NoSuchRevision, e:
+                raise errors.BzrCheckError("mainline revision {%s} not in repository"
+                            % revision_id)
+            # In general the first entry on the revision history has no parents.
+            # But it's not illegal for it to have parents listed; this can happen
+            # in imports from Arch when the parents weren't reachable.
+            if mainline_parent_id is not None:
+                if mainline_parent_id not in revision.parent_ids:
+                    raise errors.BzrCheckError("previous revision {%s} not listed among "
+                                        "parents of {%s}"
+                                        % (mainline_parent_id, revision_id))
+            mainline_parent_id = revision_id
+        return BranchCheckResult(self)
+
+    def _get_checkout_format(self):
+        """Return the most suitable metadir for a checkout of this branch.
+        Weaves are used if this branch's repostory uses weaves.
+        """
+        if isinstance(self.bzrdir, bzrdir.BzrDirPreSplitOut):
+            from bzrlib import repository
+            format = bzrdir.BzrDirMetaFormat1()
+            format.repository_format = repository.RepositoryFormat7()
+        else:
+            format = self.repository.bzrdir.cloning_metadir()
+        return format
+
+    def create_checkout(self, to_location, revision_id=None, 
+                        lightweight=False):
+        """Create a checkout of a branch.
+        
+        :param to_location: The url to produce the checkout at
+        :param revision_id: The revision to check out
+        :param lightweight: If True, produce a lightweight checkout, otherwise,
+        produce a bound branch (heavyweight checkout)
+        :return: The tree of the created checkout
+        """
+        t = transport.get_transport(to_location)
+        try:
+            t.mkdir('.')
+        except errors.FileExists:
+            pass
+        if lightweight:
+            checkout = bzrdir.BzrDirMetaFormat1().initialize_on_transport(t)
+            BranchReferenceFormat().initialize(checkout, self)
+        else:
+            format = self._get_checkout_format()
+            checkout_branch = bzrdir.BzrDir.create_branch_convenience(
+                to_location, force_new_tree=False, format=format)
+            checkout = checkout_branch.bzrdir
+            checkout_branch.bind(self)
+            # pull up to the specified revision_id to set the initial 
+            # branch tip correctly, and seed it with history.
+            checkout_branch.pull(self, stop_revision=revision_id)
+        return checkout.create_workingtree(revision_id)
 
 
 class BranchFormat(object):
@@ -573,7 +657,7 @@ class BranchFormat(object):
         except NoSuchFile:
             raise NotBranchError(path=transport.base)
         except KeyError:
-            raise errors.UnknownFormatError(format_string)
+            raise errors.UnknownFormatError(format=format_string)
 
     @classmethod
     def get_default_format(klass):
@@ -590,7 +674,7 @@ class BranchFormat(object):
 
     def initialize(self, a_bzrdir):
         """Create a branch of this format in a_bzrdir."""
-        raise NotImplementedError(self.initialized)
+        raise NotImplementedError(self.initialize)
 
     def is_supported(self):
         """Is this format supported?
@@ -706,7 +790,7 @@ class BzrBranchFormat5(BranchFormat):
         utf8_files = [('revision-history', ''),
                       ('branch-name', ''),
                       ]
-        control_files = LockableFiles(branch_transport, 'lock', LockDir)
+        control_files = LockableFiles(branch_transport, 'lock', lockdir.LockDir)
         control_files.create_lock()
         control_files.lock_write()
         control_files.put_utf8('format', self.get_format_string())
@@ -731,7 +815,7 @@ class BzrBranchFormat5(BranchFormat):
             format = BranchFormat.find_format(a_bzrdir)
             assert format.__class__ == self.__class__
         transport = a_bzrdir.get_branch_transport(None)
-        control_files = LockableFiles(transport, 'lock', LockDir)
+        control_files = LockableFiles(transport, 'lock', lockdir.LockDir)
         return BzrBranch5(_format=self,
                           _control_files=control_files,
                           a_bzrdir=a_bzrdir,
@@ -768,9 +852,9 @@ class BranchReferenceFormat(BranchFormat):
             raise errors.UninitializableFormat(self)
         mutter('creating branch reference in %s', a_bzrdir.transport.base)
         branch_transport = a_bzrdir.get_branch_transport(self)
-        # FIXME rbc 20060209 one j-a-ms encoding branch lands this str() cast is not needed.
-        branch_transport.put('location', StringIO(str(target_branch.bzrdir.root_transport.base)))
-        branch_transport.put('format', StringIO(self.get_format_string()))
+        branch_transport.put_bytes('location',
+            target_branch.bzrdir.root_transport.base)
+        branch_transport.put_bytes('format', self.get_format_string())
         return self.open(a_bzrdir, _found=True)
 
     def __init__(self):
@@ -852,7 +936,7 @@ class BzrBranch(Branch):
         self._base = self._transport.base
         self._format = _format
         if _control_files is None:
-            raise BzrBadParameterMissing('_control_files')
+            raise ValueError('BzrBranch _control_files is None')
         self.control_files = _control_files
         if deprecated_passed(init):
             warn("BzrBranch.__init__(..., init=XXX): The init parameter is "
@@ -873,11 +957,7 @@ class BzrBranch(Branch):
                  stacklevel=2)
             if (not relax_version_check
                 and not self._format.is_supported()):
-                raise errors.UnsupportedFormatError(
-                        'sorry, branch format %r not supported' % fmt,
-                        ['use a different bzr version',
-                         'or remove the .bzr directory'
-                         ' and "bzr init" again'])
+                raise errors.UnsupportedFormatError(format=fmt)
         if deprecated_passed(transport):
             warn("BzrBranch.__init__(transport=XXX...): The transport "
                  "parameter is deprecated as of bzr 0.8. "
@@ -891,21 +971,6 @@ class BzrBranch(Branch):
 
     __repr__ = __str__
 
-    def __del__(self):
-        # TODO: It might be best to do this somewhere else,
-        # but it is nice for a Branch object to automatically
-        # cache it's information.
-        # Alternatively, we could have the Transport objects cache requests
-        # See the earlier discussion about how major objects (like Branch)
-        # should never expect their __del__ function to run.
-        # XXX: cache_root seems to be unused, 2006-01-13 mbp
-        if hasattr(self, 'cache_root') and self.cache_root is not None:
-            try:
-                rmtree(self.cache_root)
-            except:
-                pass
-            self.cache_root = None
-
     def _get_base(self):
         return self._base
 
@@ -950,7 +1015,7 @@ class BzrBranch(Branch):
         FIXME: DELETE THIS METHOD when pre 0.8 support is removed.
         """
         if format is None:
-            format = BzrBranchFormat.find_format(self.bzrdir)
+            format = BranchFormat.find_format(self.bzrdir)
         self._format = format
         mutter("got branch format %s", self._format)
 
@@ -964,21 +1029,27 @@ class BzrBranch(Branch):
         return self.control_files.is_locked()
 
     def lock_write(self):
-        # TODO: test for failed two phase locks. This is known broken.
-        self.control_files.lock_write()
         self.repository.lock_write()
+        try:
+            self.control_files.lock_write()
+        except:
+            self.repository.unlock()
+            raise
 
     def lock_read(self):
-        # TODO: test for failed two phase locks. This is known broken.
-        self.control_files.lock_read()
         self.repository.lock_read()
+        try:
+            self.control_files.lock_read()
+        except:
+            self.repository.unlock()
+            raise
 
     def unlock(self):
         # TODO: test for failed two phase locks. This is known broken.
         try:
-            self.repository.unlock()
-        finally:
             self.control_files.unlock()
+        finally:
+            self.repository.unlock()
         
     def peek_lock_mode(self):
         if self.control_files._lock_count == 0:
@@ -1022,42 +1093,52 @@ class BzrBranch(Branch):
             # not really an object yet, and the transaction is for objects.
             # transaction.register_clean(history)
 
-    def get_revision_delta(self, revno):
-        """Return the delta for one revision.
-
-        The delta is relative to its mainline predecessor, or the
-        empty tree for revision 1.
-        """
-        assert isinstance(revno, int)
-        rh = self.revision_history()
-        if not (1 <= revno <= len(rh)):
-            raise InvalidRevisionNumber(revno)
-
-        # revno is 1-based; list is 0-based
-
-        new_tree = self.repository.revision_tree(rh[revno-1])
-        if revno == 1:
-            old_tree = EmptyTree()
-        else:
-            old_tree = self.repository.revision_tree(rh[revno-2])
-        return compare_trees(old_tree, new_tree)
-
     @needs_read_lock
     def revision_history(self):
         """See Branch.revision_history."""
         transaction = self.get_transaction()
         history = transaction.map.find_revision_history()
         if history is not None:
-            mutter("cache hit for revision-history in %s", self)
+            # mutter("cache hit for revision-history in %s", self)
             return list(history)
-        history = [l.rstrip('\r\n') for l in
-                self.control_files.get_utf8('revision-history').readlines()]
+        decode_utf8 = cache_utf8.decode
+        history = [decode_utf8(l.rstrip('\r\n')) for l in
+                self.control_files.get('revision-history').readlines()]
         transaction.map.add_revision_history(history)
         # this call is disabled because revision_history is 
         # not really an object yet, and the transaction is for objects.
         # transaction.register_clean(history, precious=True)
         return list(history)
 
+    @needs_write_lock
+    def generate_revision_history(self, revision_id, last_rev=None, 
+        other_branch=None):
+        """Create a new revision history that will finish with revision_id.
+        
+        :param revision_id: the new tip to use.
+        :param last_rev: The previous last_revision. If not None, then this
+            must be a ancestory of revision_id, or DivergedBranches is raised.
+        :param other_branch: The other branch that DivergedBranches should
+            raise with respect to.
+        """
+        # stop_revision must be a descendant of last_revision
+        stop_graph = self.repository.get_revision_graph(revision_id)
+        if last_rev is not None and last_rev not in stop_graph:
+            # our previous tip is not merged into stop_revision
+            raise errors.DivergedBranches(self, other_branch)
+        # make a new revision history from the graph
+        current_rev_id = revision_id
+        new_history = []
+        while current_rev_id not in (None, revision.NULL_REVISION):
+            new_history.append(current_rev_id)
+            current_rev_id_parents = stop_graph[current_rev_id]
+            try:
+                current_rev_id = current_rev_id_parents[0]
+            except IndexError:
+                current_rev_id = None
+        new_history.reverse()
+        self.set_revision_history(new_history)
+
     @needs_write_lock
     def update_revisions(self, other, stop_revision=None):
         """See Branch.update_revisions."""
@@ -1078,45 +1159,11 @@ class BzrBranch(Branch):
             if stop_revision in my_ancestry:
                 # last_revision is a descendant of stop_revision
                 return
-            # stop_revision must be a descendant of last_revision
-            stop_graph = self.repository.get_revision_graph(stop_revision)
-            if last_rev is not None and last_rev not in stop_graph:
-                # our previous tip is not merged into stop_revision
-                raise errors.DivergedBranches(self, other)
-            # make a new revision history from the graph
-            current_rev_id = stop_revision
-            new_history = []
-            while current_rev_id not in (None, NULL_REVISION):
-                new_history.append(current_rev_id)
-                current_rev_id_parents = stop_graph[current_rev_id]
-                try:
-                    current_rev_id = current_rev_id_parents[0]
-                except IndexError:
-                    current_rev_id = None
-            new_history.reverse()
-            self.set_revision_history(new_history)
+            self.generate_revision_history(stop_revision, last_rev=last_rev,
+                other_branch=other)
         finally:
             other.unlock()
 
-    @deprecated_method(zero_eight)
-    def pullable_revisions(self, other, stop_revision):
-        """Please use bzrlib.missing instead."""
-        other_revno = other.revision_id_to_revno(stop_revision)
-        try:
-            return self.missing_revisions(other, other_revno)
-        except DivergedBranches, e:
-            try:
-                pullable_revs = get_intervening_revisions(self.last_revision(),
-                                                          stop_revision, 
-                                                          self.repository)
-                assert self.last_revision() not in pullable_revs
-                return pullable_revs
-            except bzrlib.errors.NotAncestor:
-                if is_ancestor(self.last_revision(), stop_revision, self):
-                    return []
-                else:
-                    raise e
-        
     def basis_tree(self):
         """See Branch.basis_tree."""
         return self.repository.revision_tree(self.last_revision())
@@ -1124,7 +1171,7 @@ class BzrBranch(Branch):
     @deprecated_method(zero_eight)
     def working_tree(self):
         """Create a Working tree object for this branch."""
-        from bzrlib.workingtree import WorkingTree
+
         from bzrlib.transport.local import LocalTransport
         if (self.base.find('://') != -1 or 
             not isinstance(self._transport, LocalTransport)):
@@ -1138,7 +1185,7 @@ class BzrBranch(Branch):
         try:
             old_count = len(self.revision_history())
             try:
-                self.update_revisions(source,stop_revision)
+                self.update_revisions(source, stop_revision)
             except DivergedBranches:
                 if not overwrite:
                     raise
@@ -1151,25 +1198,33 @@ class BzrBranch(Branch):
 
     def get_parent(self):
         """See Branch.get_parent."""
-        import errno
+
         _locs = ['parent', 'pull', 'x-pull']
+        assert self.base[-1] == '/'
         for l in _locs:
             try:
-                return self.control_files.get_utf8(l).read().strip('\n')
+                parent = self.control_files.get(l).read().strip('\n')
             except NoSuchFile:
-                pass
+                continue
+            # This is an old-format absolute path to a local branch
+            # turn it into a url
+            if parent.startswith('/'):
+                parent = urlutils.local_path_to_url(parent.decode('utf8'))
+            try:
+                return urlutils.join(self.base[:-1], parent)
+            except errors.InvalidURLJoin, e:
+                raise errors.InaccessibleParent(parent, self.base)
         return None
 
     def get_push_location(self):
         """See Branch.get_push_location."""
-        config = bzrlib.config.BranchConfig(self)
-        push_loc = config.get_user_option('push_location')
+        push_loc = self.get_config().get_user_option('push_location')
         return push_loc
 
     def set_push_location(self, location):
         """See Branch.set_push_location."""
-        config = bzrlib.config.LocationConfig(self.base)
-        config.set_user_option('push_location', location)
+        self.get_config().set_user_option('push_location', location, 
+                                          local=True)
 
     @needs_write_lock
     def set_parent(self, url):
@@ -1182,9 +1237,21 @@ class BzrBranch(Branch):
         if url is None:
             self.control_files._transport.delete('parent')
         else:
-            self.control_files.put_utf8('parent', url + '\n')
+            if isinstance(url, unicode):
+                try: 
+                    url = url.encode('ascii')
+                except UnicodeEncodeError:
+                    raise bzrlib.errors.InvalidURL(url,
+                        "Urls must be 7-bit ascii, "
+                        "use bzrlib.urlutils.escape")
+                    
+            url = urlutils.relative_url(self.base, url)
+            self.control_files.put('parent', StringIO(url + '\n'))
 
+    @deprecated_function(zero_nine)
     def tree_config(self):
+        """DEPRECATED; call get_config instead.  
+        TreeConfig has become part of BranchConfig."""
         return TreeConfig(self)
 
 
@@ -1230,7 +1297,7 @@ class BzrBranch5(BzrBranch):
 
         This could memoise the branch, but if thats done
         it must be revalidated on each new lock.
-        So for now we just dont memoise it.
+        So for now we just don't memoise it.
         # RBC 20060304 review this decision.
         """
         bound_loc = self.get_bound_location()
@@ -1259,8 +1326,13 @@ class BzrBranch5(BzrBranch):
 
     @needs_write_lock
     def bind(self, other):
-        """Bind the local branch the other branch.
+        """Bind this branch to the branch other.
 
+        This does not push or pull data between the branches, though it does
+        check for divergence to raise an error when the branches are not
+        either the same, or one a prefix of the other. That behaviour may not
+        be useful, so that check may be removed in future.
+        
         :param other: The branch to bind to
         :type other: Branch
         """
@@ -1271,35 +1343,27 @@ class BzrBranch5(BzrBranch):
         #       but binding itself may not be.
         #       Since we *have* to check at commit time, we don't
         #       *need* to check here
-        self.pull(other)
-
-        # we are now equal to or a suffix of other.
-
-        # Since we have 'pulled' from the remote location,
-        # now we should try to pull in the opposite direction
-        # in case the local tree has more revisions than the
-        # remote one.
-        # There may be a different check you could do here
-        # rather than actually trying to install revisions remotely.
-        # TODO: capture an exception which indicates the remote branch
-        #       is not writeable. 
-        #       If it is up-to-date, this probably should not be a failure
-        
-        # lock other for write so the revision-history syncing cannot race
-        other.lock_write()
-        try:
-            other.pull(self)
-            # if this does not error, other now has the same last rev we do
-            # it can only error if the pull from other was concurrent with
-            # a commit to other from someone else.
-
-            # until we ditch revision-history, we need to sync them up:
-            self.set_revision_history(other.revision_history())
-            # now other and self are up to date with each other and have the
-            # same revision-history.
-        finally:
-            other.unlock()
 
+        # we want to raise diverged if:
+        # last_rev is not in the other_last_rev history, AND
+        # other_last_rev is not in our history, and do it without pulling
+        # history around
+        last_rev = self.last_revision()
+        if last_rev is not None:
+            other.lock_read()
+            try:
+                other_last_rev = other.last_revision()
+                if other_last_rev is not None:
+                    # neither branch is new, we have to do some work to
+                    # ascertain diversion.
+                    remote_graph = other.repository.get_revision_graph(
+                        other_last_rev)
+                    local_graph = self.repository.get_revision_graph(last_rev)
+                    if (last_rev not in remote_graph and
+                        other_last_rev not in local_graph):
+                        raise errors.DivergedBranches(self, other)
+            finally:
+                other.unlock()
         self.set_bound_location(other.base)
 
     @needs_write_lock
@@ -1354,17 +1418,30 @@ class BranchTestProviderAdapter(object):
         return result
 
 
+class BranchCheckResult(object):
+    """Results of checking branch consistency.
+
+    :see: Branch.check
+    """
+
+    def __init__(self, branch):
+        self.branch = branch
+
+    def report_results(self, verbose):
+        """Report the check results via trace.note.
+        
+        :param verbose: Requests more detailed display of what was checked,
+            if any.
+        """
+        note('checked branch %s format %s',
+             self.branch.base,
+             self.branch._format)
+
+
 ######################################################################
 # predicates
 
 
-@deprecated_function(zero_eight)
-def ScratchBranch(*args, **kwargs):
-    """See bzrlib.bzrdir.ScratchDir."""
-    d = ScratchDir(*args, **kwargs)
-    return d.open_branch()
-
-
 @deprecated_function(zero_eight)
 def is_control_file(*args, **kwargs):
     """See bzrlib.workingtree.is_control_file."""
