@@ -1,15 +1,15 @@
 # Copyright (C) 2005 Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -36,7 +36,9 @@ def dir_exporter(tree, dest, root):
     os.mkdir(dest)
     mutter('export version %r', tree)
     inv = tree.inventory
-    for dp, ie in inv.iter_entries():
+    entries = inv.iter_entries()
+    entries.next() # skip root
+    for dp, ie in entries:
         # .bzrignore has no meaning outside of a working tree
         # so do not export it
         if dp == ".bzrignore":
