@@ -1,15 +1,15 @@
-# Copyright (C) 2004, 2005 by Canonical Ltd
-
+# Copyright (C) 2004, 2005, 2006 by Canonical Ltd
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -20,13 +20,11 @@
 # executable files with reasonable names.
 
 # TODO: `help commands --all` should show hidden commands
+import textwrap
 
 global_help = \
-"""Bazaar-NG -- a free distributed version-control tool
-http://bazaar-ng.org/
-
-WARNING: This is an unstable development version.
-         Please keep backups.
+"""Bazaar -- a free distributed version-control tool
+http://bazaar-vcs.org/
 
 Basic commands:
 
@@ -55,9 +53,9 @@ import sys
 
 
 def help(topic=None, outfile = None):
-    if outfile == None:
+    if outfile is None:
         outfile = sys.stdout
-    if topic == None:
+    if topic is None:
         outfile.write(global_help)
     elif topic == 'commands':
         help_commands(outfile = outfile)
@@ -87,18 +85,30 @@ def command_usage(cmd_object):
     return s
 
 
+def print_command_plugin(cmd_object, outfile, format):
+    """Print the plugin that provides a command object, if any.
+
+    If the cmd_object is provided by a plugin, prints the plugin name to
+    outfile using the provided format string.
+    """
+    plugin_name = cmd_object.plugin_name()
+    if plugin_name is not None:
+        out_str = '(From plugin "%s")' % plugin_name
+        outfile.write(format % out_str)
+
+
 def help_on_command(cmdname, outfile=None):
     from bzrlib.commands import get_cmd_object
 
     cmdname = str(cmdname)
 
-    if outfile == None:
+    if outfile is None:
         outfile = sys.stdout
 
     cmd_object = get_cmd_object(cmdname)
 
     doc = cmd_object.help()
-    if doc == None:
+    if doc is None:
         raise NotImplementedError("sorry, no detailed help yet for %r" % cmdname)
 
     print >>outfile, 'usage:', command_usage(cmd_object) 
@@ -109,30 +119,21 @@ def help_on_command(cmdname, outfile=None):
 
     print >>outfile
 
+    print_command_plugin(cmd_object, outfile, '%s\n\n')
+
     outfile.write(doc)
     if doc[-1] != '\n':
         outfile.write('\n')
-    
-    help_on_options(cmd_object.takes_options, outfile=None)
+    help_on_command_options(cmd_object, outfile)
 
 
-def help_on_options(options, outfile=None):
-    from bzrlib.commands import SHORT_OPTIONS
-    
-    if not options:
-        return
-    
-    if outfile == None:
+def help_on_command_options(cmd, outfile=None):
+    from bzrlib.option import Option, get_optparser
+    if outfile is None:
         outfile = sys.stdout
-
-    outfile.write('\noptions:\n')
-    for on in options:
-        l = '    --' + on
-        for shortname, longname in SHORT_OPTIONS.items():
-            if longname == on:
-                l += ', -' + shortname
-                break
-        outfile.write(l + '\n')
+    options = cmd.options()
+    outfile.write('\n')
+    outfile.write(get_optparser(options).format_option_help())
 
 
 def help_commands(outfile=None):
@@ -141,7 +142,7 @@ def help_commands(outfile=None):
                                  plugin_command_names,
                                  get_cmd_object)
 
-    if outfile == None:
+    if outfile is None:
         outfile = sys.stdout
 
     names = set()                       # to eliminate duplicates
@@ -155,8 +156,12 @@ def help_commands(outfile=None):
         if cmd_object.hidden:
             continue
         print >>outfile, command_usage(cmd_object)
+
+        plugin_name = cmd_object.plugin_name()
+        print_command_plugin(cmd_object, outfile, '        %s\n')
+
         cmd_help = cmd_object.help()
         if cmd_help:
             firstline = cmd_help.split('\n', 1)[0]
-            print >>outfile, '    ' + firstline
+            print >>outfile, '        ' + firstline
         
