@@ -1,17 +1,15 @@
-# Copyright (C) 2004, 2005 by Martin Pool
-# Copyright (C) 2005 by Canonical Ltd
-
-
+# Copyright (C) 2005, 2006 by Canonical Ltd
+# 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+# 
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+# 
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -26,13 +24,14 @@ from bzrlib.errors import (NoWorkingTree, NotBranchError,
                            NoRepositoryPresent, NotLocalUrl)
 from bzrlib.missing import find_unmerged
 import bzrlib.osutils as osutils
-from bzrlib.symbol_versioning import *
+from bzrlib.symbol_versioning import (deprecated_function, 
+        zero_eight)
 
 
 def plural(n, base='', pl=None):
     if n == 1:
         return base
-    elif pl != None:
+    elif pl is not None:
         return pl
     else:
         return 's'
@@ -63,15 +62,15 @@ def _show_location_info(repository, branch=None, working=None):
         branch_path = branch.bzrdir.root_transport.base
         if working_path != branch_path:
             # lightweight checkout
-            print '  light checkout root: %s' % working_path
+            print ' light checkout root: %s' % working_path
             if repository.is_shared():
                 # lightweight checkout of branch in shared repository
-                print '    shared repository: %s' % repository_path
-                print '    repository branch: %s' % (
+                print '   shared repository: %s' % repository_path
+                print '   repository branch: %s' % (
                     _repo_relpath(repository_path, branch_path))
             else:
                 # lightweight checkout of standalone branch
-                print '   checkout of branch: %s' % branch_path
+                print '  checkout of branch: %s' % branch_path
         elif repository.is_shared():
             # branch with tree inside shared repository
             print '    shared repository: %s' % repository_path
@@ -172,9 +171,12 @@ def _show_missing_revisions_working(working):
     branch = working.branch
     basis = working.basis_tree()
     work_inv = working.inventory
-    delta = diff.compare_trees(basis, working, want_unchanged=True)
+    delta = working.changes_from(basis, want_unchanged=True)
     history = branch.revision_history()
-    tree_last_id = working.last_revision()
+    try:
+        tree_last_id = working.get_parent_ids()[0]
+    except IndexError:
+        tree_last_id = None
 
     if len(history) and tree_last_id != history[-1]:
         tree_last_revno = branch.revision_id_to_revno(tree_last_id)
@@ -188,7 +190,7 @@ def _show_working_stats(working):
     """Show statistics about a working tree."""
     basis = working.basis_tree()
     work_inv = working.inventory
-    delta = diff.compare_trees(basis, working, want_unchanged=True)
+    delta = working.changes_from(basis, want_unchanged=True)
 
     print
     print 'In the working tree:'
@@ -208,9 +210,9 @@ def _show_working_stats(working):
     print '  %8d ignored' % ignore_cnt
 
     dir_cnt = 0
-    for file_id in work_inv:
-        if work_inv.get_file_kind(file_id) == 'directory':
-            dir_cnt += 1
+    entries = work_inv.iter_entries()
+    entries.next()
+    dir_cnt = sum(1 for path, ie in entries if ie.kind == 'directory')
     print '  %8d versioned %s' \
           % (dir_cnt,
              plural(dir_cnt, 'subdirectory', 'subdirectories'))
