@@ -28,13 +28,16 @@
 ROOT_ID = "TREE_ROOT"
 
 
+import collections
 import os.path
 import re
 import sys
 import tarfile
 import types
+from warnings import warn
 
 import bzrlib
+from bzrlib import errors, osutils
 from bzrlib.osutils import (pumpfile, quotefn, splitpath, joinpath,
                             pathjoin, sha_strings)
 from bzrlib.errors import (NotVersionedError, InvalidEntryName,
@@ -76,35 +79,37 @@ class InventoryEntry(object):
     >>> i.path2id('')
     'TREE_ROOT'
     >>> i.add(InventoryDirectory('123', 'src', ROOT_ID))
-    InventoryDirectory('123', 'src', parent_id='TREE_ROOT')
+    InventoryDirectory('123', 'src', parent_id='TREE_ROOT', revision=None)
     >>> i.add(InventoryFile('2323', 'hello.c', parent_id='123'))
-    InventoryFile('2323', 'hello.c', parent_id='123')
-    >>> shouldbe = {0: 'src', 1: pathjoin('src','hello.c')}
+    InventoryFile('2323', 'hello.c', parent_id='123', sha1=None, len=None)
+    >>> shouldbe = {0: '', 1: 'src', 2: pathjoin('src','hello.c')}
     >>> for ix, j in enumerate(i.iter_entries()):
     ...   print (j[0] == shouldbe[ix], j[1])
     ... 
-    (True, InventoryDirectory('123', 'src', parent_id='TREE_ROOT'))
-    (True, InventoryFile('2323', 'hello.c', parent_id='123'))
+    (True, InventoryDirectory('TREE_ROOT', '', parent_id=None, revision=None))
+    (True, InventoryDirectory('123', 'src', parent_id='TREE_ROOT', revision=None))
+    (True, InventoryFile('2323', 'hello.c', parent_id='123', sha1=None, len=None))
     >>> i.add(InventoryFile('2323', 'bye.c', '123'))
     Traceback (most recent call last):
     ...
     BzrError: inventory already contains entry with id {2323}
     >>> i.add(InventoryFile('2324', 'bye.c', '123'))
-    InventoryFile('2324', 'bye.c', parent_id='123')
+    InventoryFile('2324', 'bye.c', parent_id='123', sha1=None, len=None)
     >>> i.add(InventoryDirectory('2325', 'wibble', '123'))
-    InventoryDirectory('2325', 'wibble', parent_id='123')
+    InventoryDirectory('2325', 'wibble', parent_id='123', revision=None)
     >>> i.path2id('src/wibble')
     '2325'
     >>> '2325' in i
     True
     >>> i.add(InventoryFile('2326', 'wibble.c', '2325'))
-    InventoryFile('2326', 'wibble.c', parent_id='2325')
+    InventoryFile('2326', 'wibble.c', parent_id='2325', sha1=None, len=None)
     >>> i['2326']
-    InventoryFile('2326', 'wibble.c', parent_id='2325')
+    InventoryFile('2326', 'wibble.c', parent_id='2325', sha1=None, len=None)
     >>> for path, entry in i.iter_entries():
     ...     print path
     ...     assert i.path2id(path)
     ... 
+    <BLANKLINE>
     src
     src/bye.c
     src/hello.c
@@ -122,15 +127,7 @@ class InventoryEntry(object):
     RENAMED = 'renamed'
     MODIFIED_AND_RENAMED = 'modified and renamed'
     
-    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', 'executable', 
-                 'revision']
-
-    def _add_text_to_weave(self, new_lines, parents, weave_store, transaction):
-        versionedfile = weave_store.get_weave_or_empty(self.file_id,
-                                                       transaction)
-        versionedfile.add_lines(self.revision, parents, new_lines)
-        versionedfile.clear_cache()
+    __slots__ = []
 
     def detect_changes(self, old_entry):
         """Return a (text_modified, meta_modified) from this to old_entry.
@@ -165,7 +162,7 @@ class InventoryEntry(object):
                             versioned_file_store,
                             transaction,
                             entry_vf=None):
-        """Return the revisions and entries that directly preceed this.
+        """Return the revisions and entries that directly precede this.
 
         Returned as a map from revision to inventory entry.
 
@@ -249,7 +246,7 @@ class InventoryEntry(object):
 
     def get_tar_item(self, root, dp, now, tree):
         """Get a tarfile item and a file stream for its content."""
-        item = tarfile.TarInfo(pathjoin(root, dp))
+        item = tarfile.TarInfo(pathjoin(root, dp).encode('utf8'))
         # TODO: would be cool to actually set it to the timestamp of the
         # revision it was last changed
         item.mtime = now
@@ -299,7 +296,7 @@ class InventoryEntry(object):
         """Return a short kind indicator useful for appending to names."""
         raise BzrError('unknown kind %r' % self.kind)
 
-    known_kinds = ('file', 'directory', 'symlink', 'root_directory')
+    known_kinds = ('file', 'directory', 'symlink')
 
     def _put_in_tar(self, item, tree):
         """populate item for stashing in a tar, and return the content stream.
@@ -316,21 +313,19 @@ class InventoryEntry(object):
         """
         fullpath = pathjoin(dest, dp)
         self._put_on_disk(fullpath, tree)
-        mutter("  export {%s} kind %s to %s", self.file_id,
-                self.kind, fullpath)
+        # mutter("  export {%s} kind %s to %s", self.file_id,
+        #         self.kind, fullpath)
 
     def _put_on_disk(self, fullpath, tree):
         """Put this entry onto disk at fullpath, from tree tree."""
         raise BzrError("don't know how to export {%s} of kind %r" % (self.file_id, self.kind))
 
     def sorted_children(self):
-        l = self.children.items()
-        l.sort()
-        return l
+        return sorted(self.children.items())
 
     @staticmethod
     def versionable_kind(kind):
-        return kind in ('file', 'directory', 'symlink')
+        return (kind in ('file', 'directory', 'symlink'))
 
     def check(self, checker, rev_id, inv, tree):
         """Check this inventory entry is intact.
@@ -346,7 +341,7 @@ class InventoryEntry(object):
         :param inv: Inventory from which the entry was loaded.
         :param tree: RevisionTree for this entry.
         """
-        if self.parent_id != None:
+        if self.parent_id is not None:
             if not inv.has_id(self.parent_id):
                 raise BzrCheckError('missing parent {%s} in inventory for revision {%s}'
                         % (self.parent_id, rev_id))
@@ -402,33 +397,36 @@ class InventoryEntry(object):
         return 'unchanged'
 
     def __repr__(self):
-        return ("%s(%r, %r, parent_id=%r)"
+        return ("%s(%r, %r, parent_id=%r, revision=%r)"
                 % (self.__class__.__name__,
                    self.file_id,
                    self.name,
-                   self.parent_id))
+                   self.parent_id,
+                   self.revision))
 
     def snapshot(self, revision, path, previous_entries,
-                 work_tree, weave_store, transaction):
+                 work_tree, commit_builder):
         """Make a snapshot of this entry which may or may not have changed.
         
         This means that all its fields are populated, that it has its
         text stored in the text store or weave.
         """
-        mutter('new parents of %s are %r', path, previous_entries)
+        # mutter('new parents of %s are %r', path, previous_entries)
         self._read_tree_state(path, work_tree)
+        # TODO: Where should we determine whether to reuse a
+        # previous revision id or create a new revision? 20060606
         if len(previous_entries) == 1:
             # cannot be unchanged unless there is only one parent file rev.
             parent_ie = previous_entries.values()[0]
             if self._unchanged(parent_ie):
-                mutter("found unchanged entry")
+                # mutter("found unchanged entry")
                 self.revision = parent_ie.revision
                 return "unchanged"
         return self._snapshot_into_revision(revision, previous_entries, 
-                                            work_tree, weave_store, transaction)
+                                            work_tree, commit_builder)
 
     def _snapshot_into_revision(self, revision, previous_entries, work_tree,
-                                weave_store, transaction):
+                                commit_builder):
         """Record this revision unconditionally into a store.
 
         The entry's last-changed revision property (`revision`) is updated to 
@@ -438,19 +436,16 @@ class InventoryEntry(object):
 
         :returns: String description of the commit (e.g. "merged", "modified"), etc.
         """
-        mutter('new revision {%s} for {%s}', revision, self.file_id)
+        # mutter('new revision {%s} for {%s}', revision, self.file_id)
         self.revision = revision
-        self._snapshot_text(previous_entries, work_tree, weave_store,
-                            transaction)
+        self._snapshot_text(previous_entries, work_tree, commit_builder)
 
-    def _snapshot_text(self, file_parents, work_tree, weave_store, transaction): 
+    def _snapshot_text(self, file_parents, work_tree, commit_builder): 
         """Record the 'text' of this entry, whatever form that takes.
         
         This default implementation simply adds an empty text.
         """
-        mutter('storing file {%s} in revision {%s}',
-               self.file_id, self.revision)
-        self._add_text_to_weave([], file_parents.keys(), weave_store, transaction)
+        raise NotImplementedError(self._snapshot_text)
 
     def __eq__(self, other):
         if not isinstance(other, InventoryEntry):
@@ -477,7 +472,7 @@ class InventoryEntry(object):
     def _unchanged(self, previous_ie):
         """Has this entry changed relative to previous_ie.
 
-        This method should be overriden in child classes.
+        This method should be overridden in child classes.
         """
         compatible = True
         # different inv parent
@@ -505,15 +500,23 @@ class InventoryEntry(object):
 
 class RootEntry(InventoryEntry):
 
+    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
+                 'text_id', 'parent_id', 'children', 'executable', 
+                 'revision', 'symlink_target']
+
     def _check(self, checker, rev_id, tree):
         """See InventoryEntry._check"""
 
     def __init__(self, file_id):
         self.file_id = file_id
         self.children = {}
-        self.kind = 'root_directory'
+        self.kind = 'directory'
         self.parent_id = None
         self.name = u''
+        self.revision = None
+        warn('RootEntry is deprecated as of bzr 0.10.  Please use '
+             'InventoryDirectory instead.',
+            DeprecationWarning, stacklevel=2)
 
     def __eq__(self, other):
         if not isinstance(other, RootEntry):
@@ -526,9 +529,13 @@ class RootEntry(InventoryEntry):
 class InventoryDirectory(InventoryEntry):
     """A directory in an inventory."""
 
+    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
+                 'text_id', 'parent_id', 'children', 'executable', 
+                 'revision', 'symlink_target']
+
     def _check(self, checker, rev_id, tree):
         """See InventoryEntry._check"""
-        if self.text_sha1 != None or self.text_size != None or self.text_id != None:
+        if self.text_sha1 is not None or self.text_size is not None or self.text_id is not None:
             raise BzrCheckError('directory {%s} has text in revision {%s}'
                                 % (self.file_id, rev_id))
 
@@ -561,10 +568,18 @@ class InventoryDirectory(InventoryEntry):
         """See InventoryEntry._put_on_disk."""
         os.mkdir(fullpath)
 
+    def _snapshot_text(self, file_parents, work_tree, commit_builder):
+        """See InventoryEntry._snapshot_text."""
+        commit_builder.modified_directory(self.file_id, file_parents)
+
 
 class InventoryFile(InventoryEntry):
     """A file in an inventory."""
 
+    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
+                 'text_id', 'parent_id', 'children', 'executable', 
+                 'revision', 'symlink_target']
+
     def _check(self, checker, tree_revision_id, tree):
         """See InventoryEntry._check"""
         t = (self.file_id, self.revision)
@@ -609,8 +624,8 @@ class InventoryFile(InventoryEntry):
 
     def detect_changes(self, old_entry):
         """See InventoryEntry.detect_changes."""
-        assert self.text_sha1 != None
-        assert old_entry.text_sha1 != None
+        assert self.text_sha1 is not None
+        assert old_entry.text_sha1 is not None
         text_modified = (self.text_sha1 != old_entry.text_sha1)
         meta_modified = (self.executable != old_entry.executable)
         return text_modified, meta_modified
@@ -668,32 +683,29 @@ class InventoryFile(InventoryEntry):
 
     def _read_tree_state(self, path, work_tree):
         """See InventoryEntry._read_tree_state."""
-        self.text_sha1 = work_tree.get_file_sha1(self.file_id)
-        self.executable = work_tree.is_executable(self.file_id)
+        self.text_sha1 = work_tree.get_file_sha1(self.file_id, path=path)
+        # FIXME: 20050930 probe for the text size when getting sha1
+        # in _read_tree_state
+        self.executable = work_tree.is_executable(self.file_id, path=path)
+
+    def __repr__(self):
+        return ("%s(%r, %r, parent_id=%r, sha1=%r, len=%s)"
+                % (self.__class__.__name__,
+                   self.file_id,
+                   self.name,
+                   self.parent_id,
+                   self.text_sha1,
+                   self.text_size))
 
     def _forget_tree_state(self):
         self.text_sha1 = None
-        self.executable = None
 
-    def _snapshot_text(self, file_parents, work_tree, versionedfile_store, transaction):
+    def _snapshot_text(self, file_parents, work_tree, commit_builder):
         """See InventoryEntry._snapshot_text."""
-        mutter('storing text of file {%s} in revision {%s} into %r',
-               self.file_id, self.revision, versionedfile_store)
-        # special case to avoid diffing on renames or 
-        # reparenting
-        if (len(file_parents) == 1
-            and self.text_sha1 == file_parents.values()[0].text_sha1
-            and self.text_size == file_parents.values()[0].text_size):
-            previous_ie = file_parents.values()[0]
-            versionedfile = versionedfile_store.get_weave(self.file_id, transaction)
-            versionedfile.clone_text(self.revision, previous_ie.revision, file_parents.keys())
-        else:
-            new_lines = work_tree.get_file(self.file_id).readlines()
-            self._add_text_to_weave(new_lines, file_parents.keys(), versionedfile_store,
-                                    transaction)
-            self.text_sha1 = sha_strings(new_lines)
-            self.text_size = sum(map(len, new_lines))
-
+        def get_content_byte_lines():
+            return work_tree.get_file(self.file_id).readlines()
+        self.text_sha1, self.text_size = commit_builder.modified_file_text(
+            self.file_id, file_parents, get_content_byte_lines, self.text_sha1, self.text_size)
 
     def _unchanged(self, previous_ie):
         """See InventoryEntry._unchanged."""
@@ -712,14 +724,16 @@ class InventoryFile(InventoryEntry):
 class InventoryLink(InventoryEntry):
     """A file in an inventory."""
 
-    __slots__ = ['symlink_target']
+    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
+                 'text_id', 'parent_id', 'children', 'executable', 
+                 'revision', 'symlink_target']
 
     def _check(self, checker, rev_id, tree):
         """See InventoryEntry._check"""
-        if self.text_sha1 != None or self.text_size != None or self.text_id != None:
+        if self.text_sha1 is not None or self.text_size is not None or self.text_id is not None:
             raise BzrCheckError('symlink {%s} has text in revision {%s}'
                     % (self.file_id, rev_id))
-        if self.symlink_target == None:
+        if self.symlink_target is None:
             raise BzrCheckError('symlink {%s} has no target in revision {%s}'
                     % (self.file_id, rev_id))
 
@@ -793,6 +807,11 @@ class InventoryLink(InventoryEntry):
             compatible = False
         return compatible
 
+    def _snapshot_text(self, file_parents, work_tree, commit_builder):
+        """See InventoryEntry._snapshot_text."""
+        commit_builder.modified_link(
+            self.file_id, file_parents, self.symlink_target)
+
 
 class Inventory(object):
     """Inventory of versioned files in a tree.
@@ -813,7 +832,7 @@ class Inventory(object):
 
     >>> inv = Inventory()
     >>> inv.add(InventoryFile('123-123', 'hello.c', ROOT_ID))
-    InventoryFile('123-123', 'hello.c', parent_id='TREE_ROOT')
+    InventoryFile('123-123', 'hello.c', parent_id='TREE_ROOT', sha1=None, len=None)
     >>> inv['123-123'].name
     'hello.c'
 
@@ -827,10 +846,10 @@ class Inventory(object):
     May also look up by name:
 
     >>> [x[0] for x in inv.iter_entries()]
-    ['hello.c']
+    ['', u'hello.c']
     >>> inv = Inventory('TREE_ROOT-12345678-12345678')
     >>> inv.add(InventoryFile('123-123', 'hello.c', ROOT_ID))
-    InventoryFile('123-123', 'hello.c', parent_id='TREE_ROOT-12345678-12345678')
+    InventoryFile('123-123', 'hello.c', parent_id='TREE_ROOT-12345678-12345678', sha1=None, len=None)
     """
     def __init__(self, root_id=ROOT_ID, revision_id=None):
         """Create or read an inventory.
@@ -846,48 +865,112 @@ class Inventory(object):
         # root id. Rather than generating a random one here.
         #if root_id is None:
         #    root_id = bzrlib.branch.gen_file_id('TREE_ROOT')
-        self.root = RootEntry(root_id)
+        if root_id is not None:
+            self._set_root(InventoryDirectory(root_id, '', None))
+        else:
+            self.root = None
+            self._byid = {}
+        # FIXME: this isn't ever used, changing it to self.revision may break
+        # things. TODO make everything use self.revision_id
         self.revision_id = revision_id
-        self._byid = {self.root.file_id: self.root}
 
+    def _set_root(self, ie):
+        self.root = ie
+        self._byid = {self.root.file_id: self.root}
 
     def copy(self):
         # TODO: jam 20051218 Should copy also copy the revision_id?
-        other = Inventory(self.root.file_id)
+        entries = self.iter_entries()
+        other = Inventory(entries.next()[1].file_id)
         # copy recursively so we know directories will be added before
         # their children.  There are more efficient ways than this...
-        for path, entry in self.iter_entries():
-            if entry == self.root:
-                continue
+        for path, entry in entries():
             other.add(entry.copy())
         return other
 
-
     def __iter__(self):
         return iter(self._byid)
 
-
     def __len__(self):
         """Returns number of entries."""
         return len(self._byid)
 
-
     def iter_entries(self, from_dir=None):
         """Return (path, entry) pairs, in order by name."""
-        if from_dir == None:
+        if from_dir is None:
             assert self.root
             from_dir = self.root
+            yield '', self.root
         elif isinstance(from_dir, basestring):
             from_dir = self._byid[from_dir]
             
-        kids = from_dir.children.items()
-        kids.sort()
-        for name, ie in kids:
-            yield name, ie
-            if ie.kind == 'directory':
-                for cn, cie in self.iter_entries(from_dir=ie.file_id):
-                    yield pathjoin(name, cn), cie
+        # unrolling the recursive called changed the time from
+        # 440ms/663ms (inline/total) to 116ms/116ms
+        children = from_dir.children.items()
+        children.sort()
+        children = collections.deque(children)
+        stack = [(u'', children)]
+        while stack:
+            from_dir_relpath, children = stack[-1]
 
+            while children:
+                name, ie = children.popleft()
+
+                # we know that from_dir_relpath never ends in a slash
+                # and 'f' doesn't begin with one, we can do a string op, rather
+                # than the checks of pathjoin(), though this means that all paths
+                # start with a slash
+                path = from_dir_relpath + '/' + name
+
+                yield path[1:], ie
+
+                if ie.kind != 'directory':
+                    continue
+
+                # But do this child first
+                new_children = ie.children.items()
+                new_children.sort()
+                new_children = collections.deque(new_children)
+                stack.append((path, new_children))
+                # Break out of inner loop, so that we start outer loop with child
+                break
+            else:
+                # if we finished all children, pop it off the stack
+                stack.pop()
+
+    def iter_entries_by_dir(self, from_dir=None):
+        """Iterate over the entries in a directory first order.
+
+        This returns all entries for a directory before returning
+        the entries for children of a directory. This is not
+        lexicographically sorted order, and is a hybrid between
+        depth-first and breadth-first.
+
+        :return: This yields (path, entry) pairs
+        """
+        # TODO? Perhaps this should return the from_dir so that the root is
+        # yielded? or maybe an option?
+        if from_dir is None:
+            assert self.root
+            from_dir = self.root
+            yield '', self.root
+        elif isinstance(from_dir, basestring):
+            from_dir = self._byid[from_dir]
+            
+        stack = [(u'', from_dir)]
+        while stack:
+            cur_relpath, cur_dir = stack.pop()
+
+            child_dirs = []
+            for child_name, child_ie in sorted(cur_dir.children.iteritems()):
+
+                child_relpath = cur_relpath + child_name
+
+                yield child_relpath, child_ie
+
+                if child_ie.kind == 'directory':
+                    child_dirs.append((child_relpath+'/', child_ie))
+            stack.extend(reversed(child_dirs))
 
     def entries(self):
         """Return list of (path, ie) for all entries except the root.
@@ -907,7 +990,6 @@ class Inventory(object):
         descend(self.root, u'')
         return accum
 
-
     def directories(self):
         """Return (path, entry) pairs for all directories, including the root.
         """
@@ -924,47 +1006,42 @@ class Inventory(object):
         descend(self.root, u'')
         return accum
         
-
-
     def __contains__(self, file_id):
         """True if this entry contains a file with given id.
 
         >>> inv = Inventory()
         >>> inv.add(InventoryFile('123', 'foo.c', ROOT_ID))
-        InventoryFile('123', 'foo.c', parent_id='TREE_ROOT')
+        InventoryFile('123', 'foo.c', parent_id='TREE_ROOT', sha1=None, len=None)
         >>> '123' in inv
         True
         >>> '456' in inv
         False
         """
-        return file_id in self._byid
-
+        return (file_id in self._byid)
 
     def __getitem__(self, file_id):
         """Return the entry for given file_id.
 
         >>> inv = Inventory()
         >>> inv.add(InventoryFile('123123', 'hello.c', ROOT_ID))
-        InventoryFile('123123', 'hello.c', parent_id='TREE_ROOT')
+        InventoryFile('123123', 'hello.c', parent_id='TREE_ROOT', sha1=None, len=None)
         >>> inv['123123'].name
         'hello.c'
         """
         try:
             return self._byid[file_id]
         except KeyError:
-            if file_id == None:
+            if file_id is None:
                 raise BzrError("can't look up file_id None")
             else:
                 raise BzrError("file_id {%s} not in inventory" % file_id)
 
-
     def get_file_kind(self, file_id):
         return self._byid[file_id].kind
 
     def get_child(self, parent_id, filename):
         return self[parent_id].children.get(filename)
 
-
     def add(self, entry):
         """Add entry to inventory.
 
@@ -976,7 +1053,12 @@ class Inventory(object):
         if entry.file_id in self._byid:
             raise BzrError("inventory already contains entry with id {%s}" % entry.file_id)
 
-        if entry.parent_id == ROOT_ID or entry.parent_id is None:
+        if entry.parent_id is None:
+            assert self.root is None and len(self._byid) == 0
+            self._set_root(entry)
+            return entry
+        if entry.parent_id == ROOT_ID:
+            assert self.root is not None, self
             entry.parent_id = self.root.file_id
 
         try:
@@ -984,7 +1066,7 @@ class Inventory(object):
         except KeyError:
             raise BzrError("parent_id {%s} not in inventory" % entry.parent_id)
 
-        if parent.children.has_key(entry.name):
+        if entry.name in parent.children:
             raise BzrError("%s is already versioned" %
                     pathjoin(self.id2path(parent.file_id), entry.name))
 
@@ -992,46 +1074,35 @@ class Inventory(object):
         parent.children[entry.name] = entry
         return entry
 
-
-    def add_path(self, relpath, kind, file_id=None):
+    def add_path(self, relpath, kind, file_id=None, parent_id=None):
         """Add entry from a path.
 
         The immediate parent must already be versioned.
 
         Returns the new entry object."""
-        from bzrlib.workingtree import gen_file_id
         
-        parts = bzrlib.osutils.splitpath(relpath)
-
-        if file_id == None:
-            file_id = gen_file_id(relpath)
+        parts = osutils.splitpath(relpath)
 
         if len(parts) == 0:
-            self.root = RootEntry(file_id)
+            if file_id is None:
+                file_id = bzrlib.workingtree.gen_root_id()
+            self.root = InventoryDirectory(file_id, '', None)
             self._byid = {self.root.file_id: self.root}
             return
         else:
             parent_path = parts[:-1]
             parent_id = self.path2id(parent_path)
-            if parent_id == None:
+            if parent_id is None:
                 raise NotVersionedError(path=parent_path)
-        if kind == 'directory':
-            ie = InventoryDirectory(file_id, parts[-1], parent_id)
-        elif kind == 'file':
-            ie = InventoryFile(file_id, parts[-1], parent_id)
-        elif kind == 'symlink':
-            ie = InventoryLink(file_id, parts[-1], parent_id)
-        else:
-            raise BzrError("unknown kind %r" % kind)
+        ie = make_entry(kind, parts[-1], parent_id, file_id)
         return self.add(ie)
 
-
     def __delitem__(self, file_id):
         """Remove entry by id.
 
         >>> inv = Inventory()
         >>> inv.add(InventoryFile('123', 'foo.c', ROOT_ID))
-        InventoryFile('123', 'foo.c', parent_id='TREE_ROOT')
+        InventoryFile('123', 'foo.c', parent_id='TREE_ROOT', sha1=None, len=None)
         >>> '123' in inv
         True
         >>> del inv['123']
@@ -1047,7 +1118,6 @@ class Inventory(object):
         if ie.parent_id is not None:
             del self[ie.parent_id].children[ie.name]
 
-
     def __eq__(self, other):
         """Compare two sets by comparing their contents.
 
@@ -1056,34 +1126,28 @@ class Inventory(object):
         >>> i1 == i2
         True
         >>> i1.add(InventoryFile('123', 'foo', ROOT_ID))
-        InventoryFile('123', 'foo', parent_id='TREE_ROOT')
+        InventoryFile('123', 'foo', parent_id='TREE_ROOT', sha1=None, len=None)
         >>> i1 == i2
         False
         >>> i2.add(InventoryFile('123', 'foo', ROOT_ID))
-        InventoryFile('123', 'foo', parent_id='TREE_ROOT')
+        InventoryFile('123', 'foo', parent_id='TREE_ROOT', sha1=None, len=None)
         >>> i1 == i2
         True
         """
         if not isinstance(other, Inventory):
             return NotImplemented
 
-        if len(self._byid) != len(other._byid):
-            # shortcut: obviously not the same
-            return False
-
         return self._byid == other._byid
 
-
     def __ne__(self, other):
         return not self.__eq__(other)
 
-
     def __hash__(self):
         raise ValueError('not hashable')
 
     def _iter_file_id_parents(self, file_id):
         """Yield the parents of file_id up to the root."""
-        while file_id != None:
+        while file_id is not None:
             try:
                 ie = self._byid[file_id]
             except KeyError:
@@ -1127,12 +1191,12 @@ class Inventory(object):
         This returns the entry of the last component in the path,
         which may be either a file or a directory.
 
-        Returns None iff the path is not found.
+        Returns None IFF the path is not found.
         """
         if isinstance(name, types.StringTypes):
             name = splitpath(name)
 
-        mutter("lookup path %r" % name)
+        # mutter("lookup path %r" % name)
 
         parent = self.root
         for f in name:
@@ -1147,14 +1211,29 @@ class Inventory(object):
 
         return parent.file_id
 
-
     def has_filename(self, names):
         return bool(self.path2id(names))
 
-
     def has_id(self, file_id):
-        return self._byid.has_key(file_id)
+        return (file_id in self._byid)
 
+    def remove_recursive_id(self, file_id):
+        """Remove file_id, and children, from the inventory.
+        
+        :param file_id: A file_id to remove.
+        """
+        to_find_delete = [self._byid[file_id]]
+        to_delete = []
+        while to_find_delete:
+            ie = to_find_delete.pop()
+            to_delete.append(ie.file_id)
+            if ie.kind == 'directory':
+                to_find_delete.extend(ie.children.values())
+        for file_id in reversed(to_delete):
+            ie = self[file_id]
+            del self._byid[file_id]
+            if ie.parent_id is not None:
+                del self[ie.parent_id].children[ie.name]
 
     def rename(self, file_id, new_parent_id, new_name):
         """Move a file within the inventory.
@@ -1186,13 +1265,41 @@ class Inventory(object):
         file_ie.parent_id = new_parent_id
 
 
+def make_entry(kind, name, parent_id, file_id=None):
+    """Create an inventory entry.
+
+    :param kind: the type of inventory entry to create.
+    :param name: the basename of the entry.
+    :param parent_id: the parent_id of the entry.
+    :param file_id: the file_id to use. if None, one will be created.
+    """
+    if file_id is None:
+        file_id = bzrlib.workingtree.gen_file_id(name)
+
+    norm_name, can_access = osutils.normalized_filename(name)
+    if norm_name != name:
+        if can_access:
+            name = norm_name
+        else:
+            # TODO: jam 20060701 This would probably be more useful
+            #       if the error was raised with the full path
+            raise errors.InvalidNormalization(name)
+
+    if kind == 'directory':
+        return InventoryDirectory(file_id, name, parent_id)
+    elif kind == 'file':
+        return InventoryFile(file_id, name, parent_id)
+    elif kind == 'symlink':
+        return InventoryLink(file_id, name, parent_id)
+    else:
+        raise BzrError("unknown kind %r" % kind)
 
 
 _NAME_RE = None
 
 def is_valid_name(name):
     global _NAME_RE
-    if _NAME_RE == None:
+    if _NAME_RE is None:
         _NAME_RE = re.compile(r'^[^/\\]+$')
         
     return bool(_NAME_RE.match(name))
