@@ -1,15 +1,15 @@
 # Copyright (C) 2005, 2006 by Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -17,14 +17,16 @@
 
 """Commit message editor support."""
 
-
+import codecs
 import errno
 import os
 from subprocess import call
 import sys
 
+import bzrlib
 import bzrlib.config as config
 from bzrlib.errors import BzrError
+from bzrlib.trace import warning, mutter
 
 
 def _get_editor():
@@ -39,7 +41,7 @@ def _get_editor():
         yield e
         
     for varname in 'VISUAL', 'EDITOR':
-        if os.environ.has_key(varname):
+        if varname in os.environ:
             yield os.environ[varname]
 
     if sys.platform == 'win32':
@@ -55,6 +57,7 @@ def _run_editor(filename):
     for e in _get_editor():
         edargs = e.split(' ')
         try:
+            ## mutter("trying editor: %r", (edargs +[filename]))
             x = call(edargs + [filename])
         except OSError, e:
            # We're searching for an editor, so catch safe errors and continue
@@ -89,13 +92,15 @@ def edit_commit_message(infotext, ignoreline=DEFAULT_IGNORE_LINE):
     """
     import tempfile
 
+    msgfilename = None
     try:
         tmp_fileno, msgfilename = tempfile.mkstemp(prefix='bzr_log.', dir=u'.')
         msgfile = os.close(tmp_fileno)
         if infotext is not None and infotext != "":
             hasinfo = True
             msgfile = file(msgfilename, "w")
-            msgfile.write("\n%s\n\n%s" % (ignoreline, infotext))
+            msgfile.write("\n\n%s\n\n%s" % (ignoreline,
+                infotext.encode(bzrlib.user_encoding, 'replace')))
             msgfile.close()
         else:
             hasinfo = False
@@ -106,7 +111,7 @@ def edit_commit_message(infotext, ignoreline=DEFAULT_IGNORE_LINE):
         started = False
         msg = []
         lastline, nlines = 0, 0
-        for line in file(msgfilename, "r"):
+        for line in codecs.open(msgfilename, 'r', bzrlib.user_encoding):
             stripped_line = line.strip()
             # strip empty line before the log message starts
             if not started:
@@ -135,8 +140,11 @@ def edit_commit_message(infotext, ignoreline=DEFAULT_IGNORE_LINE):
             return "".join(msg)
     finally:
         # delete the msg file in any case
-        try: os.unlink(msgfilename)
-        except IOError: pass
+        if msgfilename is not None:
+            try:
+                os.unlink(msgfilename)
+            except IOError, e:
+                warning("failed to unlink %s: %s; ignored", msgfilename, e)
 
 
 def make_commit_message_template(working_tree, specific_files):
