@@ -1,15 +1,15 @@
 # Copyright (C) 2004, 2005, 2006 by Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -17,10 +17,10 @@
 # TODO: For things like --diff-prefix, we want a way to customize the display
 # of the option argument.
 
+import optparse
 import re
 
-import bzrlib.commands
-from bzrlib.trace import warning, mutter
+from bzrlib.trace import warning
 from bzrlib.revisionspec import RevisionSpec
 from bzrlib.errors import BzrCommandError
 
@@ -32,25 +32,25 @@ def _parse_revision_str(revstr):
     each revision specifier supplied.
 
     >>> _parse_revision_str('234')
-    [<RevisionSpec_int 234>]
+    [<RevisionSpec_revno 234>]
     >>> _parse_revision_str('234..567')
-    [<RevisionSpec_int 234>, <RevisionSpec_int 567>]
+    [<RevisionSpec_revno 234>, <RevisionSpec_revno 567>]
     >>> _parse_revision_str('..')
     [<RevisionSpec None>, <RevisionSpec None>]
     >>> _parse_revision_str('..234')
-    [<RevisionSpec None>, <RevisionSpec_int 234>]
+    [<RevisionSpec None>, <RevisionSpec_revno 234>]
     >>> _parse_revision_str('234..')
-    [<RevisionSpec_int 234>, <RevisionSpec None>]
+    [<RevisionSpec_revno 234>, <RevisionSpec None>]
     >>> _parse_revision_str('234..456..789') # Maybe this should be an error
-    [<RevisionSpec_int 234>, <RevisionSpec_int 456>, <RevisionSpec_int 789>]
+    [<RevisionSpec_revno 234>, <RevisionSpec_revno 456>, <RevisionSpec_revno 789>]
     >>> _parse_revision_str('234....789') #Error ?
-    [<RevisionSpec_int 234>, <RevisionSpec None>, <RevisionSpec_int 789>]
+    [<RevisionSpec_revno 234>, <RevisionSpec None>, <RevisionSpec_revno 789>]
     >>> _parse_revision_str('revid:test@other.com-234234')
     [<RevisionSpec_revid revid:test@other.com-234234>]
     >>> _parse_revision_str('revid:test@other.com-234234..revid:test@other.com-234235')
     [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_revid revid:test@other.com-234235>]
     >>> _parse_revision_str('revid:test@other.com-234234..23')
-    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_int 23>]
+    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_revno 23>]
     >>> _parse_revision_str('date:2005-04-12')
     [<RevisionSpec_date date:2005-04-12>]
     >>> _parse_revision_str('date:2005-04-12 12:24:33')
@@ -60,40 +60,29 @@ def _parse_revision_str(revstr):
     >>> _parse_revision_str('date:2005-04-12,12:24:33')
     [<RevisionSpec_date date:2005-04-12,12:24:33>]
     >>> _parse_revision_str('-5..23')
-    [<RevisionSpec_int -5>, <RevisionSpec_int 23>]
+    [<RevisionSpec_revno -5>, <RevisionSpec_revno 23>]
     >>> _parse_revision_str('-5')
-    [<RevisionSpec_int -5>]
+    [<RevisionSpec_revno -5>]
     >>> _parse_revision_str('123a')
     Traceback (most recent call last):
       ...
-    BzrError: No namespace registered for string: '123a'
+    NoSuchRevisionSpec: No namespace registered for string: '123a'
     >>> _parse_revision_str('abc')
     Traceback (most recent call last):
       ...
-    BzrError: No namespace registered for string: 'abc'
+    NoSuchRevisionSpec: No namespace registered for string: 'abc'
     >>> _parse_revision_str('branch:../branch2')
     [<RevisionSpec_branch branch:../branch2>]
     >>> _parse_revision_str('branch:../../branch2')
     [<RevisionSpec_branch branch:../../branch2>]
     >>> _parse_revision_str('branch:../../branch2..23')
-    [<RevisionSpec_branch branch:../../branch2>, <RevisionSpec_int 23>]
+    [<RevisionSpec_branch branch:../../branch2>, <RevisionSpec_revno 23>]
     """
     # TODO: Maybe move this into revisionspec.py
-    old_format_re = re.compile('\d*:\d*')
-    m = old_format_re.match(revstr)
     revs = []
-    if m:
-        warning('Colon separator for revision numbers is deprecated.'
-                ' Use .. instead')
-        for rev in revstr.split(':'):
-            if rev:
-                revs.append(RevisionSpec(int(rev)))
-            else:
-                revs.append(RevisionSpec(None))
-    else:
-        sep = re.compile("\\.\\.(?!/)")
-        for x in sep.split(revstr):
-            revs.append(RevisionSpec(x or None))
+    sep = re.compile("\\.\\.(?!/)")
+    for x in sep.split(revstr):
+        revs.append(RevisionSpec.from_string(x or None))
     return revs
 
 
@@ -135,7 +124,7 @@ class Option(object):
         argname -- name of option argument, if any
         """
         # TODO: perhaps a subclass that automatically does 
-        # --option, --no-option for reversable booleans
+        # --option, --no-option for reversible booleans
         self.name = name
         self.help = help
         self.type = type
@@ -154,6 +143,68 @@ class Option(object):
             if option is self:
                 return short
 
+    def get_negation_name(self):
+        if self.name.startswith('no-'):
+            return self.name[3:]
+        else:
+            return 'no-' + self.name
+
+    def add_option(self, parser, short_name):
+        """Add this option to an Optparse parser"""
+        option_strings = ['--%s' % self.name]
+        if short_name is not None:
+            option_strings.append('-%s' % short_name)
+        optargfn = self.type
+        if optargfn is None:
+            parser.add_option(action='store_true', dest=self.name, 
+                              help=self.help,
+                              default=OptionParser.DEFAULT_VALUE,
+                              *option_strings)
+            negation_strings = ['--%s' % self.get_negation_name()]
+            parser.add_option(action='store_false', dest=self.name, 
+                              help=optparse.SUPPRESS_HELP, *negation_strings)
+        else:
+            parser.add_option(action='callback', 
+                              callback=self._optparse_callback, 
+                              type='string', metavar=self.argname.upper(),
+                              help=self.help,
+                              default=OptionParser.DEFAULT_VALUE, 
+                              *option_strings)
+
+    def _optparse_callback(self, option, opt, value, parser):
+        setattr(parser.values, self.name, self.type(value))
+
+    def iter_switches(self):
+        """Iterate through the list of switches provided by the option
+        
+        :return: an iterator of (name, short_name, argname, help)
+        """
+        argname =  self.argname
+        if argname is not None:
+            argname = argname.upper()
+        yield self.name, self.short_name(), argname, self.help
+
+
+class OptionParser(optparse.OptionParser):
+    """OptionParser that raises exceptions instead of exiting"""
+
+    DEFAULT_VALUE = object()
+
+    def error(self, message):
+        raise BzrCommandError(message)
+
+
+def get_optparser(options):
+    """Generate an optparse parser for bzrlib-style options"""
+
+    parser = OptionParser()
+    parser.remove_option('--help')
+    short_options = dict((k.name, v) for v, k in 
+                         Option.SHORT_OPTIONS.iteritems())
+    for option in options.itervalues():
+        option.add_option(parser, short_options.get(option.name))
+    return parser
+
 
 def _global_option(name, **kwargs):
     """Register o as a global option."""
@@ -196,7 +247,8 @@ _global_option('short', help='Use moderately short log format. Same as --log-for
 _global_option('line', help='Use log format with one line per revision. Same as --log-format line')
 _global_option('root', type=str)
 _global_option('no-backup')
-_global_option('merge-type', type=_parse_merge_type)
+_global_option('merge-type', type=_parse_merge_type, 
+               help='Select a particular merge algorithm')
 _global_option('pattern', type=str)
 _global_option('quiet')
 _global_option('remember', help='Remember the specified location as a'
