@@ -16,19 +16,20 @@
 
 import sys
 
-from bzrlib.delta import compare_trees
 from bzrlib.diff import _raise_if_nonexistent
-from bzrlib.errors import NoSuchRevision
+import bzrlib.errors as errors
 from bzrlib.log import line_log
 from bzrlib.osutils import is_inside_any
-from bzrlib.symbol_versioning import *
+from bzrlib.symbol_versioning import (deprecated_function,
+        zero_eight,
+        )
 
 # TODO: when showing single-line logs, truncate to the width of the terminal
 # if known, but only if really going to the terminal (not into a file)
 
 
 @deprecated_function(zero_eight)
-def show_status(branch, show_unchanged=False,
+def show_status(branch, show_unchanged=None,
                 specific_files=None,
                 show_ids=False,
                 to_file=None,
@@ -69,7 +70,7 @@ def show_status(branch, show_unchanged=False,
                      specific_files, show_ids, to_file, show_pending, revision)
 
 
-def show_tree_status(wt, show_unchanged=False,
+def show_tree_status(wt, show_unchanged=None,
                      specific_files=None,
                      show_ids=False,
                      to_file=None,
@@ -87,29 +88,24 @@ def show_tree_status(wt, show_unchanged=False,
     If showing the status of a working tree, extra information is included
     about unknown files, conflicts, and pending merges.
 
-    show_unchanged
-        If set, includes unchanged files.
-
-    specific_files
-        If set, a list of filenames whose status should be shown.
-        It is an error to give a filename that is not in the working 
+    :param show_unchanged: Deprecated parameter. If set, includes unchanged 
+        files.
+    :param specific_files: If set, a list of filenames whose status should be
+        shown.  It is an error to give a filename that is not in the working 
         tree, or in the working inventory or in the basis inventory.
-
-    show_ids
-        If set, includes each file's id.
-
-    to_file
-        If set, write to this file (default stdout.)
-
-    show_pending
-        If set, write pending merges.
-
-    revision
-        If None the compare latest revision with working tree
+    :param show_ids: If set, includes each file's id.
+    :param to_file: If set, write to this file (default stdout.)
+    :param show_pending: If set, write pending merges.
+    :param revision: If None the compare latest revision with working tree
+        If not None it must be a RevisionSpec list.
         If one revision show compared it with working tree.
         If two revisions show status between first and second.
     """
-    if to_file == None:
+    if show_unchanged is not None:
+        warn("show_status_trees with show_unchanged has been deprecated "
+             "since bzrlib 0.9", DeprecationWarning, stacklevel=2)
+
+    if to_file is None:
         to_file = sys.stdout
     
     wt.lock_read()
@@ -122,32 +118,32 @@ def show_tree_status(wt, show_unchanged=False,
             try:
                 rev_id = revision[0].in_history(wt.branch).rev_id
                 old = wt.branch.repository.revision_tree(rev_id)
-            except NoSuchRevision, e:
-                raise BzrCommandError(str(e))
+            except errors.NoSuchRevision, e:
+                raise errors.BzrCommandError(str(e))
             if (len(revision) > 1) and (revision[1].spec is not None):
                 try:
                     rev_id = revision[1].in_history(wt.branch).rev_id
                     new = wt.branch.repository.revision_tree(rev_id)
                     new_is_working_tree = False
-                except NoSuchRevision, e:
-                    raise BzrCommandError(str(e))
+                except errors.NoSuchRevision, e:
+                    raise errors.BzrCommandError(str(e))
             else:
                 new = wt
         _raise_if_nonexistent(specific_files, old, new)
-        delta = compare_trees(old, new, want_unchanged=show_unchanged,
+        delta = new.changes_from(old, want_unchanged=show_unchanged,
                               specific_files=specific_files)
         delta.show(to_file,
                    show_ids=show_ids,
                    show_unchanged=show_unchanged)
 
-        if new_is_working_tree:
-            list_paths('unknown', new.unknowns(), specific_files, to_file)
-            conflict_title = False
-            for conflict in wt.conflicts():
-                if conflict_title is False:
-                    print >> to_file, "conflicts:"
-                    conflict_title = True
-                print >> to_file, "  %s" % conflict
+        list_paths('unknown', new.unknowns(), specific_files, to_file)
+        conflict_title = False
+        # show the new conflicts only for now. XXX: get them from the delta.
+        for conflict in new.conflicts():
+            if conflict_title is False:
+                print >> to_file, "conflicts:"
+                conflict_title = True
+            print >> to_file, "  %s" % conflict
         if new_is_working_tree and show_pending:
             show_pending_merges(new, to_file)
     finally:
@@ -155,33 +151,43 @@ def show_tree_status(wt, show_unchanged=False,
 
 def show_pending_merges(new, to_file):
     """Write out a display of pending merges in a working tree."""
-    pending = new.pending_merges()
-    branch = new.branch
-    if len(pending) == 0:
+    parents = new.get_parent_ids()
+    if len(parents) < 2:
         return
+    pending = parents[1:]
+    branch = new.branch
+    last_revision = parents[0]
     print >>to_file, 'pending merges:'
-    last_revision = branch.last_revision()
     if last_revision is not None:
-        ignore = set(branch.repository.get_ancestry(last_revision))
+        try:
+            ignore = set(branch.repository.get_ancestry(last_revision))
+        except errors.NoSuchRevision:
+            # the last revision is a ghost : assume everything is new 
+            # except for it
+            ignore = set([None, last_revision])
     else:
-        ignore = set()
-    for merge in new.pending_merges():
+        ignore = set([None])
+    # TODO: this could be improved using merge_sorted - we'd get the same 
+    # output rather than one level of indent.
+    for merge in pending:
         ignore.add(merge)
         try:
+            from bzrlib.osutils import terminal_width
+            width = terminal_width()
             m_revision = branch.repository.get_revision(merge)
-            print >> to_file, ' ', line_log(m_revision, 77)
+            print >> to_file, ' ', line_log(m_revision, width - 3)
             inner_merges = branch.repository.get_ancestry(merge)
-            assert inner_merges[0] == None
+            assert inner_merges[0] is None
             inner_merges.pop(0)
             inner_merges.reverse()
             for mmerge in inner_merges:
                 if mmerge in ignore:
                     continue
                 mm_revision = branch.repository.get_revision(mmerge)
-                print >> to_file, '   ', line_log(mm_revision, 75)
+                print >> to_file, '   ', line_log(mm_revision, width - 5)
                 ignore.add(mmerge)
-        except NoSuchRevision:
-            print >> to_file, ' ', merge 
+        except errors.NoSuchRevision:
+            print >> to_file, ' ', merge
         
 def list_paths(header, paths, specific_files, to_file):
     done_header = False
