@@ -1,15 +1,15 @@
 # Copyright (C) 2005 by Canonical Development Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -63,7 +63,7 @@ class TextStore(bzrlib.store.TransportStore):
 
     def _try_put(self, fn, f):
         try:
-            self._transport.put(fn, f, mode=self._file_mode)
+            self._transport.put_file(fn, f, mode=self._file_mode)
         except NoSuchFile:
             if not self._prefixed:
                 raise
@@ -71,7 +71,7 @@ class TextStore(bzrlib.store.TransportStore):
                 self._transport.mkdir(os.path.dirname(fn), mode=self._dir_mode)
             except FileExists:
                 pass
-            self._transport.put(fn, f, mode=self._file_mode)
+            self._transport.put_file(fn, f, mode=self._file_mode)
 
     def _get(self, fn):
         if fn.endswith('.gz'):
@@ -114,13 +114,9 @@ class TextStore(bzrlib.store.TransportStore):
         # gzip.GzipFile.read() requires a tell() function
         # but some transports return objects that cannot seek
         # so buffer them in a StringIO instead
-        if hasattr(f, 'tell'):
+        if getattr(f, 'tell', None) is not None:
             return gzip.GzipFile(mode='rb', fileobj=f)
         else:
             from cStringIO import StringIO
             sio = StringIO(f.read())
             return gzip.GzipFile(mode='rb', fileobj=sio)
-
-
-def ScratchTextStore():
-    return TextStore(ScratchTransport())
