@@ -1,20 +1,19 @@
 # Copyright (C) 2005, 2006 by Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-
 """Tests of status command.
 
 Most of these depend on the particular formatting used.
@@ -23,93 +22,103 @@ tests are not using self.capture. If we add tests for the programmatic
 interface later, they will be non blackbox tests.
 """
 
-
 from cStringIO import StringIO
-from os import mkdir, chdir
-from tempfile import TemporaryFile
 import codecs
+from os import mkdir, chdir
+import sys
+from tempfile import TemporaryFile
 
+from bzrlib import bzrdir, errors
 import bzrlib.branch
 from bzrlib.builtins import merge
-import bzrlib.bzrdir as bzrdir
-import bzrlib.errors as errors
 from bzrlib.osutils import pathjoin
 from bzrlib.revisionspec import RevisionSpec
 from bzrlib.status import show_tree_status
-from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests import TestCaseWithTransport, TestSkipped
 from bzrlib.workingtree import WorkingTree
 
 
 class BranchStatus(TestCaseWithTransport):
     
-    def test_branch_status(self): 
+    def assertStatus(self, output_lines, working_tree,
+        revision=None):
+        """Run status in working_tree and look for output.
+        
+        :param output_lines: The lines to look for.
+        :param working_tree: The tree to run status in.
+        """
+        output_string = self.status_string(working_tree, revision)
+        self.assertEqual(output_lines, output_string.splitlines(True))
+    
+    def status_string(self, wt, revision=None):
+        # use a real file rather than StringIO because it doesn't handle
+        # Unicode very well.
+        tof = codecs.getwriter('utf-8')(TemporaryFile())
+        show_tree_status(wt, to_file=tof, revision=revision)
+        tof.seek(0)
+        return tof.read().decode('utf-8')
+
+    def test_branch_status(self):
         """Test basic branch status"""
         wt = self.make_branch_and_tree('.')
-        b = wt.branch
 
-        # status with nothing
-        tof = StringIO()
-        show_tree_status(wt, to_file=tof)
-        self.assertEquals(tof.getvalue(), "")
+        # status with no commits or files - it must
+        # work and show no output. We do this with no
+        # commits to be sure that it's not going to fail
+        # as a corner case.
+        self.assertStatus([], wt)
 
-        tof = StringIO()
         self.build_tree(['hello.c', 'bye.c'])
-        wt.add_pending_merge('pending@pending-0-0')
-        show_tree_status(wt, to_file=tof)
-        tof.seek(0)
-        self.assertEquals(tof.readlines(),
-                          ['unknown:\n',
-                           '  bye.c\n',
-                           '  hello.c\n',
-                           'pending merges:\n',
-                           '  pending@pending-0-0\n'
-                           ])
+        self.assertStatus([
+                'unknown:\n',
+                '  bye.c\n',
+                '  hello.c\n',
+            ],
+            wt)
+
+        # add a commit to allow showing pending merges.
+        wt.commit('create a parent to allow testing merge output')
+
+        wt.add_parent_tree_id('pending@pending-0-0')
+        self.assertStatus([
+                'unknown:\n',
+                '  bye.c\n',
+                '  hello.c\n',
+                'pending merges:\n',
+                '  pending@pending-0-0\n',
+            ],
+            wt)
 
     def test_branch_status_revisions(self):
         """Tests branch status with revisions"""
         wt = self.make_branch_and_tree('.')
-        b = wt.branch
 
-        tof = StringIO()
         self.build_tree(['hello.c', 'bye.c'])
         wt.add('hello.c')
         wt.add('bye.c')
         wt.commit('Test message')
 
-        tof = StringIO()
-        revs =[]
-        revs.append(RevisionSpec(0))
-        
-        show_tree_status(wt, to_file=tof, revision=revs)
-        
-        tof.seek(0)
-        self.assertEquals(tof.readlines(),
-                          ['added:\n',
-                           '  bye.c\n',
-                           '  hello.c\n'])
+        revs = [RevisionSpec.from_string('0')]
+        self.assertStatus([
+                'added:\n',
+                '  bye.c\n',
+                '  hello.c\n'
+            ],
+            wt,
+            revision=revs)
 
         self.build_tree(['more.c'])
         wt.add('more.c')
         wt.commit('Another test message')
         
-        tof = StringIO()
-        revs.append(RevisionSpec(1))
-        
-        show_tree_status(wt, to_file=tof, revision=revs)
-        
-        tof.seek(0)
-        self.assertEquals(tof.readlines(),
-                          ['added:\n',
-                           '  bye.c\n',
-                           '  hello.c\n'])
-
-    def status_string(self, wt):
-        # use a real file rather than StringIO because it doesn't handle
-        # Unicode very well.
-        tof = codecs.getwriter('utf-8')(TemporaryFile())
-        show_tree_status(wt, to_file=tof)
-        tof.seek(0)
-        return tof.read().decode('utf-8')
+        revs.append(RevisionSpec.from_string('1'))
+        self.assertStatus([
+                'added:\n',
+                '  bye.c\n',
+                '  hello.c\n',
+            ],
+            wt,
+            revision=revs)
 
     def test_pending(self):
         """Pending merges display works, including Unicode"""
@@ -219,3 +228,49 @@ class TestStatus(TestCaseWithTransport):
         self.assertEquals(result2, result)
 
 
+class TestStatusEncodings(TestCaseWithTransport):
+    
+    def setUp(self):
+        TestCaseWithTransport.setUp(self)
+        self.user_encoding = bzrlib.user_encoding
+        self.stdout = sys.stdout
+
+    def tearDown(self):
+        bzrlib.user_encoding = self.user_encoding
+        sys.stdout = self.stdout
+        TestCaseWithTransport.tearDown(self)
+
+    def make_uncommitted_tree(self):
+        """Build a branch with uncommitted unicode named changes in the cwd."""
+        working_tree = self.make_branch_and_tree(u'.')
+        filename = u'hell\u00d8'
+        try:
+            self.build_tree_contents([(filename, 'contents of hello')])
+        except UnicodeEncodeError:
+            raise TestSkipped("can't build unicode working tree in "
+                "filesystem encoding %s" % sys.getfilesystemencoding())
+        working_tree.add(filename)
+        return working_tree
+
+    def test_stdout_ascii(self):
+        sys.stdout = StringIO()
+        bzrlib.user_encoding = 'ascii'
+        working_tree = self.make_uncommitted_tree()
+        stdout, stderr = self.run_bzr("status")
+
+        self.assertEquals(stdout, """\
+added:
+  hell?
+""")
+
+    def test_stdout_latin1(self):
+        sys.stdout = StringIO()
+        bzrlib.user_encoding = 'latin-1'
+        working_tree = self.make_uncommitted_tree()
+        stdout, stderr = self.run_bzr('status')
+
+        self.assertEquals(stdout, u"""\
+added:
+  hell\u00d8
+""".encode('latin-1'))
+
