@@ -1,16 +1,16 @@
 # Copyright (C) 2005 by Canonical Ltd
 # -*- coding: utf-8 -*-
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -36,46 +36,11 @@ from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
 from bzrlib.lockable_files import LockableFiles
 from bzrlib.tests import TestCaseWithTransport, TestSkipped
+from bzrlib.tests.test_permissions import chmod_r, check_mode_r
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
 from bzrlib.transport import get_transport
 from bzrlib.workingtree import WorkingTree
 
-# TODO RBC consolidate the helper methods here and in tests/test_permissions.py
-
-def chmod_r(base, file_mode, dir_mode):
-    """Recursively chmod from a base directory"""
-    assert os.path.isdir(base)
-    os.chmod(base, dir_mode)
-    for root, dirs, files in os.walk(base):
-        for d in dirs:
-            p = os.path.join(root, d)
-            os.chmod(p, dir_mode)
-        for f in files:
-            p = os.path.join(root, f)
-            os.chmod(p, file_mode)
-
-
-def check_mode_r(test, base, file_mode, dir_mode, include_base=True):
-    """Check that all permissions match
-
-    :param test: The TestCase being run
-    :param base: The path to the root directory to check
-    :param file_mode: The mode for all files
-    :param dir_mode: The mode for all directories
-    :param include_base: If false, only check the subdirectories
-    """
-    assert os.path.isdir(base)
-    t = get_transport(".")
-    if include_base:
-        test.assertTransportMode(t, base, dir_mode)
-    for root, dirs, files in os.walk(base):
-        for d in dirs:
-            p = os.path.join(root, d)
-            test.assertTransportMode(t, p, dir_mode)
-        for f in files:
-            p = os.path.join(root, f)
-            test.assertTransportMode(t, p, file_mode)
-
 
 class TestPermissions(TestCaseWithTransport):
 
@@ -95,18 +60,18 @@ class TestPermissions(TestCaseWithTransport):
         b = self.make_branch('b')
         self.assertEqualMode(02777, b.control_files._dir_mode)
         self.assertEqualMode(00666, b.control_files._file_mode)
-        check_mode_r(self, b.control_files._transport.base, 00666, 02777)
+        check_mode_r(self, 'b/.bzr', 00666, 02777)
 
         os.mkdir('c')
         os.chmod('c', 02750)
         b = self.make_branch('c')
         self.assertEqualMode(02750, b.control_files._dir_mode)
         self.assertEqualMode(00640, b.control_files._file_mode)
-        check_mode_r(self, b.control_files._transport.base, 00640, 02750)
+        check_mode_r(self, 'c/.bzr', 00640, 02750)
 
         os.mkdir('d')
         os.chmod('d', 0700)
         b = self.make_branch('d')
         self.assertEqualMode(0700, b.control_files._dir_mode)
         self.assertEqualMode(0600, b.control_files._file_mode)
-        check_mode_r(self, b.control_files._transport.base, 00600, 00700)
+        check_mode_r(self, 'd/.bzr', 00600, 00700)
