@@ -1,15 +1,15 @@
-# (C) 2005, 2006 Canonical Ltd
-
+# Copyright (C) 2005, 2006 Canonical Ltd
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -21,17 +21,16 @@ import re
 import sys
 
 import bzrlib
-import bzrlib.bzrdir as bzrdir
+from bzrlib import bzrdir, errors, repository
 from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
-from bzrlib.commit import commit
-import bzrlib.errors as errors
+from bzrlib.delta import TreeDelta
 from bzrlib.errors import (FileExists,
                            NoSuchRevision,
                            NoSuchFile,
                            UninitializableFormat,
                            NotBranchError,
                            )
-import bzrlib.repository as repository
+from bzrlib.inventory import Inventory
 from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
 from bzrlib.tests.bzrdir_implementations.test_bzrdir import TestCaseWithBzrDir
@@ -149,9 +148,9 @@ class TestRepository(TestCaseWithRepository):
         tree = wt.branch.repository.revision_tree('revision-1')
         self.assertEqual(list(tree.list_files()), [])
         tree = wt.branch.repository.revision_tree(None)
-        self.assertEqual(len(tree.list_files()), 0)
+        self.assertEqual([], list(tree.list_files()))
         tree = wt.branch.repository.revision_tree(NULL_REVISION)
-        self.assertEqual(len(tree.list_files()), 0)
+        self.assertEqual([], list(tree.list_files()))
 
     def test_fetch(self):
         # smoke test fetch to ensure that the convenience function works.
@@ -163,10 +162,53 @@ class TestRepository(TestCaseWithRepository):
         tree_a.commit('rev1', rev_id='rev1')
         # fetch with a default limit (grab everything)
         repo = bzrdir.BzrDir.create_repository(self.get_url('b'))
+        if (tree_a.branch.repository._format.rich_root_data and not
+            repo._format.rich_root_data):
+            raise TestSkipped('Cannot fetch from model2 to model1')
         repo.fetch(tree_a.branch.repository,
                    revision_id=None,
                    pb=bzrlib.progress.DummyProgress())
 
+    def test_fetch_knit2(self):
+        tree_a = self.make_branch_and_tree('a')
+        self.build_tree(['a/foo'])
+        tree_a.add('foo', 'file1')
+        tree_a.commit('rev1', rev_id='rev1')
+        # fetch with a default limit (grab everything)
+        f = bzrdir.BzrDirMetaFormat1()
+        f._repository_format = repository.RepositoryFormatKnit2()
+        os.mkdir('b')
+        b_bzrdir = f.initialize(self.get_url('b'))
+        repo = b_bzrdir.create_repository()
+        repo.fetch(tree_a.branch.repository,
+                   revision_id=None,
+                   pb=bzrlib.progress.DummyProgress())
+        rev1_tree = repo.revision_tree('rev1')
+        lines = rev1_tree.get_file_lines(rev1_tree.inventory.root.file_id)
+        self.assertEqual([], lines)
+        b_branch = b_bzrdir.create_branch()
+        b_branch.pull(tree_a.branch)
+        tree_b = b_bzrdir.create_workingtree()
+        tree_b.commit('no change', rev_id='rev2')
+        rev2_tree = repo.revision_tree('rev2')
+        self.assertEqual('rev1', rev2_tree.inventory.root.revision)
+
+    def test_get_revision_delta(self):
+        tree_a = self.make_branch_and_tree('a')
+        self.build_tree(['a/foo'])
+        tree_a.add('foo', 'file1')
+        tree_a.commit('rev1', rev_id='rev1')
+        self.build_tree(['a/vla'])
+        tree_a.add('vla', 'file2')
+        tree_a.commit('rev2', rev_id='rev2')
+
+        delta = tree_a.branch.repository.get_revision_delta('rev1')
+        self.assertIsInstance(delta, TreeDelta)
+        self.assertEqual([('foo', 'file1', 'file')], delta.added)
+        delta = tree_a.branch.repository.get_revision_delta('rev2')
+        self.assertIsInstance(delta, TreeDelta)
+        self.assertEqual([('vla', 'file2', 'file')], delta.added)
+
     def test_clone_bzrdir_repository_revision(self):
         # make a repository with some revisions,
         # and clone it, this should not have unreferenced revisions.
@@ -225,6 +267,8 @@ class TestRepository(TestCaseWithRepository):
         except errors.UpToDateFormat:
             # this is in the most current format already.
             return
+        except errors.BadConversionTarget, e:
+            raise TestSkipped(str(e))
         wt = WorkingTree.open(wt.basedir)
         new_signature = wt.branch.repository.get_signature_text('A')
         self.assertEqual(old_signature, new_signature)
@@ -270,6 +314,92 @@ class TestRepository(TestCaseWithRepository):
         self.assertMessageRoundtrips(
             "All 8-bit chars: " +  ''.join([unichr(x) for x in range(256)]))
 
+    def test_check_repository(self):
+        """Check a fairly simple repository's history"""
+        tree = self.make_branch_and_tree('.')
+        tree.commit('initial empty commit', rev_id='a-rev',
+                    allow_pointless=True)
+        result = tree.branch.repository.check(['a-rev'])
+        # writes to log; should accept both verbose or non-verbose
+        result.report_results(verbose=True)
+        result.report_results(verbose=False)
+
+    def test_get_revisions(self):
+        tree = self.make_branch_and_tree('.')
+        tree.commit('initial empty commit', rev_id='a-rev',
+                    allow_pointless=True)
+        tree.commit('second empty commit', rev_id='b-rev',
+                    allow_pointless=True)
+        tree.commit('third empty commit', rev_id='c-rev',
+                    allow_pointless=True)
+        repo = tree.branch.repository
+        revision_ids = ['a-rev', 'b-rev', 'c-rev']
+        revisions = repo.get_revisions(revision_ids)
+        assert len(revisions) == 3, repr(revisions)
+        zipped = zip(revisions, revision_ids)
+        self.assertEqual(len(zipped), 3)
+        for revision, revision_id in zipped:
+            self.assertEqual(revision.revision_id, revision_id)
+            self.assertEqual(revision, repo.get_revision(revision_id))
+
+    def test_root_entry_has_revision(self):
+        tree = self.make_branch_and_tree('.')
+        tree.commit('message', rev_id='rev_id')
+        self.assertEqual('rev_id', tree.basis_tree().inventory.root.revision)
+        rev_tree = tree.branch.repository.revision_tree(tree.get_parent_ids()[0])
+        self.assertEqual('rev_id', rev_tree.inventory.root.revision)
+
+    def test_create_basis_inventory(self):
+        # Needs testing here because differences between repo and working tree
+        # basis inventory formats can lead to bugs.
+        t = self.make_branch_and_tree('.')
+        b = t.branch
+        open('a', 'wb').write('a\n')
+        t.add('a')
+        t.commit('a', rev_id='r1')
+
+        t._control_files.get_utf8('basis-inventory-cache')
+
+        basis_inv = t.basis_tree().inventory
+        self.assertEquals('r1', basis_inv.revision_id)
+        
+        store_inv = b.repository.get_inventory('r1')
+        self.assertEquals(store_inv._byid, basis_inv._byid)
+
+        open('b', 'wb').write('b\n')
+        t.add('b')
+        t.commit('b', rev_id='r2')
+
+        t._control_files.get_utf8('basis-inventory-cache')
+
+        basis_inv_txt = t.read_basis_inventory()
+        basis_inv = bzrlib.xml6.serializer_v6.read_inventory_from_string(basis_inv_txt)
+        self.assertEquals('r2', basis_inv.revision_id)
+        store_inv = b.repository.get_inventory('r2')
+
+        self.assertEquals(store_inv._byid, basis_inv._byid)
+
+    def test_upgrade_from_format4(self):
+        from bzrlib.tests.test_upgrade import _upgrade_dir_template
+        if self.repository_format.__class__ == repository.RepositoryFormat4:
+            raise TestSkipped('Cannot convert format-4 to itself')
+        self.build_tree_contents(_upgrade_dir_template)
+        old_repodir = bzrlib.bzrdir.BzrDir.open_unsupported('.')
+        old_repo_format = old_repodir.open_repository()._format
+        format = self.repository_format._matchingbzrdir
+        try:
+            format.repository_format = self.repository_format
+        except AttributeError:
+            pass
+        upgrade('.', format)
+
+    def test_pointless_commit(self):
+        tree = self.make_branch_and_tree('.')
+        self.assertRaises(errors.PointlessCommit, tree.commit, 'pointless',
+                          allow_pointless=False)
+        tree.commit('pointless', allow_pointless=True)
+
+
 class TestCaseWithComplexRepository(TestCaseWithRepository):
 
     def setUp(self):
@@ -287,13 +417,31 @@ class TestCaseWithComplexRepository(TestCaseWithRepository):
         # add a real revision 'rev2' based on rev1
         tree_a.commit('rev2', rev_id='rev2', allow_pointless=True)
         # add a reference to a ghost
-        tree_a.add_pending_merge('ghost1')
+        tree_a.add_parent_tree_id('ghost1')
         tree_a.commit('rev3', rev_id='rev3', allow_pointless=True)
         # add another reference to a ghost, and a second ghost.
-        tree_a.add_pending_merge('ghost1')
-        tree_a.add_pending_merge('ghost2')
+        tree_a.add_parent_tree_id('ghost1')
+        tree_a.add_parent_tree_id('ghost2')
         tree_a.commit('rev4', rev_id='rev4', allow_pointless=True)
 
+    def test_revision_trees(self):
+        revision_ids = ['rev1', 'rev2', 'rev3', 'rev4']
+        repository = self.bzrdir.open_repository()
+        trees1 = list(repository.revision_trees(revision_ids))
+        trees2 = [repository.revision_tree(t) for t in revision_ids]
+        assert len(trees1) == len(trees2)
+        for tree1, tree2 in zip(trees1, trees2):
+            assert not tree2.changes_from(tree1).has_changed()
+
+    def test_get_deltas_for_revisions(self):
+        repository = self.bzrdir.open_repository()
+        revisions = [repository.get_revision(r) for r in 
+                     ['rev1', 'rev2', 'rev3', 'rev4']]
+        deltas1 = list(repository.get_deltas_for_revisions(revisions))
+        deltas2 = [repository.get_revision_delta(r.revision_id) for r in
+                   revisions]
+        assert deltas1 == deltas2
+
     def test_all_revision_ids(self):
         # all_revision_ids -> all revisions
         self.assertEqual(['rev1', 'rev2', 'rev3', 'rev4'],
@@ -327,6 +475,9 @@ class TestCaseWithComplexRepository(TestCaseWithRepository):
                           'rev3':['rev2'],
                           },
                          self.bzrdir.open_repository().get_revision_graph('rev3'))
+        # and we can ask for the NULLREVISION graph
+        self.assertEqual({},
+            self.bzrdir.open_repository().get_revision_graph(NULL_REVISION))
 
     def test_get_revision_graph_with_ghosts(self):
         # we can get a graph object with roots, ghosts, ancestors and
@@ -349,6 +500,10 @@ class TestCaseWithComplexRepository(TestCaseWithRepository):
                           'rev4':{},
                           },
                           graph.get_descendants())
+        # and we can ask for the NULLREVISION graph
+        graph = repo.get_revision_graph_with_ghosts([NULL_REVISION])
+        self.assertEqual({}, graph.get_ancestors())
+        self.assertEqual({}, graph.get_descendants())
 
 
 class TestCaseWithCorruptRepository(TestCaseWithRepository):
@@ -358,7 +513,8 @@ class TestCaseWithCorruptRepository(TestCaseWithRepository):
         # a inventory with no parents and the revision has parents..
         # i.e. a ghost.
         repo = self.make_repository('inventory_with_unnecessary_ghost')
-        inv = bzrlib.tree.EmptyTree().inventory
+        inv = Inventory(revision_id = 'ghost')
+        inv.root.revision = 'ghost'
         sha1 = repo.add_inventory('ghost', inv, [])
         rev = bzrlib.revision.Revision(timestamp=0,
                                        timezone=None,
@@ -369,6 +525,8 @@ class TestCaseWithCorruptRepository(TestCaseWithRepository):
         rev.parent_ids = ['the_ghost']
         repo.add_revision('ghost', rev)
          
+        inv = Inventory(revision_id = 'the_ghost')
+        inv.root.revision = 'the_ghost'
         sha1 = repo.add_inventory('the_ghost', inv, [])
         rev = bzrlib.revision.Revision(timestamp=0,
                                        timezone=None,
@@ -398,3 +556,29 @@ class TestCaseWithCorruptRepository(TestCaseWithRepository):
     def test_corrupt_revision_get_revision_reconcile(self):
         repo = repository.Repository.open('inventory_with_unnecessary_ghost')
         repo.get_revision_reconcile('ghost')
+
+
+class TestEscaping(TestCaseWithTransport):
+    """Test that repositories can be stored correctly on VFAT transports.
+    
+    Makes sure we have proper escaping of invalid characters, etc.
+
+    It'd be better to test all operations on the FakeVFATTransportDecorator,
+    but working trees go straight to the os not through the Transport layer.
+    Therefore we build some history first in the regular way and then 
+    check it's safe to access for vfat.
+    """
+
+    def test_on_vfat(self):
+        FOO_ID = 'foo<:>ID'
+        REV_ID = 'revid-1'
+        wt = self.make_branch_and_tree('repo')
+        self.build_tree(["repo/foo"])
+        # add file with id containing wierd characters
+        wt.add(['foo'], [FOO_ID])
+        wt.commit('this is my new commit', rev_id=REV_ID)
+        # now access over vfat; should be safe
+        branch = bzrdir.BzrDir.open('vfat+' + self.get_url('repo')).open_branch()
+        revtree = branch.repository.revision_tree(REV_ID)
+        contents = revtree.get_file_text(FOO_ID)
+        self.assertEqual(contents, 'contents of repo/foo\n')
