@@ -1,4 +1,4 @@
-# Copyright (C) 2005 by Canonical Ltd
+# Copyright (C) 2005, 2006 by Canonical Ltd
 #   Authors: Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -23,9 +23,15 @@ import os
 import sys
 
 #import bzrlib specific imports here
-import bzrlib.config as config
-import bzrlib.errors as errors
-from bzrlib.tests import TestCase, TestCaseInTempDir
+from bzrlib import (
+    config,
+    errors,
+    osutils,
+    urlutils,
+    )
+from bzrlib.branch import Branch
+from bzrlib.bzrdir import BzrDir
+from bzrlib.tests import TestCase, TestCaseInTempDir, TestCaseWithTransport
 
 
 sample_long_alias="log -r-15..-1 --line"
@@ -41,15 +47,18 @@ sample_config_text = ("[DEFAULT]\n"
 
 
 sample_always_signatures = ("[DEFAULT]\n"
-                            "check_signatures=require\n")
+                            "check_signatures=ignore\n"
+                            "create_signatures=always")
 
 
 sample_ignore_signatures = ("[DEFAULT]\n"
-                            "check_signatures=ignore\n")
+                            "check_signatures=require\n"
+                            "create_signatures=never")
 
 
 sample_maybe_signatures = ("[DEFAULT]\n"
-                            "check_signatures=check-available\n")
+                            "check_signatures=ignore\n"
+                            "create_signatures=when-required")
 
 
 sample_branches_text = ("[http://www.example.com]\n"
@@ -99,15 +108,25 @@ class InstrumentedConfigObj(object):
 
 class FakeBranch(object):
 
-    def __init__(self):
-        self.base = "http://example.com/branches/demo"
-        self.control_files = FakeControlFiles()
+    def __init__(self, base=None, user_id=None):
+        if base is None:
+            self.base = "http://example.com/branches/demo"
+        else:
+            self.base = base
+        self.control_files = FakeControlFiles(user_id=user_id)
+
+    def lock_write(self):
+        pass
+
+    def unlock(self):
+        pass
 
 
 class FakeControlFiles(object):
 
-    def __init__(self):
-        self.email = 'Robert Collins <robertc@example.net>\n'
+    def __init__(self, user_id=None):
+        self.email = user_id
+        self.files = {}
 
     def get_utf8(self, filename):
         if filename != 'email':
@@ -116,6 +135,15 @@ class FakeControlFiles(object):
             return StringIO(self.email)
         raise errors.NoSuchFile(filename)
 
+    def get(self, filename):
+        try:
+            return StringIO(self.files[filename])
+        except KeyError:
+            raise errors.NoSuchFile(filename)
+
+    def put(self, filename, fileobj):
+        self.files[filename] = fileobj.read()
+
 
 class InstrumentedConfig(config.Config):
     """An instrumented config that supplies stubs for template methods."""
@@ -172,8 +200,11 @@ class TestConfig(TestCase):
 
     def test_signatures_default(self):
         my_config = config.Config()
+        self.assertFalse(my_config.signature_needed())
         self.assertEqual(config.CHECK_IF_POSSIBLE,
                          my_config.signature_checking())
+        self.assertEqual(config.SIGN_WHEN_REQUIRED,
+                         my_config.signing_policy())
 
     def test_signatures_template_method(self):
         my_config = InstrumentedConfig()
@@ -248,6 +279,14 @@ class TestConfigPath(TestCase):
             self.assertEqual(config.branches_config_filename(),
                              '/home/bogus/.bazaar/branches.conf')
 
+    def test_locations_config_filename(self):
+        if sys.platform == 'win32':
+            self.assertEqual(config.locations_config_filename(), 
+                'C:/Documents and Settings/bogus/Application Data/bazaar/2.0/locations.conf')
+        else:
+            self.assertEqual(config.locations_config_filename(),
+                             '/home/bogus/.bazaar/locations.conf')
+
 class TestIniConfig(TestCase):
 
     def test_contructs(self):
@@ -286,7 +325,7 @@ class TestGetConfig(TestCase):
                                           'utf-8')])
 
 
-class TestBranchConfig(TestCaseInTempDir):
+class TestBranchConfig(TestCaseWithTransport):
 
     def test_constructs(self):
         branch = FakeBranch()
@@ -300,6 +339,58 @@ class TestBranchConfig(TestCaseInTempDir):
         self.assertEqual(branch.base, location_config.location)
         self.failUnless(location_config is my_config._get_location_config())
 
+    def test_get_config(self):
+        """The Branch.get_config method works properly"""
+        b = BzrDir.create_standalone_workingtree('.').branch
+        my_config = b.get_config()
+        self.assertIs(my_config.get_user_option('wacky'), None)
+        my_config.set_user_option('wacky', 'unlikely')
+        self.assertEqual(my_config.get_user_option('wacky'), 'unlikely')
+
+        # Ensure we get the same thing if we start again
+        b2 = Branch.open('.')
+        my_config2 = b2.get_config()
+        self.assertEqual(my_config2.get_user_option('wacky'), 'unlikely')
+
+    def test_has_explicit_nickname(self):
+        b = self.make_branch('.')
+        self.assertFalse(b.get_config().has_explicit_nickname())
+        b.nick = 'foo'
+        self.assertTrue(b.get_config().has_explicit_nickname())
+
+    def test_config_url(self):
+        """The Branch.get_config will use section that uses a local url"""
+        branch = self.make_branch('branch')
+        self.assertEqual('branch', branch.nick)
+
+        locations = config.locations_config_filename()
+        config.ensure_config_dir_exists()
+        local_url = urlutils.local_path_to_url('branch')
+        open(locations, 'wb').write('[%s]\nnickname = foobar' 
+                                    % (local_url,))
+        self.assertEqual('foobar', branch.nick)
+
+    def test_config_local_path(self):
+        """The Branch.get_config will use a local system path"""
+        branch = self.make_branch('branch')
+        self.assertEqual('branch', branch.nick)
+
+        locations = config.locations_config_filename()
+        config.ensure_config_dir_exists()
+        open(locations, 'wb').write('[%s/branch]\nnickname = barry' 
+                                    % (osutils.getcwd().encode('utf8'),))
+        self.assertEqual('barry', branch.nick)
+
+    def test_config_creates_local(self):
+        """Creating a new entry in config uses a local path."""
+        branch = self.make_branch('branch')
+        branch.set_push_location('http://foobar')
+        locations = config.locations_config_filename()
+        local_path = osutils.getcwd().encode('utf8')
+        # Surprisingly ConfigObj doesn't create a trailing newline
+        self.check_file_contents(locations,
+            '[%s/branch]\npush_location = http://foobar' % (local_path,))
+
 
 class TestGlobalConfigItems(TestCase):
 
@@ -326,24 +417,30 @@ class TestGlobalConfigItems(TestCase):
         config_file = StringIO(sample_always_signatures)
         my_config = config.GlobalConfig()
         my_config._parser = my_config._get_parser(file=config_file)
-        self.assertEqual(config.CHECK_ALWAYS,
+        self.assertEqual(config.CHECK_NEVER,
                          my_config.signature_checking())
+        self.assertEqual(config.SIGN_ALWAYS,
+                         my_config.signing_policy())
         self.assertEqual(True, my_config.signature_needed())
 
     def test_signatures_if_possible(self):
         config_file = StringIO(sample_maybe_signatures)
         my_config = config.GlobalConfig()
         my_config._parser = my_config._get_parser(file=config_file)
-        self.assertEqual(config.CHECK_IF_POSSIBLE,
+        self.assertEqual(config.CHECK_NEVER,
                          my_config.signature_checking())
+        self.assertEqual(config.SIGN_WHEN_REQUIRED,
+                         my_config.signing_policy())
         self.assertEqual(False, my_config.signature_needed())
 
     def test_signatures_ignore(self):
         config_file = StringIO(sample_ignore_signatures)
         my_config = config.GlobalConfig()
         my_config._parser = my_config._get_parser(file=config_file)
-        self.assertEqual(config.CHECK_NEVER,
+        self.assertEqual(config.CHECK_ALWAYS,
                          my_config.signature_checking())
+        self.assertEqual(config.SIGN_NEVER,
+                         my_config.signing_policy())
         self.assertEqual(False, my_config.signature_needed())
 
     def _get_sample_config(self):
@@ -396,7 +493,8 @@ class TestGlobalConfigItems(TestCase):
         my_config = self._get_sample_config()
         self.assertEqual(sample_long_alias, my_config.get_alias('ll'))
 
-class TestLocationConfig(TestCase):
+
+class TestLocationConfig(TestCaseInTempDir):
 
     def test_constructs(self):
         my_config = config.LocationConfig('http://example.com')
@@ -409,156 +507,163 @@ class TestLocationConfig(TestCase):
         # replace the class that is constructured, to check its parameters
         oldparserclass = config.ConfigObj
         config.ConfigObj = InstrumentedConfigObj
-        my_config = config.LocationConfig('http://www.example.com')
         try:
+            my_config = config.LocationConfig('http://www.example.com')
             parser = my_config._get_parser()
         finally:
             config.ConfigObj = oldparserclass
         self.failUnless(isinstance(parser, InstrumentedConfigObj))
         self.assertEqual(parser._calls,
-                         [('__init__', config.branches_config_filename())])
+                         [('__init__', config.locations_config_filename(),
+                           'utf-8')])
+        config.ensure_config_dir_exists()
+        #os.mkdir(config.config_dir())
+        f = file(config.branches_config_filename(), 'wb')
+        f.write('')
+        f.close()
+        oldparserclass = config.ConfigObj
+        config.ConfigObj = InstrumentedConfigObj
+        try:
+            my_config = config.LocationConfig('http://www.example.com')
+            parser = my_config._get_parser()
+        finally:
+            config.ConfigObj = oldparserclass
 
     def test_get_global_config(self):
-        my_config = config.LocationConfig('http://example.com')
+        my_config = config.BranchConfig(FakeBranch('http://example.com'))
         global_config = my_config._get_global_config()
         self.failUnless(isinstance(global_config, config.GlobalConfig))
         self.failUnless(global_config is my_config._get_global_config())
 
     def test__get_section_no_match(self):
-        self.get_location_config('/')
-        self.assertEqual(None, self.my_config._get_section())
+        self.get_branch_config('/')
+        self.assertEqual(None, self.my_location_config._get_section())
         
     def test__get_section_exact(self):
-        self.get_location_config('http://www.example.com')
+        self.get_branch_config('http://www.example.com')
         self.assertEqual('http://www.example.com',
-                         self.my_config._get_section())
+                         self.my_location_config._get_section())
    
     def test__get_section_suffix_does_not(self):
-        self.get_location_config('http://www.example.com-com')
-        self.assertEqual(None, self.my_config._get_section())
+        self.get_branch_config('http://www.example.com-com')
+        self.assertEqual(None, self.my_location_config._get_section())
 
     def test__get_section_subdir_recursive(self):
-        self.get_location_config('http://www.example.com/com')
+        self.get_branch_config('http://www.example.com/com')
         self.assertEqual('http://www.example.com',
-                         self.my_config._get_section())
+                         self.my_location_config._get_section())
 
     def test__get_section_subdir_matches(self):
-        self.get_location_config('http://www.example.com/useglobal')
+        self.get_branch_config('http://www.example.com/useglobal')
         self.assertEqual('http://www.example.com/useglobal',
-                         self.my_config._get_section())
+                         self.my_location_config._get_section())
 
     def test__get_section_subdir_nonrecursive(self):
-        self.get_location_config(
+        self.get_branch_config(
             'http://www.example.com/useglobal/childbranch')
         self.assertEqual('http://www.example.com',
-                         self.my_config._get_section())
+                         self.my_location_config._get_section())
 
     def test__get_section_subdir_trailing_slash(self):
-        self.get_location_config('/b')
-        self.assertEqual('/b/', self.my_config._get_section())
+        self.get_branch_config('/b')
+        self.assertEqual('/b/', self.my_location_config._get_section())
 
     def test__get_section_subdir_child(self):
-        self.get_location_config('/a/foo')
-        self.assertEqual('/a/*', self.my_config._get_section())
+        self.get_branch_config('/a/foo')
+        self.assertEqual('/a/*', self.my_location_config._get_section())
 
     def test__get_section_subdir_child_child(self):
-        self.get_location_config('/a/foo/bar')
-        self.assertEqual('/a/', self.my_config._get_section())
+        self.get_branch_config('/a/foo/bar')
+        self.assertEqual('/a/', self.my_location_config._get_section())
 
     def test__get_section_trailing_slash_with_children(self):
-        self.get_location_config('/a/')
-        self.assertEqual('/a/', self.my_config._get_section())
+        self.get_branch_config('/a/')
+        self.assertEqual('/a/', self.my_location_config._get_section())
 
     def test__get_section_explicit_over_glob(self):
-        self.get_location_config('/a/c')
-        self.assertEqual('/a/c', self.my_config._get_section())
+        self.get_branch_config('/a/c')
+        self.assertEqual('/a/c', self.my_location_config._get_section())
 
-    def get_location_config(self, location, global_config=None):
-        if global_config is None:
-            global_file = StringIO(sample_config_text)
-        else:
-            global_file = StringIO(global_config)
-        branches_file = StringIO(sample_branches_text)
-        self.my_config = config.LocationConfig(location)
-        self.my_config._get_parser(branches_file)
-        self.my_config._get_global_config()._get_parser(global_file)
 
     def test_location_without_username(self):
-        self.get_location_config('http://www.example.com/useglobal')
-        self.assertEqual('Robert Collins <robertc@example.com>',
+        self.get_branch_config('http://www.example.com/useglobal')
+        self.assertEqual(u'Erik B\u00e5gfors <erik@bagfors.nu>',
                          self.my_config.username())
 
     def test_location_not_listed(self):
-        self.get_location_config('/home/robertc/sources')
-        self.assertEqual('Robert Collins <robertc@example.com>',
+        """Test that the global username is used when no location matches"""
+        self.get_branch_config('/home/robertc/sources')
+        self.assertEqual(u'Erik B\u00e5gfors <erik@bagfors.nu>',
                          self.my_config.username())
 
     def test_overriding_location(self):
-        self.get_location_config('http://www.example.com/foo')
+        self.get_branch_config('http://www.example.com/foo')
         self.assertEqual('Robert Collins <robertc@example.org>',
                          self.my_config.username())
 
     def test_signatures_not_set(self):
-        self.get_location_config('http://www.example.com',
+        self.get_branch_config('http://www.example.com',
                                  global_config=sample_ignore_signatures)
-        self.assertEqual(config.CHECK_NEVER,
+        self.assertEqual(config.CHECK_ALWAYS,
                          self.my_config.signature_checking())
+        self.assertEqual(config.SIGN_NEVER,
+                         self.my_config.signing_policy())
 
     def test_signatures_never(self):
-        self.get_location_config('/a/c')
+        self.get_branch_config('/a/c')
         self.assertEqual(config.CHECK_NEVER,
                          self.my_config.signature_checking())
         
     def test_signatures_when_available(self):
-        self.get_location_config('/a/', global_config=sample_ignore_signatures)
+        self.get_branch_config('/a/', global_config=sample_ignore_signatures)
         self.assertEqual(config.CHECK_IF_POSSIBLE,
                          self.my_config.signature_checking())
         
     def test_signatures_always(self):
-        self.get_location_config('/b')
+        self.get_branch_config('/b')
         self.assertEqual(config.CHECK_ALWAYS,
                          self.my_config.signature_checking())
         
     def test_gpg_signing_command(self):
-        self.get_location_config('/b')
+        self.get_branch_config('/b')
         self.assertEqual("gnome-gpg", self.my_config.gpg_signing_command())
 
     def test_gpg_signing_command_missing(self):
-        self.get_location_config('/a')
+        self.get_branch_config('/a')
         self.assertEqual("false", self.my_config.gpg_signing_command())
 
     def test_get_user_option_global(self):
-        self.get_location_config('/a')
+        self.get_branch_config('/a')
         self.assertEqual('something',
                          self.my_config.get_user_option('user_global_option'))
 
     def test_get_user_option_local(self):
-        self.get_location_config('/a')
+        self.get_branch_config('/a')
         self.assertEqual('local',
                          self.my_config.get_user_option('user_local_option'))
         
     def test_post_commit_default(self):
-        self.get_location_config('/a/c')
+        self.get_branch_config('/a/c')
         self.assertEqual('bzrlib.tests.test_config.post_commit',
                          self.my_config.post_commit())
 
-
-class TestLocationConfig(TestCaseInTempDir):
-
-    def get_location_config(self, location, global_config=None):
+    def get_branch_config(self, location, global_config=None):
         if global_config is None:
             global_file = StringIO(sample_config_text.encode('utf-8'))
         else:
             global_file = StringIO(global_config.encode('utf-8'))
         branches_file = StringIO(sample_branches_text.encode('utf-8'))
-        self.my_config = config.LocationConfig(location)
-        self.my_config._get_parser(branches_file)
+        self.my_config = config.BranchConfig(FakeBranch(location))
+        # Force location config to use specified file
+        self.my_location_config = self.my_config._get_location_config()
+        self.my_location_config._get_parser(branches_file)
+        # Force global config to use specified file
         self.my_config._get_global_config()._get_parser(global_file)
 
     def test_set_user_setting_sets_and_saves(self):
-        self.get_location_config('/a/c')
+        self.get_branch_config('/a/c')
         record = InstrumentedConfigObj("foo")
-        self.my_config._parser = record
+        self.my_location_config._parser = record
 
         real_mkdir = os.mkdir
         self.created = False
@@ -569,7 +674,7 @@ class TestLocationConfig(TestCaseInTempDir):
 
         os.mkdir = checked_mkdir
         try:
-            self.my_config.set_user_option('foo', 'bar')
+            self.my_config.set_user_option('foo', 'bar', local=True)
         finally:
             os.mkdir = real_mkdir
 
@@ -582,72 +687,136 @@ class TestLocationConfig(TestCaseInTempDir):
                           ('write',)],
                          record._calls[1:])
 
+    def test_set_user_setting_sets_and_saves2(self):
+        self.get_branch_config('/a/c')
+        self.assertIs(self.my_config.get_user_option('foo'), None)
+        self.my_config.set_user_option('foo', 'bar')
+        self.assertEqual(
+            self.my_config.branch.control_files.files['branch.conf'], 
+            'foo = bar')
+        self.assertEqual(self.my_config.get_user_option('foo'), 'bar')
+        self.my_config.set_user_option('foo', 'baz', local=True)
+        self.assertEqual(self.my_config.get_user_option('foo'), 'baz')
+        self.my_config.set_user_option('foo', 'qux')
+        self.assertEqual(self.my_config.get_user_option('foo'), 'baz')
+        
 
-class TestBranchConfigItems(TestCase):
+precedence_global = 'option = global'
+precedence_branch = 'option = branch'
+precedence_location = """
+[http://]
+recurse = true
+option = recurse
+[http://example.com/specific]
+option = exact
+"""
+
+
+class TestBranchConfigItems(TestCaseInTempDir):
+
+    def get_branch_config(self, global_config=None, location=None, 
+                          location_config=None, branch_data_config=None):
+        my_config = config.BranchConfig(FakeBranch(location))
+        if global_config is not None:
+            global_file = StringIO(global_config.encode('utf-8'))
+            my_config._get_global_config()._get_parser(global_file)
+        self.my_location_config = my_config._get_location_config()
+        if location_config is not None:
+            location_file = StringIO(location_config.encode('utf-8'))
+            self.my_location_config._get_parser(location_file)
+        if branch_data_config is not None:
+            my_config.branch.control_files.files['branch.conf'] = \
+                branch_data_config
+        return my_config
 
     def test_user_id(self):
-        branch = FakeBranch()
+        branch = FakeBranch(user_id='Robert Collins <robertc@example.net>')
         my_config = config.BranchConfig(branch)
         self.assertEqual("Robert Collins <robertc@example.net>",
-                         my_config._get_user_id())
+                         my_config.username())
         branch.control_files.email = "John"
-        self.assertEqual("John", my_config._get_user_id())
+        my_config.set_user_option('email', 
+                                  "Robert Collins <robertc@example.org>")
+        self.assertEqual("John", my_config.username())
+        branch.control_files.email = None
+        self.assertEqual("Robert Collins <robertc@example.org>",
+                         my_config.username())
 
     def test_not_set_in_branch(self):
-        branch = FakeBranch()
-        my_config = config.BranchConfig(branch)
-        branch.control_files.email = None
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        (my_config._get_location_config().
-            _get_global_config()._get_parser(config_file))
+        my_config = self.get_branch_config(sample_config_text)
+        my_config.branch.control_files.email = None
         self.assertEqual(u"Erik B\u00e5gfors <erik@bagfors.nu>",
                          my_config._get_user_id())
-        branch.control_files.email = "John"
+        my_config.branch.control_files.email = "John"
         self.assertEqual("John", my_config._get_user_id())
 
-    def test_BZREMAIL_OVERRIDES(self):
-        os.environ['BZREMAIL'] = "Robert Collins <robertc@example.org>"
+    def test_BZR_EMAIL_OVERRIDES(self):
+        os.environ['BZR_EMAIL'] = "Robert Collins <robertc@example.org>"
         branch = FakeBranch()
         my_config = config.BranchConfig(branch)
         self.assertEqual("Robert Collins <robertc@example.org>",
                          my_config.username())
     
     def test_signatures_forced(self):
-        branch = FakeBranch()
-        my_config = config.BranchConfig(branch)
-        config_file = StringIO(sample_always_signatures)
-        (my_config._get_location_config().
-            _get_global_config()._get_parser(config_file))
-        self.assertEqual(config.CHECK_ALWAYS, my_config.signature_checking())
+        my_config = self.get_branch_config(
+            global_config=sample_always_signatures)
+        self.assertEqual(config.CHECK_NEVER, my_config.signature_checking())
+        self.assertEqual(config.SIGN_ALWAYS, my_config.signing_policy())
+        self.assertTrue(my_config.signature_needed())
+
+    def test_signatures_forced_branch(self):
+        my_config = self.get_branch_config(
+            global_config=sample_ignore_signatures,
+            branch_data_config=sample_always_signatures)
+        self.assertEqual(config.CHECK_NEVER, my_config.signature_checking())
+        self.assertEqual(config.SIGN_ALWAYS, my_config.signing_policy())
+        self.assertTrue(my_config.signature_needed())
 
     def test_gpg_signing_command(self):
-        branch = FakeBranch()
-        my_config = config.BranchConfig(branch)
+        my_config = self.get_branch_config(
+            # branch data cannot set gpg_signing_command
+            branch_data_config="gpg_signing_command=pgp")
         config_file = StringIO(sample_config_text.encode('utf-8'))
-        (my_config._get_location_config().
-            _get_global_config()._get_parser(config_file))
+        my_config._get_global_config()._get_parser(config_file)
         self.assertEqual('gnome-gpg', my_config.gpg_signing_command())
 
     def test_get_user_option_global(self):
         branch = FakeBranch()
         my_config = config.BranchConfig(branch)
         config_file = StringIO(sample_config_text.encode('utf-8'))
-        (my_config._get_location_config().
-            _get_global_config()._get_parser(config_file))
+        (my_config._get_global_config()._get_parser(config_file))
         self.assertEqual('something',
                          my_config.get_user_option('user_global_option'))
 
     def test_post_commit_default(self):
         branch = FakeBranch()
-        branch.base='/a/c'
-        my_config = config.BranchConfig(branch)
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        (my_config._get_location_config().
-            _get_global_config()._get_parser(config_file))
-        branch_file = StringIO(sample_branches_text)
-        my_config._get_location_config()._get_parser(branch_file)
+        my_config = self.get_branch_config(sample_config_text, '/a/c',
+                                           sample_branches_text)
+        self.assertEqual(my_config.branch.base, '/a/c')
         self.assertEqual('bzrlib.tests.test_config.post_commit',
                          my_config.post_commit())
+        my_config.set_user_option('post_commit', 'rmtree_root')
+        # post-commit is ignored when bresent in branch data
+        self.assertEqual('bzrlib.tests.test_config.post_commit',
+                         my_config.post_commit())
+        my_config.set_user_option('post_commit', 'rmtree_root', local=True)
+        self.assertEqual('rmtree_root', my_config.post_commit())
+
+    def test_config_precedence(self):
+        my_config = self.get_branch_config(global_config=precedence_global)
+        self.assertEqual(my_config.get_user_option('option'), 'global')
+        my_config = self.get_branch_config(global_config=precedence_global, 
+                                      branch_data_config=precedence_branch)
+        self.assertEqual(my_config.get_user_option('option'), 'branch')
+        my_config = self.get_branch_config(global_config=precedence_global, 
+                                      branch_data_config=precedence_branch,
+                                      location_config=precedence_location)
+        self.assertEqual(my_config.get_user_option('option'), 'recurse')
+        my_config = self.get_branch_config(global_config=precedence_global, 
+                                      branch_data_config=precedence_branch,
+                                      location_config=precedence_location,
+                                      location='http://example.com/specific')
+        self.assertEqual(my_config.get_user_option('option'), 'exact')
 
 
 class TestMailAddressExtraction(TestCase):
