@@ -8,10 +8,9 @@ from bzrlib.builtins import merge
 from bzrlib.conflicts import ContentsConflict, TextConflict, PathConflict
 from bzrlib.errors import (NotBranchError, NotVersionedError,
                            WorkingTreeNotRevision, BzrCommandError, NoDiff3)
-from bzrlib.inventory import RootEntry
 import bzrlib.inventory as inventory
 from bzrlib.merge import Merge3Merger, Diff3Merger, WeaveMerger
-from bzrlib.osutils import (file_kind, mkdtemp, pathjoin, rename, rmtree,
+from bzrlib.osutils import (file_kind, getcwd, mkdtemp, pathjoin, rename, rmtree,
                             sha_file, 
                             )
 from bzrlib.transform import TreeTransform
@@ -20,15 +19,17 @@ from bzrlib.workingtree import WorkingTree
 
 
 class MergeBuilder(object):
-    def __init__(self):
-        self.dir = mkdtemp(prefix="merge-test")
+    def __init__(self, dir=None):
+        self.dir = mkdtemp(prefix="merge-test", dir=dir)
         def wt(name):
            path = pathjoin(self.dir, name)
            os.mkdir(path)
            wt = bzrlib.bzrdir.BzrDir.create_standalone_workingtree(path)
+           # the tests perform pulls, so need a branch that is writeable.
+           wt.lock_write()
            tt = TreeTransform(wt)
            return wt, tt
-        self.base, self.base_tt = wt('base') 
+        self.base, self.base_tt = wt('base')
         self.this, self.this_tt = wt('this')
         self.other, self.other_tt = wt('other')
 
@@ -51,8 +52,9 @@ class MergeBuilder(object):
         self.base_tt.apply()
         self.base.commit('base commit')
         for tt, wt in ((self.this_tt, self.this), (self.other_tt, self.other)):
+            # why does this not do wt.pull() ?
             wt.branch.pull(self.base.branch)
-            wt.set_last_revision(wt.branch.last_revision())
+            wt.set_parent_ids([wt.branch.last_revision()])
             tt.apply()
             wt.commit('branch commit')
             assert len(wt.branch.revision_history()) == 2
@@ -124,12 +126,6 @@ class MergeBuilder(object):
     def change_perms_tree(self, id, tree, mode):
         os.chmod(tree.full_path(id), mode)
 
-    def merge_changeset(self, merge_factory):
-        conflict_handler = changeset.ExceptionConflictHandler()
-        return make_merge_changeset(self.cset, self.this, self.base,
-                                    self.other, conflict_handler,
-                                    merge_factory)
-
     def apply_inv_change(self, inventory_change, orig_inventory):
         orig_inventory_by_path = {}
         for file_id, path in orig_inventory.iteritems():
@@ -146,7 +142,7 @@ class MergeBuilder(object):
             return orig_inventory_by_path[parent_dir]
         
         def new_path(file_id):
-            if inventory_change.has_key(file_id):
+            if fild_id in inventory_change:
                 return inventory_change[file_id]
             else:
                 parent = parent_id(file_id)
@@ -163,28 +159,23 @@ class MergeBuilder(object):
             new_inventory[file_id] = path
 
         for file_id, path in inventory_change.iteritems():
-            if orig_inventory.has_key(file_id):
+            if file_id in orig_inventory:
                 continue
             new_inventory[file_id] = path
         return new_inventory
 
-    def apply_changeset(self, cset, conflict_handler=None):
-        inventory_change = changeset.apply_changeset(cset,
-                                                     self.this.inventory_dict,
-                                                     self.this.dir,
-                                                     conflict_handler)
-        self.this.inventory_dict =  self.apply_inv_change(inventory_change, 
-                                                     self.this.inventory_dict)
-
     def cleanup(self):
+        self.base.unlock()
+        self.this.unlock()
+        self.other.unlock()
         rmtree(self.dir)
 
 
-class MergeTest(TestCase):
+class MergeTest(TestCaseWithTransport):
 
     def test_change_name(self):
         """Test renames"""
-        builder = MergeBuilder()
+        builder = MergeBuilder(getcwd())
         builder.add_file("1", "TREE_ROOT", "name1", "hello1", True)
         builder.change_name("1", other="name2")
         builder.add_file("2", "TREE_ROOT", "name3", "hello2", True)
@@ -193,7 +184,7 @@ class MergeTest(TestCase):
         builder.change_name("3", this="name6")
         builder.merge()
         builder.cleanup()
-        builder = MergeBuilder()
+        builder = MergeBuilder(getcwd())
         builder.add_file("1", "TREE_ROOT", "name1", "hello1", False)
         builder.change_name("1", other="name2", this="name3")
         conflicts = builder.merge()
@@ -201,7 +192,7 @@ class MergeTest(TestCase):
         builder.cleanup()
 
     def test_merge_one(self):
-        builder = MergeBuilder()
+        builder = MergeBuilder(getcwd())
         builder.add_file("1", "TREE_ROOT", "name1", "hello1", True)
         builder.change_contents("1", other="text4")
         builder.add_file("2", "TREE_ROOT", "name2", "hello1", True)
@@ -209,10 +200,11 @@ class MergeTest(TestCase):
         builder.merge(interesting_ids=["1"])
         self.assertEqual(builder.this.get_file("1").read(), "text4" )
         self.assertEqual(builder.this.get_file("2").read(), "hello1" )
+        builder.cleanup()
         
     def test_file_moves(self):
         """Test moves"""
-        builder = MergeBuilder()
+        builder = MergeBuilder(getcwd())
         builder.add_dir("1", "TREE_ROOT", "dir1")
         builder.add_dir("2", "TREE_ROOT", "dir2")
         builder.add_file("3", "1", "file1", "hello1", True)
@@ -224,7 +216,7 @@ class MergeTest(TestCase):
         builder.merge()
         builder.cleanup()
 
-        builder = MergeBuilder()
+        builder = MergeBuilder(getcwd())
         builder.add_dir("1", "TREE_ROOT", "dir1")
         builder.add_dir("2", "TREE_ROOT", "dir2")
         builder.add_dir("3", "TREE_ROOT", "dir3")
@@ -253,7 +245,7 @@ class MergeTest(TestCase):
 
     def test_reprocess_weave(self):
         # Reprocess works on weaves, and behaves as expected
-        builder = MergeBuilder()
+        builder = MergeBuilder(getcwd())
         builder.add_file('a', 'TREE_ROOT', 'blah', 'a', False)
         builder.change_contents('a', this='b\nc\nd\ne\n', other='z\nc\nd\ny\n')
         builder.merge(WeaveMerger, reprocess=True)
@@ -271,8 +263,7 @@ y
 >>>>>>> MERGE-SOURCE
 """
         self.assertEqualDiff(builder.this.get_file("a").read(), expected)
-
- 
+        builder.cleanup()
 
     def do_contents_test(self, merge_factory):
         """Test merging with specified ContentsChange factory"""
@@ -281,7 +272,7 @@ y
         self.contents_test_conflicts(merge_factory)
 
     def contents_test_success(self, merge_factory):
-        builder = MergeBuilder()
+        builder = MergeBuilder(getcwd())
         builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
         builder.change_contents("1", other="text4")
         builder.add_file("2", "TREE_ROOT", "name3", "text2", False)
@@ -303,20 +294,25 @@ y
         return builder
 
     def contents_test_conflicts(self, merge_factory):
-        builder = MergeBuilder()
+        builder = MergeBuilder(getcwd())
         builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
         builder.change_contents("1", other="text4", this="text3")
         builder.add_file("2", "TREE_ROOT", "name2", "text1", True)
         builder.change_contents("2", other="\x00", this="text3")
+        builder.add_file("3", "TREE_ROOT", "name3", "text5", False)
+        builder.change_perms("3", this=True)
+        builder.change_contents('3', this='moretext')
+        builder.remove_file('3', other=True)
         conflicts = builder.merge(merge_factory)
         self.assertEqual(conflicts, [TextConflict('name1', file_id='1'),
-                                     ContentsConflict('name2', file_id='2')])
+                                     ContentsConflict('name2', file_id='2'),
+                                     ContentsConflict('name3', file_id='3')])
         self.assertEqual(builder.this.get_file('2').read(), '\x00')
         builder.cleanup()
 
     def test_symlink_conflicts(self):
         if sys.platform != "win32":
-            builder = MergeBuilder()
+            builder = MergeBuilder(getcwd())
             builder.add_symlink("2", "TREE_ROOT", "name2", "target1")
             builder.change_target("2", other="target4", base="text3")
             conflicts = builder.merge()
@@ -326,7 +322,7 @@ y
 
     def test_symlink_merge(self):
         if sys.platform != "win32":
-            builder = MergeBuilder()
+            builder = MergeBuilder(getcwd())
             builder.add_symlink("1", "TREE_ROOT", "name1", "target1")
             builder.add_symlink("2", "TREE_ROOT", "name2", "target1")
             builder.add_symlink("3", "TREE_ROOT", "name3", "target1")
@@ -340,14 +336,14 @@ y
             builder.cleanup()
 
     def test_no_passive_add(self):
-        builder = MergeBuilder()
+        builder = MergeBuilder(getcwd())
         builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
         builder.remove_file("1", this=True)
         builder.merge()
         builder.cleanup()
 
     def test_perms_merge(self):
-        builder = MergeBuilder()
+        builder = MergeBuilder(getcwd())
         builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
         builder.change_perms("1", other=False)
         builder.add_file("2", "TREE_ROOT", "name2", "text2", True)
@@ -364,7 +360,7 @@ y
         builder.cleanup();
 
     def test_new_suffix(self):
-        builder = MergeBuilder()
+        builder = MergeBuilder(getcwd())
         builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
         builder.change_contents("1", other="text3")
         builder.add_file("2", "TREE_ROOT", "name1.new", "text2", True)
@@ -373,13 +369,14 @@ y
         builder.cleanup()
 
     def test_spurious_conflict(self):
-        builder = MergeBuilder()
+        builder = MergeBuilder(getcwd())
         builder.add_file("1", "TREE_ROOT", "name1", "text1", False)
         builder.remove_file("1", other=True)
         builder.add_file("2", "TREE_ROOT", "name1", "text1", False, this=False, 
                          base=False)
         conflicts = builder.merge()
         self.assertEqual(conflicts, []) 
+        builder.cleanup()
 
 
 class FunctionalMergeTest(TestCaseWithTransport):
@@ -459,11 +456,10 @@ class FunctionalMergeTest(TestCaseWithTransport):
         b = wtb.branch
         file('b/b_file', 'wb').write('contents\n')
         wtb.add('b_file')
-        wtb.commit('b_revision', allow_pointless=False)
+        b_rev = wtb.commit('b_revision', allow_pointless=False)
         merge(['b', -1], ['b', 0], this_dir='a')
         self.assert_(os.path.lexists('a/b_file'))
-        self.assertEqual(wta.pending_merges(),
-                         [b.last_revision()]) 
+        self.assertEqual([b_rev], wta.get_parent_ids()[1:])
 
     def test_merge_unrelated_conflicting(self):
         """Sucessfully merges unrelated branches with common names"""
@@ -476,11 +472,11 @@ class FunctionalMergeTest(TestCaseWithTransport):
         b = wtb.branch
         file('b/file', 'wb').write('contents\n')
         wtb.add('file')
-        wtb.commit('b_revision', allow_pointless=False)
+        b_rev = wtb.commit('b_revision', allow_pointless=False)
         merge(['b', -1], ['b', 0], this_dir='a')
         self.assert_(os.path.lexists('a/file'))
         self.assert_(os.path.lexists('a/file.moved'))
-        self.assertEqual(wta.pending_merges(), [b.last_revision()])
+        self.assertEqual([b_rev], wta.get_parent_ids()[1:])
 
     def test_merge_deleted_conflicts(self):
         wta = self.make_branch_and_tree('a')
