@@ -1,15 +1,15 @@
 # Copyright (C) 2005 by Canonical Development Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -38,6 +38,14 @@ class TestStores(object):
         f = store.get(fileid)
         self.assertEqual(f.read(), value)
 
+    def test_add_str_deprecated(self):
+        os.mkdir('a')
+        store = self.get_store('a')
+        self.callDeprecated(['Passing a string to Store.add'
+                             ' was deprecated in version 0.11.'],
+                            store.add, 'foo', '1')
+        self.assertEqual('foo', store.get('1').read())
+
     def fill_store(self, store):
         store.add(StringIO('hello'), 'a')
         store.add(StringIO('other'), 'b')
@@ -48,7 +56,7 @@ class TestStores(object):
         """Test copying"""
         os.mkdir('a')
         store_a = self.get_store('a')
-        store_a.add('foo', '1')
+        store_a.add(StringIO('foo'), '1')
         os.mkdir('b')
         store_b = self.get_store('b')
         store_b.copy_all_ids(store_a)
@@ -81,7 +89,7 @@ class TestStores(object):
 class TestCompressedTextStore(TestCaseInTempDir, TestStores):
 
     def get_store(self, path=u'.'):
-        t = LocalTransport(path)
+        t = transport.get_transport(path)
         return TextStore(t, compressed=True)
 
     def test_total_size(self):
@@ -138,7 +146,7 @@ class TestMemoryStore(TestCase):
 class TestTextStore(TestCaseInTempDir, TestStores):
 
     def get_store(self, path=u'.'):
-        t = LocalTransport(path)
+        t = transport.get_transport(path)
         return TextStore(t, compressed=False)
 
     def test_total_size(self):
@@ -156,7 +164,7 @@ class TestTextStore(TestCaseInTempDir, TestStores):
 class TestMixedTextStore(TestCaseInTempDir, TestStores):
 
     def get_store(self, path=u'.', compressed=True):
-        t = LocalTransport(path)
+        t = transport.get_transport(path)
         return TextStore(t, compressed=compressed)
 
     def test_get_mixed(self):
