@@ -1,4 +1,4 @@
-# (C) 2005,2006 Canonical Ltd
+# Copyright (C) 2005, 2006 Canonical Ltd
 # Authors:  Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -17,30 +17,27 @@
 
 from cStringIO import StringIO
 import os
+import sys
 
 import bzrlib
-import bzrlib.branch
-from bzrlib.branch import Branch
-import bzrlib.bzrdir as bzrdir
-from bzrlib.bzrdir import BzrDir
-import bzrlib.errors as errors
-from bzrlib.errors import (NotBranchError, NotVersionedError, 
-                           UnsupportedOperation)
+from bzrlib import branch, bzrdir, errors, osutils, urlutils, workingtree
+from bzrlib.errors import (NotBranchError, NotVersionedError,
+                           UnsupportedOperation, PathsNotVersionedError)
 from bzrlib.osutils import pathjoin, getcwd, has_symlinks
 from bzrlib.tests import TestSkipped
 from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
 from bzrlib.trace import mutter
-import bzrlib.workingtree as workingtree
 from bzrlib.workingtree import (TreeEntry, TreeDirectory, TreeFile, TreeLink,
                                 WorkingTree)
+from bzrlib.conflicts import ConflictList, TextConflict, ContentsConflict
+
 
 
 class TestWorkingTree(TestCaseWithWorkingTree):
 
-    def test_listfiles(self):
+    def test_list_files(self):
         tree = self.make_branch_and_tree('.')
-        os.mkdir('dir')
-        print >> open('file', 'w'), "content"
+        self.build_tree(['dir/', 'file'])
         if has_symlinks():
             os.symlink('target', 'symlink')
         files = list(tree.list_files())
@@ -49,20 +46,67 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         if has_symlinks():
             self.assertEqual(files[2], ('symlink', '?', 'symlink', None, TreeLink()))
 
+    def test_list_files_sorted(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['dir/', 'file', 'dir/file', 'dir/b',
+                         'dir/subdir/', 'a', 'dir/subfile',
+                         'zz_dir/', 'zz_dir/subfile'])
+        files = [(path, kind) for (path, v, kind, file_id, entry)
+                               in tree.list_files()]
+        self.assertEqual([
+            ('a', 'file'),
+            ('dir', 'directory'),
+            ('file', 'file'),
+            ('zz_dir', 'directory'),
+            ], files)
+
+        tree.add(['dir', 'zz_dir'])
+        files = [(path, kind) for (path, v, kind, file_id, entry)
+                               in tree.list_files()]
+        self.assertEqual([
+            ('a', 'file'),
+            ('dir', 'directory'),
+            ('dir/b', 'file'),
+            ('dir/file', 'file'),
+            ('dir/subdir', 'directory'),
+            ('dir/subfile', 'file'),
+            ('file', 'file'),
+            ('zz_dir', 'directory'),
+            ('zz_dir/subfile', 'file'),
+            ], files)
+
     def test_open_containing(self):
         branch = self.make_branch_and_tree('.').branch
+        local_base = urlutils.local_path_from_url(branch.base)
+
+        # Empty opens '.'
         wt, relpath = WorkingTree.open_containing()
         self.assertEqual('', relpath)
-        self.assertEqual(wt.basedir + '/', branch.base)
+        self.assertEqual(wt.basedir + '/', local_base)
+
+        # '.' opens this dir
         wt, relpath = WorkingTree.open_containing(u'.')
         self.assertEqual('', relpath)
-        self.assertEqual(wt.basedir + '/', branch.base)
+        self.assertEqual(wt.basedir + '/', local_base)
+
+        # './foo' finds '.' and a relpath of 'foo'
         wt, relpath = WorkingTree.open_containing('./foo')
         self.assertEqual('foo', relpath)
-        self.assertEqual(wt.basedir + '/', branch.base)
-        wt, relpath = WorkingTree.open_containing('file://' + getcwd() + '/foo')
+        self.assertEqual(wt.basedir + '/', local_base)
+
+        # abspath(foo) finds '.' and relpath of 'foo'
+        wt, relpath = WorkingTree.open_containing('./foo')
+        wt, relpath = WorkingTree.open_containing(getcwd() + '/foo')
         self.assertEqual('foo', relpath)
-        self.assertEqual(wt.basedir + '/', branch.base)
+        self.assertEqual(wt.basedir + '/', local_base)
+
+        # can even be a url: finds '.' and relpath of 'foo'
+        wt, relpath = WorkingTree.open_containing('./foo')
+        wt, relpath = WorkingTree.open_containing(
+                    urlutils.local_path_to_url(getcwd() + '/foo'))
+        self.assertEqual('foo', relpath)
+        self.assertEqual(wt.basedir + '/', local_base)
+
 
     def test_basic_relpath(self):
         # for comprehensive relpath tests, see whitebox.py.
@@ -88,7 +132,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         self.build_tree(['hello.txt'])
         file('hello.txt', 'w').write('initial hello')
 
-        self.assertRaises(NotVersionedError,
+        self.assertRaises(PathsNotVersionedError,
                           tree.revert, ['hello.txt'])
         tree.add(['hello.txt'])
         tree.commit('create initial hello.txt')
@@ -125,31 +169,26 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         tree.revert(['hello.txt'])
         self.failUnlessExists('hello.txt')
 
+    def test_versioned_files_not_unknown(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['hello.txt'])
+        tree.add('hello.txt')
+        self.assertEquals(list(tree.unknowns()),
+                          [])
+
     def test_unknowns(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['hello.txt',
                          'hello.txt.~1~'])
+        self.build_tree_contents([('.bzrignore', '*.~*\n')])
+        tree.add('.bzrignore')
         self.assertEquals(list(tree.unknowns()),
                           ['hello.txt'])
 
-    def test_hashcache(self):
-        from bzrlib.tests.test_hashcache import pause
-        tree = self.make_branch_and_tree('.')
-        self.build_tree(['hello.txt',
-                         'hello.txt.~1~'])
-        tree.add('hello.txt')
-        pause()
-        sha = tree.get_file_sha1(tree.path2id('hello.txt'))
-        self.assertEqual(1, tree._hashcache.miss_count)
-        tree2 = WorkingTree.open('.')
-        sha2 = tree2.get_file_sha1(tree2.path2id('hello.txt'))
-        self.assertEqual(0, tree2._hashcache.miss_count)
-        self.assertEqual(1, tree2._hashcache.hit_count)
-
     def test_initialize(self):
         # initialize should create a working tree and branch in an existing dir
         t = self.make_branch_and_tree('.')
-        b = Branch.open('.')
+        b = branch.Branch.open('.')
         self.assertEqual(t.branch.base, b.base)
         t2 = WorkingTree.open('.')
         self.assertEqual(t.basedir, t2.basedir)
@@ -224,30 +263,31 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         wt = self.make_branch_and_tree('source')
         cloned_dir = wt.bzrdir.clone('target')
         cloned = cloned_dir.open_workingtree()
-        self.assertEqual(cloned.last_revision(), wt.last_revision())
+        self.assertEqual(cloned.get_parent_ids(), wt.get_parent_ids())
 
     def test_last_revision(self):
         wt = self.make_branch_and_tree('source')
-        self.assertEqual(None, wt.last_revision())
+        self.assertEqual([], wt.get_parent_ids())
         wt.commit('A', allow_pointless=True, rev_id='A')
-        self.assertEqual('A', wt.last_revision())
+        self.assertEqual(['A'], wt.get_parent_ids())
 
     def test_set_last_revision(self):
         wt = self.make_branch_and_tree('source')
-        self.assertEqual(None, wt.last_revision())
-        # cannot set the last revision to one not in the branch history.
-        self.assertRaises(errors.NoSuchRevision, wt.set_last_revision, 'A')
+        # set last-revision to one not in the history
+        wt.set_last_revision('A')
+        # set it back to None for an empty tree.
+        wt.set_last_revision(None)
         wt.commit('A', allow_pointless=True, rev_id='A')
-        self.assertEqual('A', wt.last_revision())
+        self.assertEqual(['A'], wt.get_parent_ids())
         # None is aways in the branch
         wt.set_last_revision(None)
-        self.assertEqual(None, wt.last_revision())
+        self.assertEqual([], wt.get_parent_ids())
         # and now we can set it to 'A'
         # because some formats mutate the branch to set it on the tree
         # we need to alter the branch to let this pass.
         wt.branch.set_revision_history(['A', 'B'])
         wt.set_last_revision('A')
-        self.assertEqual('A', wt.last_revision())
+        self.assertEqual(['A'], wt.get_parent_ids())
 
     def test_set_last_revision_different_to_branch(self):
         # working tree formats from the meta-dir format and newer support
@@ -268,19 +308,20 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         wt = branch.bzrdir.create_workingtree()
         wt.commit('A', allow_pointless=True, rev_id='A')
         wt.set_last_revision(None)
-        self.assertEqual(None, wt.last_revision())
+        self.assertEqual([], wt.get_parent_ids())
         self.assertEqual('A', wt.branch.last_revision())
         # and now we can set it back to 'A'
         wt.set_last_revision('A')
-        self.assertEqual('A', wt.last_revision())
+        self.assertEqual(['A'], wt.get_parent_ids())
         self.assertEqual('A', wt.branch.last_revision())
 
     def test_clone_and_commit_preserves_last_revision(self):
+        """Doing a commit into a clone tree does not affect the source."""
         wt = self.make_branch_and_tree('source')
         cloned_dir = wt.bzrdir.clone('target')
         wt.commit('A', allow_pointless=True, rev_id='A')
-        self.assertNotEqual(cloned_dir.open_workingtree().last_revision(),
-                            wt.last_revision())
+        self.assertNotEqual(cloned_dir.open_workingtree().get_parent_ids(),
+                            wt.get_parent_ids())
 
     def test_clone_preserves_content(self):
         wt = self.make_branch_and_tree('source')
@@ -306,10 +347,10 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         wt.commit('A', rev_id='A')
         wt.rename_one('foo', 'bar')
         wt.commit('B', rev_id='B')
-        wt.set_last_revision('B')
+        wt.set_parent_ids(['B'])
         tree = wt.basis_tree()
         self.failUnless(tree.has_filename('bar'))
-        wt.set_last_revision('A')
+        wt.set_parent_ids(['A'])
         tree = wt.basis_tree()
         self.failUnless(tree.has_filename('foo'))
 
@@ -339,57 +380,8 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         source.branch.repository.clone(made_control)
         source.branch.clone(made_control)
         made_tree = self.workingtree_format.initialize(made_control, revision_id='a')
-        self.assertEqual('a', made_tree.last_revision())
+        self.assertEqual(['a'], made_tree.get_parent_ids())
 
-    def test_commit_sets_last_revision(self):
-        tree = self.make_branch_and_tree('tree')
-        tree.commit('foo', rev_id='foo', allow_pointless=True)
-        self.assertEqual('foo', tree.last_revision())
-
-    def test_commit_local_unbound(self):
-        # using the library api to do a local commit on unbound branches is 
-        # also an error
-        tree = self.make_branch_and_tree('tree')
-        self.assertRaises(errors.LocalRequiresBoundBranch,
-                          tree.commit,
-                          'foo',
-                          local=True)
- 
-    def test_local_commit_ignores_master(self):
-        # a --local commit does not require access to the master branch
-        # at all, or even for it to exist.
-        # we test this by setting up a bound branch and then corrupting
-        # the master.
-        master = self.make_branch('master')
-        tree = self.make_branch_and_tree('tree')
-        try:
-            tree.branch.bind(master)
-        except errors.UpgradeRequired:
-            # older format.
-            return
-        master.bzrdir.transport.put('branch-format', StringIO('garbage'))
-        del master
-        # check its corrupted.
-        self.assertRaises(errors.UnknownFormatError,
-                          bzrdir.BzrDir.open,
-                          'master')
-        tree.commit('foo', rev_id='foo', local=True)
- 
-    def test_local_commit_does_not_push_to_master(self):
-        # a --local commit does not require access to the master branch
-        # at all, or even for it to exist.
-        # we test that even when its available it does not push to it.
-        master = self.make_branch('master')
-        tree = self.make_branch_and_tree('tree')
-        try:
-            tree.branch.bind(master)
-        except errors.UpgradeRequired:
-            # older format.
-            return
-        tree.commit('foo', rev_id='foo', local=True)
-        self.failIf(master.repository.has_revision('foo'))
-        self.assertEqual(None, master.last_revision())
-        
     def test_update_sets_last_revision(self):
         # working tree formats from the meta-dir format and newer support
         # setting the last revision on a tree independently of that on the 
@@ -411,7 +403,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         # current format
         self.build_tree(['checkout/', 'tree/file'])
         checkout = bzrdir.BzrDirMetaFormat1().initialize('checkout')
-        bzrlib.branch.BranchReferenceFormat().initialize(checkout, main_branch)
+        branch.BranchReferenceFormat().initialize(checkout, main_branch)
         old_tree = self.workingtree_format.initialize(checkout)
         # now commit to 'tree'
         wt.add('file')
@@ -419,7 +411,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         # and update old_tree
         self.assertEqual(0, old_tree.update())
         self.failUnlessExists('checkout/file')
-        self.assertEqual('A', old_tree.last_revision())
+        self.assertEqual(['A'], old_tree.get_parent_ids())
 
     def test_update_returns_conflict_count(self):
         # working tree formats from the meta-dir format and newer support
@@ -442,7 +434,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         # current format
         self.build_tree(['checkout/', 'tree/file'])
         checkout = bzrdir.BzrDirMetaFormat1().initialize('checkout')
-        bzrlib.branch.BranchReferenceFormat().initialize(checkout, main_branch)
+        branch.BranchReferenceFormat().initialize(checkout, main_branch)
         old_tree = self.workingtree_format.initialize(checkout)
         # now commit to 'tree'
         wt.add('file')
@@ -452,7 +444,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         old_tree.add('file')
         # and update old_tree
         self.assertEqual(1, old_tree.update())
-        self.assertEqual('A', old_tree.last_revision())
+        self.assertEqual(['A'], old_tree.get_parent_ids())
 
     def test_merge_revert(self):
         from bzrlib.merge import merge_inner
@@ -491,7 +483,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
             return
         master_tree.commit('foo', rev_id='foo', allow_pointless=True)
         tree.update()
-        self.assertEqual('foo', tree.last_revision())
+        self.assertEqual(['foo'], tree.get_parent_ids())
         self.assertEqual('foo', tree.branch.last_revision())
 
     def test_update_turns_local_commit_into_merge(self):
@@ -505,18 +497,25 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         # alter 'revert' to revert to the master always. But for now, its
         # good.
         master_tree = self.make_branch_and_tree('master')
+        master_tip = master_tree.commit('first master commit')
         tree = self.make_branch_and_tree('tree')
         try:
             tree.branch.bind(master_tree.branch)
         except errors.UpgradeRequired:
             # legacy branches cannot bind
             return
+        # sync with master
+        tree.update()
+        # work locally
         tree.commit('foo', rev_id='foo', allow_pointless=True, local=True)
         tree.commit('bar', rev_id='bar', allow_pointless=True, local=True)
+        # sync with master prepatory to committing
         tree.update()
-        self.assertEqual(None, tree.last_revision())
-        self.assertEqual([], tree.branch.revision_history())
-        self.assertEqual(['bar'], tree.pending_merges())
+        # which should have pivoted the local tip into a merge
+        self.assertEqual([master_tip, 'bar'], tree.get_parent_ids())
+        # and the local branch history should match the masters now.
+        self.assertEqual(master_tree.branch.revision_history(),
+            tree.branch.revision_history())
 
     def test_merge_modified(self):
         tree = self.make_branch_and_tree('master')
@@ -549,7 +548,7 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         tree.add('blo')
         tree.commit("blah", allow_pointless=False)
         base = tree.basis_tree()
-        BzrDir.open("mine").sprout("other")
+        bzrdir.BzrDir.open("mine").sprout("other")
         file('other/bloo', 'wb').write('two')
         othertree = WorkingTree.open('other')
         othertree.commit('blah', allow_pointless=False)
@@ -563,7 +562,6 @@ class TestWorkingTree(TestCaseWithWorkingTree):
         self.assertEqual(len(tree.conflicts()), 1)
 
     def test_clear_merge_conflicts(self):
-        from bzrlib.conflicts import ConflictList
         tree = self.make_merge_conflicts()
         self.assertEqual(len(tree.conflicts()), 1)
         try:
@@ -572,6 +570,27 @@ class TestWorkingTree(TestCaseWithWorkingTree):
             raise TestSkipped
         self.assertEqual(tree.conflicts(), ConflictList())
 
+    def test_add_conflicts(self):
+        tree = self.make_branch_and_tree('tree')
+        try:
+            tree.add_conflicts([TextConflict('path_a')])
+        except UnsupportedOperation:
+            raise TestSkipped()
+        self.assertEqual(ConflictList([TextConflict('path_a')]),
+                         tree.conflicts())
+        tree.add_conflicts([TextConflict('path_a')])
+        self.assertEqual(ConflictList([TextConflict('path_a')]), 
+                         tree.conflicts())
+        tree.add_conflicts([ContentsConflict('path_a')])
+        self.assertEqual(ConflictList([ContentsConflict('path_a'), 
+                                       TextConflict('path_a')]),
+                         tree.conflicts())
+        tree.add_conflicts([TextConflict('path_b')])
+        self.assertEqual(ConflictList([ContentsConflict('path_a'), 
+                                       TextConflict('path_a'),
+                                       TextConflict('path_b')]),
+                         tree.conflicts())
+
     def test_revert_clear_conflicts(self):
         tree = self.make_merge_conflicts()
         self.assertEqual(len(tree.conflicts()), 1)
@@ -599,3 +618,44 @@ class TestWorkingTree(TestCaseWithWorkingTree):
             tree.branch = tree.branch
         self.assertRaises(AttributeError, set_branch)
 
+    def test_list_files_versioned_before_ignored(self):
+        """A versioned file matching an ignore rule should not be ignored."""
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['foo.pyc'])
+        # ensure that foo.pyc is ignored
+        self.build_tree_contents([('.bzrignore', 'foo.pyc')])
+        tree.add('foo.pyc', 'anid')
+        files = sorted(list(tree.list_files()))
+        self.assertEqual((u'.bzrignore', '?', 'file', None), files[0][:-1])
+        self.assertEqual((u'foo.pyc', 'V', 'file', 'anid'), files[1][:-1])
+        self.assertEqual(2, len(files))
+
+    def test_non_normalized_add_accessible(self):
+        try:
+            self.build_tree([u'a\u030a'])
+        except UnicodeError:
+            raise TestSkipped('Filesystem does not support unicode filenames')
+        tree = self.make_branch_and_tree('.')
+        orig = osutils.normalized_filename
+        osutils.normalized_filename = osutils._accessible_normalized_filename
+        try:
+            tree.add([u'a\u030a'])
+            self.assertEqual([('', 'directory'), (u'\xe5', 'file')],
+                    [(path, ie.kind) for path,ie in 
+                                tree.inventory.iter_entries()])
+        finally:
+            osutils.normalized_filename = orig
+
+    def test_non_normalized_add_inaccessible(self):
+        try:
+            self.build_tree([u'a\u030a'])
+        except UnicodeError:
+            raise TestSkipped('Filesystem does not support unicode filenames')
+        tree = self.make_branch_and_tree('.')
+        orig = osutils.normalized_filename
+        osutils.normalized_filename = osutils._inaccessible_normalized_filename
+        try:
+            self.assertRaises(errors.InvalidNormalization,
+                tree.add, [u'a\u030a'])
+        finally:
+            osutils.normalized_filename = orig
