@@ -1,15 +1,15 @@
 # Copyright (C) 2006 Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -18,6 +18,7 @@ import os
 import errno
 from stat import S_ISREG
 
+from bzrlib import bzrdir, errors
 from bzrlib.errors import (DuplicateKey, MalformedTransform, NoSuchFile,
                            ReusingTransform, NotVersionedError, CantMoveRoot,
                            ExistingLimbo, ImmortalLimbo)
@@ -26,7 +27,9 @@ from bzrlib.osutils import (file_kind, supports_executable, pathjoin, lexists,
                             delete_any)
 from bzrlib.progress import DummyProgress, ProgressPhase
 from bzrlib.trace import mutter, warning
+from bzrlib import tree
 import bzrlib.ui 
+import bzrlib.urlutils as urlutils
 
 
 ROOT_PARENT = "root-parent"
@@ -70,16 +73,17 @@ class TreeTransform(object):
      * set_executability
     """
     def __init__(self, tree, pb=DummyProgress()):
-        """Note: a write lock is taken on the tree.
+        """Note: a tree_write lock is taken on the tree.
         
         Use TreeTransform.finalize() to release the lock
         """
         object.__init__(self)
         self._tree = tree
-        self._tree.lock_write()
+        self._tree.lock_tree_write()
         try:
             control_files = self._tree._control_files
-            self._limbodir = control_files.controlfilename('limbo')
+            self._limbodir = urlutils.local_path_from_url(
+                control_files.controlfilename('limbo'))
             try:
                 os.mkdir(self._limbodir)
             except OSError, e:
@@ -101,6 +105,10 @@ class TreeTransform(object):
         self._removed_id = set()
         self._tree_path_ids = {}
         self._tree_id_paths = {}
+        self._realpaths = {}
+        # Cache of realpath results, to speed up canonical_path
+        self._relpaths = {}
+        # Cache of relpath results, to speed up canonical_path
         self._new_root = self.trans_id_tree_file_id(tree.get_root_id())
         self.__done = False
         self._pb = pb
@@ -211,9 +219,21 @@ class TreeTransform(object):
     def canonical_path(self, path):
         """Get the canonical tree-relative path"""
         # don't follow final symlinks
-        dirname, basename = os.path.split(self._tree.abspath(path))
-        dirname = os.path.realpath(dirname)
-        return self._tree.relpath(pathjoin(dirname, basename))
+        abs = self._tree.abspath(path)
+        if abs in self._relpaths:
+            return self._relpaths[abs]
+        dirname, basename = os.path.split(abs)
+        if dirname not in self._realpaths:
+            self._realpaths[dirname] = os.path.realpath(dirname)
+        dirname = self._realpaths[dirname]
+        abs = pathjoin(dirname, basename)
+        if dirname in self._relpaths:
+            relpath = pathjoin(self._relpaths[dirname], basename)
+            relpath = relpath.rstrip('/\\')
+        else:
+            relpath = self._tree.relpath(abs)
+        self._relpaths[abs] = relpath
+        return relpath
 
     def trans_id_tree_path(self, path):
         """Determine (and maybe set) the transaction ID for a tree path."""
@@ -241,11 +261,22 @@ class TreeTransform(object):
         New file takes the permissions of any existing file with that id,
         unless mode_id is specified.
         """
-        f = file(self._limbo_name(trans_id), 'wb')
-        unique_add(self._new_contents, trans_id, 'file')
-        for segment in contents:
-            f.write(segment)
-        f.close()
+        name = self._limbo_name(trans_id)
+        f = open(name, 'wb')
+        try:
+            try:
+                unique_add(self._new_contents, trans_id, 'file')
+            except:
+                # Clean up the file, it never got registered so
+                # TreeTransform.finalize() won't clean it up.
+                f.close()
+                os.unlink(name)
+                raise
+
+            for segment in contents:
+                f.write(segment)
+        finally:
+            f.close()
         self._set_mode(trans_id, mode_id, S_ISREG)
 
     def _set_mode(self, trans_id, mode_id, typefunc):
@@ -261,7 +292,7 @@ class TreeTransform(object):
         except KeyError:
             return
         try:
-            mode = os.stat(old_path).st_mode
+            mode = os.stat(self._tree.abspath(old_path)).st_mode
         except OSError, e:
             if e.errno == errno.ENOENT:
                 return
@@ -445,7 +476,10 @@ class TreeTransform(object):
 
     def path_changed(self, trans_id):
         """Return True if a trans_id's path has changed."""
-        return trans_id in self._new_name or trans_id in self._new_parent
+        return (trans_id in self._new_name) or (trans_id in self._new_parent)
+
+    def new_contents(self, trans_id):
+        return (trans_id in self._new_contents)
 
     def find_conflicts(self):
         """Find any violations of inventory or filesystem invariants"""
@@ -477,8 +511,7 @@ class TreeTransform(object):
                         self.tree_kind(t) == 'directory'])
         for trans_id in self._removed_id:
             file_id = self.tree_file_id(trans_id)
-            if self._tree.inventory[file_id].kind in ('directory', 
-                                                      'root_directory'):
+            if self._tree.inventory[file_id].kind == 'directory':
                 parents.append(trans_id)
 
         for parent_id in parents:
@@ -521,9 +554,9 @@ class TreeTransform(object):
         if child_id is None:
             return lexists(self._tree.abspath(childpath))
         else:
-            if tt.final_parent(child_id) != parent_id:
+            if self.final_parent(child_id) != parent_id:
                 return False
-            if child_id in tt._removed_contents:
+            if child_id in self._removed_contents:
                 # XXX What about dangling file-ids?
                 return False
             else:
@@ -617,17 +650,18 @@ class TreeTransform(object):
             last_name = None
             last_trans_id = None
             for name, trans_id in name_ids:
-                if name == last_name:
-                    conflicts.append(('duplicate', last_trans_id, trans_id,
-                    name))
                 try:
                     kind = self.final_kind(trans_id)
                 except NoSuchFile:
                     kind = None
                 file_id = self.final_file_id(trans_id)
-                if kind is not None or file_id is not None:
-                    last_name = name
-                    last_trans_id = trans_id
+                if kind is None and file_id is None:
+                    continue
+                if name == last_name:
+                    conflicts.append(('duplicate', last_trans_id, trans_id,
+                    name))
+                last_name = name
+                last_trans_id = trans_id
         return conflicts
 
     def _duplicate_ids(self):
@@ -829,9 +863,12 @@ class TreeTransform(object):
         parent_id is the transaction id of the parent directory of the file.
         contents is an iterator of bytestrings, which will be used to produce
         the file.
-        file_id is the inventory ID of the file, if it is to be versioned.
+        :param file_id: The inventory ID of the file, if it is to be versioned.
+        :param executable: Only valid when a file_id has been supplied.
         """
         trans_id = self._new_entry(name, parent_id, file_id)
+        # TODO: rather than scheduling a set_executable call,
+        # have create_file create the file with the right mode.
         self.create_file(contents, trans_id)
         if executable is not None:
             self.set_executability(executable, trans_id)
@@ -902,36 +939,135 @@ def topology_sorted_ids(tree):
     file_ids.sort(key=tree.id2path)
     return file_ids
 
+
 def build_tree(tree, wt):
-    """Create working tree for a branch, using a Transaction."""
+    """Create working tree for a branch, using a TreeTransform.
+    
+    This function should be used on empty trees, having a tree root at most.
+    (see merge and revert functionality for working with existing trees)
+
+    Existing files are handled like so:
+    
+    - Existing bzrdirs take precedence over creating new items.  They are
+      created as '%s.diverted' % name.
+    - Otherwise, if the content on disk matches the content we are building,
+      it is silently replaced.
+    - Otherwise, conflict resolution will move the old file to 'oldname.moved'.
+    """
+    assert 2 > len(wt.inventory)
     file_trans_id = {}
     top_pb = bzrlib.ui.ui_factory.nested_progress_bar()
     pp = ProgressPhase("Build phase", 2, top_pb)
     tt = TreeTransform(wt)
+    divert = set()
     try:
         pp.next_phase()
-        file_trans_id[wt.get_root_id()] = tt.trans_id_tree_file_id(wt.get_root_id())
-        file_ids = topology_sorted_ids(tree)
+        file_trans_id[wt.get_root_id()] = \
+            tt.trans_id_tree_file_id(wt.get_root_id())
         pb = bzrlib.ui.ui_factory.nested_progress_bar()
         try:
-            for num, file_id in enumerate(file_ids):
-                pb.update("Building tree", num, len(file_ids))
-                entry = tree.inventory[file_id]
+            for num, (tree_path, entry) in \
+                enumerate(tree.inventory.iter_entries_by_dir()):
+                pb.update("Building tree", num, len(tree.inventory))
                 if entry.parent_id is None:
                     continue
+                reparent = False
+                file_id = entry.file_id
+                target_path = wt.abspath(tree_path)
+                try:
+                    kind = file_kind(target_path)
+                except NoSuchFile:
+                    pass
+                else:
+                    if kind == "directory":
+                        try:
+                            bzrdir.BzrDir.open(target_path)
+                        except errors.NotBranchError:
+                            pass
+                        else:
+                            divert.add(file_id)
+                    if (file_id not in divert and
+                        _content_match(tree, entry, file_id, kind,
+                        target_path)):
+                        tt.delete_contents(tt.trans_id_tree_path(tree_path))
+                        if kind == 'directory':
+                            reparent = True
                 if entry.parent_id not in file_trans_id:
                     raise repr(entry.parent_id)
                 parent_id = file_trans_id[entry.parent_id]
-                file_trans_id[file_id] = new_by_entry(tt, entry, parent_id, 
+                file_trans_id[file_id] = new_by_entry(tt, entry, parent_id,
                                                       tree)
+                if reparent:
+                    new_trans_id = file_trans_id[file_id]
+                    old_parent = tt.trans_id_tree_path(tree_path)
+                    _reparent_children(tt, old_parent, new_trans_id)
         finally:
             pb.finished()
         pp.next_phase()
+        divert_trans = set(file_trans_id[f] for f in divert)
+        resolver = lambda t, c: resolve_checkout(t, c, divert_trans)
+        raw_conflicts = resolve_conflicts(tt, pass_func=resolver)
+        conflicts = cook_conflicts(raw_conflicts, tt)
+        for conflict in conflicts:
+            warning(conflict)
+        try:
+            wt.add_conflicts(conflicts)
+        except errors.UnsupportedOperation:
+            pass
         tt.apply()
     finally:
         tt.finalize()
         top_pb.finished()
 
+
+def _reparent_children(tt, old_parent, new_parent):
+    for child in tt.iter_tree_children(old_parent):
+        tt.adjust_path(tt.final_name(child), new_parent, child)
+
+
+def _content_match(tree, entry, file_id, kind, target_path):
+    if entry.kind != kind:
+        return False
+    if entry.kind == "directory":
+        return True
+    if entry.kind == "file":
+        if tree.get_file(file_id).read() == file(target_path, 'rb').read():
+            return True
+    elif entry.kind == "symlink":
+        if tree.get_symlink_target(file_id) == os.readlink(target_path):
+            return True
+    return False
+
+
+def resolve_checkout(tt, conflicts, divert):
+    new_conflicts = set()
+    for c_type, conflict in ((c[0], c) for c in conflicts):
+        # Anything but a 'duplicate' would indicate programmer error
+        assert c_type == 'duplicate', c_type
+        # Now figure out which is new and which is old
+        if tt.new_contents(conflict[1]):
+            new_file = conflict[1]
+            old_file = conflict[2]
+        else:
+            new_file = conflict[2]
+            old_file = conflict[1]
+
+        # We should only get here if the conflict wasn't completely
+        # resolved
+        final_parent = tt.final_parent(old_file)
+        if new_file in divert:
+            new_name = tt.final_name(old_file)+'.diverted'
+            tt.adjust_path(new_name, final_parent, new_file)
+            new_conflicts.add((c_type, 'Diverted to',
+                               new_file, old_file))
+        else:
+            new_name = tt.final_name(old_file)+'.moved'
+            tt.adjust_path(new_name, final_parent, old_file)
+            new_conflicts.add((c_type, 'Moved existing file to',
+                               old_file, new_file))
+    return new_conflicts
+
+
 def new_by_entry(tt, entry, parent_id, tree):
     """Create a new file according to its inventory entry"""
     name = entry.name
@@ -950,7 +1086,7 @@ def new_by_entry(tt, entry, parent_id, tree):
 def create_by_entry(tt, entry, tree, trans_id, lines=None, mode_id=None):
     """Create new file contents according to an inventory entry."""
     if entry.kind == "file":
-        if lines == None:
+        if lines is None:
             lines = tree.get_file(entry.file_id).readlines()
         tt.create_file(lines, trans_id, mode_id=mode_id)
     elif entry.kind == "symlink":
@@ -966,20 +1102,8 @@ def create_entry_executability(tt, entry, trans_id):
 
 def find_interesting(working_tree, target_tree, filenames):
     """Find the ids corresponding to specified filenames."""
-    if not filenames:
-        interesting_ids = None
-    else:
-        interesting_ids = set()
-        for tree_path in filenames:
-            not_found = True
-            for tree in (working_tree, target_tree):
-                file_id = tree.inventory.path2id(tree_path)
-                if file_id is not None:
-                    interesting_ids.add(file_id)
-                    not_found = False
-            if not_found:
-                raise NotVersionedError(path=tree_path)
-    return interesting_ids
+    trees = (working_tree, target_tree)
+    return tree.find_ids_across_trees(filenames, trees)
 
 
 def change_entry(tt, file_id, working_tree, target_tree, 
@@ -1045,17 +1169,12 @@ def _entry_changes(file_id, entry, working_tree):
     try:
         working_kind = working_tree.kind(file_id)
         has_contents = True
-    except OSError, e:
-        if e.errno != errno.ENOENT:
-            raise
+    except NoSuchFile:
         has_contents = False
         contents_mod = True
         meta_mod = False
     if has_contents is True:
-        real_e_kind = entry.kind
-        if real_e_kind == 'root_directory':
-            real_e_kind = 'directory'
-        if real_e_kind != working_kind:
+        if entry.kind != working_kind:
             contents_mod, meta_mod = True, False
         else:
             cur_entry._read_tree_state(working_tree.id2path(file_id), 
@@ -1070,7 +1189,7 @@ def revert(working_tree, target_tree, filenames, backups=False,
     """Revert a working tree's contents to those of a target tree."""
     interesting_ids = find_interesting(working_tree, target_tree, filenames)
     def interesting(file_id):
-        return interesting_ids is None or file_id in interesting_ids
+        return interesting_ids is None or (file_id in interesting_ids)
 
     tt = TreeTransform(working_tree, pb)
     try:
@@ -1117,8 +1236,25 @@ def revert(working_tree, target_tree, filenames, backups=False,
                 if file_id not in target_tree:
                     trans_id = tt.trans_id_tree_file_id(file_id)
                     tt.unversion_file(trans_id)
-                    if file_id in merge_modified:
+                    try:
+                        file_kind = working_tree.kind(file_id)
+                    except NoSuchFile:
+                        file_kind = None
+                    if file_kind != 'file' and file_kind is not None:
+                        keep_contents = False
+                        delete_merge_modified = False
+                    else:
+                        if (file_id in merge_modified and 
+                            merge_modified[file_id] == 
+                            working_tree.get_file_sha1(file_id)):
+                            keep_contents = False
+                            delete_merge_modified = True
+                        else:
+                            keep_contents = True
+                            delete_merge_modified = False
+                    if not keep_contents:
                         tt.delete_contents(trans_id)
+                    if delete_merge_modified:
                         del merge_modified[file_id]
         finally:
             child_pb.finished()
@@ -1140,8 +1276,10 @@ def revert(working_tree, target_tree, filenames, backups=False,
     return conflicts
 
 
-def resolve_conflicts(tt, pb=DummyProgress()):
+def resolve_conflicts(tt, pb=DummyProgress(), pass_func=None):
     """Make many conflict-resolution attempts, but die if they fail"""
+    if pass_func is None:
+        pass_func = conflict_pass
     new_conflicts = set()
     try:
         for n in range(10):
@@ -1149,7 +1287,7 @@ def resolve_conflicts(tt, pb=DummyProgress()):
             conflicts = tt.find_conflicts()
             if len(conflicts) == 0:
                 return new_conflicts
-            new_conflicts.update(conflict_pass(tt, conflicts))
+            new_conflicts.update(pass_func(tt, conflicts))
         raise MalformedTransform(conflicts=conflicts)
     finally:
         pb.clear()
@@ -1188,10 +1326,11 @@ def conflict_pass(tt, conflicts):
             trans_id = conflict[1]
             try:
                 tt.cancel_deletion(trans_id)
-                new_conflicts.add((c_type, 'Not deleting', trans_id))
+                new_conflicts.add(('deleting parent', 'Not deleting', 
+                                   trans_id))
             except KeyError:
                 tt.create_directory(trans_id)
-                new_conflicts.add((c_type, 'Created directory.', trans_id))
+                new_conflicts.add((c_type, 'Created directory', trans_id))
         elif c_type == 'unversioned parent':
             tt.version_file(tt.inactive_file_id(conflict[1]), conflict[1])
             new_conflicts.add((c_type, 'Versioned directory', conflict[1]))
