@@ -19,24 +19,29 @@
 There are separate implementation modules for each http client implementation.
 """
 
-import errno
-import os
-from collections import deque
 from cStringIO import StringIO
+import errno
+import mimetools
+import os
+import posixpath
 import re
+import sys
 import urlparse
 import urllib
 from warnings import warn
 
-from bzrlib.transport import Transport, register_transport, Server
-from bzrlib.errors import (TransportNotPossible, NoSuchFile,
-                           TransportError, ConnectionError)
-from bzrlib.errors import BzrError, BzrCheckError
-from bzrlib.branch import Branch
-from bzrlib.trace import mutter
 # TODO: load these only when running http tests
 import BaseHTTPServer, SimpleHTTPServer, socket, time
 import threading
+
+from bzrlib import errors
+from bzrlib.errors import (TransportNotPossible, NoSuchFile,
+                           TransportError, ConnectionError, InvalidURL)
+from bzrlib.branch import Branch
+from bzrlib.trace import mutter
+from bzrlib.transport import Transport, register_transport, Server
+from bzrlib.transport.http.response import (HttpMultipartRangeResponse,
+                                            HttpRangeResponse)
 from bzrlib.ui import ui_factory
 
 
@@ -70,6 +75,50 @@ def extract_auth(url, password_manager):
     return url
 
 
+def _extract_headers(header_text, url):
+    """Extract the mapping for an rfc2822 header
+
+    This is a helper function for the test suite and for _pycurl.
+    (urllib already parses the headers for us)
+
+    In the case that there are multiple headers inside the file,
+    the last one is returned.
+
+    :param header_text: A string of header information.
+        This expects that the first line of a header will always be HTTP ...
+    :param url: The url we are parsing, so we can raise nice errors
+    :return: mimetools.Message object, which basically acts like a case 
+        insensitive dictionary.
+    """
+    first_header = True
+    remaining = header_text
+
+    if not remaining:
+        raise errors.InvalidHttpResponse(url, 'Empty headers')
+
+    while remaining:
+        header_file = StringIO(remaining)
+        first_line = header_file.readline()
+        if not first_line.startswith('HTTP'):
+            if first_header: # The first header *must* start with HTTP
+                raise errors.InvalidHttpResponse(url,
+                    'Opening header line did not start with HTTP: %s' 
+                    % (first_line,))
+                assert False, 'Opening header line was not HTTP'
+            else:
+                break # We are done parsing
+        first_header = False
+        m = mimetools.Message(header_file)
+
+        # mimetools.Message parses the first header up to a blank line
+        # So while there is remaining data, it probably means there is
+        # another header to be parsed.
+        # Get rid of any preceeding whitespace, which if it is all whitespace
+        # will get rid of everything.
+        remaining = header_file.read().lstrip()
+    return m
+
+
 class HttpTransportBase(Transport):
     """Base class for http implementations.
 
@@ -113,21 +162,24 @@ class HttpTransportBase(Transport):
         implementation qualifier.
         """
         assert isinstance(relpath, basestring)
+        if isinstance(relpath, unicode):
+            raise InvalidURL(relpath, 'paths must not be unicode.')
         if isinstance(relpath, basestring):
             relpath_parts = relpath.split('/')
         else:
             # TODO: Don't call this with an array - no magic interfaces
             relpath_parts = relpath[:]
-        if len(relpath_parts) > 1:
-            if relpath_parts[0] == '':
-                raise ValueError("path %r within branch %r seems to be absolute"
-                                 % (relpath, self._path))
-            if relpath_parts[-1] == '':
+        if relpath.startswith('/'):
+            basepath = []
+        else:
+            # Except for the root, no trailing slashes are allowed
+            if len(relpath_parts) > 1 and relpath_parts[-1] == '':
                 raise ValueError("path %r within branch %r seems to be a directory"
                                  % (relpath, self._path))
-        basepath = self._path.split('/')
-        if len(basepath) > 0 and basepath[-1] == '':
-            basepath = basepath[:-1]
+            basepath = self._path.split('/')
+            if len(basepath) > 0 and basepath[-1] == '':
+                basepath = basepath[:-1]
+
         for p in relpath_parts:
             if p == '..':
                 if len(basepath) == 0:
@@ -192,64 +244,51 @@ class HttpTransportBase(Transport):
         :param offsets: A list of (offset, size) tuples.
         :param return: A list or generator of (offset, data) tuples
         """
-        # Ideally we would pass one big request asking for all the ranges in
-        # one go; however then the server will give a multipart mime response
-        # back, and we can't parse them yet.  So instead we just get one range
-        # per region, and try to coallesce the regions as much as possible.
-        #
-        # The read-coallescing code is not quite regular enough to have a
-        # single driver routine and
-        # helper method in Transport.
-        def do_combined_read(combined_offsets):
-            # read one coalesced block
-            total_size = 0
-            for offset, size in combined_offsets:
-                total_size += size
-            mutter('readv coalesced %d reads.', len(combined_offsets))
-            offset = combined_offsets[0][0]
-            byte_range = (offset, offset + total_size - 1)
-            code, result_file = self._get(relpath, [byte_range])
-            if code == 206:
-                for off, size in combined_offsets:
-                    result_bytes = result_file.read(size)
-                    assert len(result_bytes) == size
-                    yield off, result_bytes
-            elif code == 200:
-                data = result_file.read(offset + total_size)[offset:offset + total_size]
-                pos = 0
-                for offset, size in combined_offsets:
-                    yield offset, data[pos:pos + size]
-                    pos += size
-                del data
-        if not len(offsets):
-            return
-        pending_offsets = deque(offsets)
-        combined_offsets = []
-        while len(pending_offsets):
-            offset, size = pending_offsets.popleft()
-            if not combined_offsets:
-                combined_offsets = [[offset, size]]
-            else:
-                if (len (combined_offsets) < 500 and
-                    combined_offsets[-1][0] + combined_offsets[-1][1] == offset):
-                    # combatible offset:
-                    combined_offsets.append([offset, size])
-                else:
-                    # incompatible, or over the threshold issue a read and yield
-                    pending_offsets.appendleft((offset, size))
-                    for result in do_combined_read(combined_offsets):
-                        yield result
-                    combined_offsets = []
-        # whatever is left is a single coalesced request
-        if len(combined_offsets):
-            for result in do_combined_read(combined_offsets):
-                yield result
+        ranges = self.offsets_to_ranges(offsets)
+        mutter('http readv of %s collapsed %s offsets => %s',
+                relpath, len(offsets), ranges)
+        code, f = self._get(relpath, ranges)
+        for start, size in offsets:
+            f.seek(start, (start < 0) and 2 or 0)
+            start = f.tell()
+            data = f.read(size)
+            if len(data) != size:
+                raise errors.ShortReadvError(relpath, start, size,
+                                             actual=len(data))
+            yield start, data
 
-    def put(self, relpath, f, mode=None):
-        """Copy the file-like or string object into the location.
+    @staticmethod
+    def offsets_to_ranges(offsets):
+        """Turn a list of offsets and sizes into a list of byte ranges.
+
+        :param offsets: A list of tuples of (start, size).  An empty list
+            is not accepted.
+        :return: a list of inclusive byte ranges (start, end) 
+            Adjacent ranges will be combined.
+        """
+        # Make sure we process sorted offsets
+        offsets = sorted(offsets)
+
+        prev_end = None
+        combined = []
+
+        for start, size in offsets:
+            end = start + size - 1
+            if prev_end is None:
+                combined.append([start, end])
+            elif start <= prev_end + 1:
+                combined[-1][1] = end
+            else:
+                combined.append([start, end])
+            prev_end = end
+
+        return combined
+
+    def put_file(self, relpath, f, mode=None):
+        """Copy the file-like object into the location.
 
         :param relpath: Location to put the contents, relative to base.
-        :param f:       File-like or string object.
+        :param f:       File-like object.
         """
         raise TransportNotPossible('http PUT not supported')
 
@@ -261,7 +300,7 @@ class HttpTransportBase(Transport):
         """See Transport.rmdir."""
         raise TransportNotPossible('http does not support rmdir()')
 
-    def append(self, relpath, f):
+    def append_file(self, relpath, f, mode=None):
         """Append the text in the file-like object into the final
         location.
         """
@@ -340,6 +379,25 @@ class HttpTransportBase(Transport):
         else:
             return self.__class__(self.abspath(offset))
 
+    @staticmethod
+    def range_header(ranges, tail_amount):
+        """Turn a list of bytes ranges into a HTTP Range header value.
+
+        :param offsets: A list of byte ranges, (start, end). An empty list
+        is not accepted.
+
+        :return: HTTP range header string.
+        """
+        strings = []
+        for start, end in ranges:
+            strings.append('%d-%d' % (start, end))
+
+        if tail_amount:
+            strings.append('-%d' % tail_amount)
+
+        return ','.join(strings)
+
+
 #---------------- test server facilities ----------------
 # TODO: load these only when running tests
 
@@ -391,12 +449,40 @@ class TestingHTTPRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
         if not self.parse_request(): # An error code has been sent, just exit
             return
         mname = 'do_' + self.command
-        if not hasattr(self, mname):
+        if getattr(self, mname, None) is None:
             self.send_error(501, "Unsupported method (%r)" % self.command)
             return
         method = getattr(self, mname)
         method()
 
+    if sys.platform == 'win32':
+        # On win32 you cannot access non-ascii filenames without
+        # decoding them into unicode first.
+        # However, under Linux, you can access bytestream paths
+        # without any problems. If this function was always active
+        # it would probably break tests when LANG=C was set
+        def translate_path(self, path):
+            """Translate a /-separated PATH to the local filename syntax.
+
+            For bzr, all url paths are considered to be utf8 paths.
+            On Linux, you can access these paths directly over the bytestream
+            request, but on win32, you must decode them, and access them
+            as Unicode files.
+            """
+            # abandon query parameters
+            path = urlparse.urlparse(path)[2]
+            path = posixpath.normpath(urllib.unquote(path))
+            path = path.decode('utf-8')
+            words = path.split('/')
+            words = filter(None, words)
+            path = os.getcwdu()
+            for word in words:
+                drive, word = os.path.splitdrive(word)
+                head, word = os.path.split(word)
+                if word in (os.curdir, os.pardir): continue
+                path = os.path.join(path, word)
+            return path
+
 
 class TestingHTTPServer(BaseHTTPServer.HTTPServer):
     def __init__(self, server_address, RequestHandlerClass, test_case):
@@ -404,16 +490,22 @@ class TestingHTTPServer(BaseHTTPServer.HTTPServer):
                                                 RequestHandlerClass)
         self.test_case = test_case
 
+
 class HttpServer(Server):
     """A test server for http transports."""
 
     # used to form the url that connects to this server
     _url_protocol = 'http'
 
+    # Subclasses can provide a specific request handler
+    def __init__(self, request_handler=TestingHTTPRequestHandler):
+        Server.__init__(self)
+        self.request_handler = request_handler
+
     def _http_start(self):
         httpd = None
         httpd = TestingHTTPServer(('localhost', 0),
-                                  TestingHTTPRequestHandler,
+                                  self.request_handler,
                                   self)
         host, port = httpd.socket.getsockname()
         self._http_base_url = '%s://localhost:%s/' % (self._url_protocol, port)
@@ -474,7 +566,7 @@ class HttpServer(Server):
         
     def get_bogus_url(self):
         """See bzrlib.transport.Server.get_bogus_url."""
-        # this is chosen to try to prevent trouble with proxies, wierd dns,
+        # this is chosen to try to prevent trouble with proxies, weird dns,
         # etc
         return 'http://127.0.0.1:1/'
 
