@@ -1,15 +1,15 @@
 # Copyright (C) 2005 Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -23,14 +23,18 @@ import getpass
 import sys
 
 import bzrlib.progress
-from bzrlib.symbol_versioning import *
+from bzrlib.symbol_versioning import (deprecated_method, 
+        zero_eight)
 from bzrlib.ui import CLIUIFactory
 
 
 class TextUIFactory(CLIUIFactory):
     """A UI factory for Text user interefaces."""
 
-    def __init__(self, bar_type=None):
+    def __init__(self,
+                 bar_type=None,
+                 stdout=None,
+                 stderr=None):
         """Create a TextUIFactory.
 
         :param bar_type: The type of progress bar to create. It defaults to 
@@ -39,7 +43,14 @@ class TextUIFactory(CLIUIFactory):
         """
         super(TextUIFactory, self).__init__()
         self._bar_type = bar_type
-        self.stdout = sys.stdout
+        if stdout is None:
+            self.stdout = sys.stdout
+        else:
+            self.stdout = stdout
+        if stderr is None:
+            self.stderr = sys.stderr
+        else:
+            self.stderr = stderr
 
     def prompt(self, prompt):
         """Emit prompt on the CLI."""
@@ -64,10 +75,9 @@ class TextUIFactory(CLIUIFactory):
         """
         prompt = (prompt % kwargs).encode(sys.stdout.encoding, 'replace')
         prompt += ': '
-        try:
-            return getpass.getpass(prompt)
-        except KeyboardInterrupt:
-            return None
+        # There's currently no way to say 'i decline to enter a password'
+        # as opposed to 'my password is empty' -- does it matter?
+        return getpass.getpass(prompt)
 
     def nested_progress_bar(self):
         """Return a nested progress bar.
