@@ -1,4 +1,4 @@
-# Copyright (C) 2005 by Canonical Ltd
+# Copyright (C) 2005, 2006 by Canonical Ltd
 #
 # Authors:
 #   Johan Rydberg <jrydberg@gnu.org>
@@ -7,12 +7,12 @@
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -26,11 +26,14 @@ from unittest import TestSuite
 
 import bzrlib.errors as errors
 from bzrlib.inter import InterObject
-from bzrlib.symbol_versioning import *
 from bzrlib.textmerge import TextMerge
 from bzrlib.transport.memory import MemoryTransport
 from bzrlib.tsort import topo_sort
 from bzrlib import ui
+from bzrlib.symbol_versioning import (deprecated_function,
+        deprecated_method,
+        zero_eight,
+        )
 
 
 class VersionedFile(object):
@@ -54,7 +57,7 @@ class VersionedFile(object):
     def copy_to(self, name, transport):
         """Copy this versioned file to name on transport."""
         raise NotImplementedError(self.copy_to)
-    
+
     @deprecated_method(zero_eight)
     def names(self):
         """Return a list of all the versions in this versioned file.
@@ -170,8 +173,19 @@ class VersionedFile(object):
         if self._access_mode != 'w':
             raise errors.ReadOnlyObjectDirtiedError(self)
 
+    def enable_cache(self):
+        """Tell this versioned file that it should cache any data it reads.
+        
+        This is advisory, implementations do not have to support caching.
+        """
+        pass
+    
     def clear_cache(self):
-        """Remove any data cached in the versioned file object."""
+        """Remove any data cached in the versioned file object.
+
+        This only needs to be supported if caches are supported
+        """
+        pass
 
     def clone_text(self, new_version_id, old_version_id, parents):
         """Add an identical text to old_version_id as new_version_id.
@@ -252,6 +266,14 @@ class VersionedFile(object):
         return ''.join(self.get_lines(version_id))
     get_string = get_text
 
+    def get_texts(self, version_ids):
+        """Return the texts of listed versions as a list of strings.
+
+        Raises RevisionNotPresent if version is not present in
+        file history.
+        """
+        return [''.join(self.get_lines(v)) for v in version_ids]
+
     def get_lines(self, version_id):
         """Return version contents as a sequence of lines.
 
@@ -287,7 +309,7 @@ class VersionedFile(object):
         
         Ghosts are not listed or referenced in the graph.
         :param version_ids: Versions to select.
-                            None means retreive all versions.
+                            None means retrieve all versions.
         """
         result = {}
         if version_ids is None:
@@ -388,7 +410,7 @@ class VersionedFile(object):
         specific version marker at this point. The api may be changed
         during development to include the version that the versioned file
         thinks is relevant, but given that such hints are just guesses,
-        its better not to have it if we dont need it.
+        its better not to have it if we don't need it.
 
         NOTES: Lines are normalised: they will all have \n terminators.
                Lines are returned in arbitrary order.
@@ -483,7 +505,7 @@ class PlanWeaveMerge(TextMerge):
        
         # We previously considered either 'unchanged' or 'killed-both' lines
         # to be possible places to resynchronize.  However, assuming agreement
-        # on killed-both lines may be too agressive. -- mbp 20060324
+        # on killed-both lines may be too aggressive. -- mbp 20060324
         for state, line in self.plan:
             if state == 'unchanged':
                 # resync and flush queued conflicts changes if any
@@ -536,7 +558,7 @@ class InterVersionedFile(InterObject):
     InterVersionedFile.get(other).method_name(parameters).
     """
 
-    _optimisers = set()
+    _optimisers = []
     """The available optimised InterVersionedFile types."""
 
     def join(self, pb=None, msg=None, version_ids=None, ignore_missing=False):
@@ -638,7 +660,7 @@ class InterVersionedFile(InterObject):
 class InterVersionedFileTestProviderAdapter(object):
     """A tool to generate a suite testing multiple inter versioned-file classes.
 
-    This is done by copying the test once for each interversionedfile provider
+    This is done by copying the test once for each InterVersionedFile provider
     and injecting the transport_server, transport_readonly_server,
     versionedfile_factory and versionedfile_factory_to classes into each copy.
     Each copy is also given a new id() to make it easy to identify.
