@@ -1,17 +1,17 @@
 #! /usr/bin/python
 
 # Copyright (C) 2005 Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -27,7 +27,7 @@
 # property.
 
 # TODO: Nothing here so far assumes the lines are really \n newlines,
-# rather than being split up in some other way.  We could accomodate
+# rather than being split up in some other way.  We could accommodate
 # binaries, perhaps by naively splitting on \n or perhaps using
 # something like a rolling checksum.
 
@@ -70,10 +70,10 @@
 
 from copy import copy
 from cStringIO import StringIO
-from difflib import SequenceMatcher
 import os
 import sha
 import time
+import warnings
 
 from bzrlib.trace import mutter
 from bzrlib.errors import (WeaveError, WeaveFormatError, WeaveParentMismatch,
@@ -84,7 +84,11 @@ from bzrlib.errors import (WeaveError, WeaveFormatError, WeaveParentMismatch,
         )
 import bzrlib.errors as errors
 from bzrlib.osutils import sha_strings
-from bzrlib.symbol_versioning import *
+import bzrlib.patiencediff
+from bzrlib.symbol_versioning import (deprecated_method,
+        deprecated_function,
+        zero_eight,
+        )
 from bzrlib.tsort import topo_sort
 from bzrlib.versionedfile import VersionedFile, InterVersionedFile
 from bzrlib.weavefile import _read_weave_v5, write_weave_v5
@@ -180,9 +184,9 @@ class Weave(VersionedFile):
     """
 
     __slots__ = ['_weave', '_parents', '_sha1s', '_names', '_name_map',
-                 '_weave_name']
+                 '_weave_name', '_matcher']
     
-    def __init__(self, weave_name=None, access_mode='w'):
+    def __init__(self, weave_name=None, access_mode='w', matcher=None):
         super(Weave, self).__init__(access_mode)
         self._weave = []
         self._parents = []
@@ -190,6 +194,10 @@ class Weave(VersionedFile):
         self._names = []
         self._name_map = {}
         self._weave_name = weave_name
+        if matcher is None:
+            self._matcher = bzrlib.patiencediff.PatienceSequenceMatcher
+        else:
+            self._matcher = matcher
 
     def __repr__(self):
         return "Weave(%r)" % self._weave_name
@@ -227,7 +235,7 @@ class Weave(VersionedFile):
 
     @deprecated_method(zero_eight)
     def lookup(self, name):
-        """Backwards compatability thunk:
+        """Backwards compatibility thunk:
 
         Return name, as name is valid in the api now, and spew deprecation
         warnings everywhere.
@@ -257,7 +265,7 @@ class Weave(VersionedFile):
 
     def has_version(self, version_id):
         """See VersionedFile.has_version."""
-        return self._name_map.has_key(version_id)
+        return (version_id in self._name_map)
 
     __contains__ = has_version
 
@@ -518,7 +526,7 @@ class Weave(VersionedFile):
         if lines == basis_lines:
             return new_version            
 
-        # add a sentinal, because we can also match against the final line
+        # add a sentinel, because we can also match against the final line
         basis_lineno.append(len(self._weave))
 
         # XXX: which line of the weave should we really consider
@@ -528,7 +536,7 @@ class Weave(VersionedFile):
         #print 'basis_lines:', basis_lines
         #print 'new_lines:  ', lines
 
-        s = SequenceMatcher(None, basis_lines, lines)
+        s = self._matcher(None, basis_lines, lines)
 
         # offset gives the number of lines that have been inserted
         # into the weave up to the current point; if the original edit instruction
@@ -631,7 +639,7 @@ class Weave(VersionedFile):
 
     def annotate(self, version_id):
         if isinstance(version_id, int):
-            warn('Weave.annotate(int) is deprecated. Please use version names'
+            warnings.warn('Weave.annotate(int) is deprecated. Please use version names'
                  ' in all circumstances as of 0.8',
                  DeprecationWarning,
                  stacklevel=2
@@ -1065,7 +1073,7 @@ class Weave(VersionedFile):
 
     @deprecated_method(zero_eight)
     def reweave(self, other, pb=None, msg=None):
-        """reweave has been superceded by plain use of join."""
+        """reweave has been superseded by plain use of join."""
         return self.join(other, pb, msg)
 
     def _reweave(self, other, pb, msg):
@@ -1124,7 +1132,7 @@ class WeaveFile(Weave):
         sio = StringIO()
         write_weave_v5(self, sio)
         sio.seek(0)
-        transport.put(name + WeaveFile.WEAVE_SUFFIX, sio, self._filemode)
+        transport.put_file(name + WeaveFile.WEAVE_SUFFIX, sio, self._filemode)
 
     def create_empty(self, name, transport, filemode=None):
         return WeaveFile(name, transport, filemode, create=True)
@@ -1135,9 +1143,9 @@ class WeaveFile(Weave):
         sio = StringIO()
         write_weave_v5(self, sio)
         sio.seek(0)
-        self._transport.put(self._weave_name + WeaveFile.WEAVE_SUFFIX,
-                            sio,
-                            self._filemode)
+        self._transport.put_file(self._weave_name + WeaveFile.WEAVE_SUFFIX,
+                                 sio,
+                                 self._filemode)
 
     @staticmethod
     def get_suffixes():
@@ -1235,7 +1243,7 @@ def weave_stats(weave_file, pb):
     from bzrlib.weavefile import read_weave
 
     wf = file(weave_file, 'rb')
-    w = read_weave(wf, WeaveVersionedFile)
+    w = read_weave(wf)
     # FIXME: doesn't work on pipes
     weave_size = wf.tell()
 
@@ -1356,13 +1364,12 @@ def main(argv):
         sys.stdout.writelines(w.get_iter(int(argv[3])))
         
     elif cmd == 'diff':
-        from difflib import unified_diff
         w = readit()
         fn = argv[2]
         v1, v2 = map(int, argv[3:5])
         lines1 = w.get(v1)
         lines2 = w.get(v2)
-        diff_gen = unified_diff(lines1, lines2,
+        diff_gen = bzrlib.patiencediff.unified_diff(lines1, lines2,
                                 '%s version %d' % (fn, v1),
                                 '%s version %d' % (fn, v2))
         sys.stdout.writelines(diff_gen)
@@ -1416,47 +1423,9 @@ def main(argv):
         raise ValueError('unknown command %r' % cmd)
     
 
-
-def profile_main(argv):
-    import tempfile, hotshot, hotshot.stats
-
-    prof_f = tempfile.NamedTemporaryFile()
-
-    prof = hotshot.Profile(prof_f.name)
-
-    ret = prof.runcall(main, argv)
-    prof.close()
-
-    stats = hotshot.stats.load(prof_f.name)
-    #stats.strip_dirs()
-    stats.sort_stats('cumulative')
-    ## XXX: Might like to write to stderr or the trace file instead but
-    ## print_stats seems hardcoded to stdout
-    stats.print_stats(20)
-            
-    return ret
-
-
-def lsprofile_main(argv): 
-    from bzrlib.lsprof import profile
-    ret,stats = profile(main, argv)
-    stats.sort()
-    stats.pprint()
-    return ret
-
-
 if __name__ == '__main__':
     import sys
-    if '--profile' in sys.argv:
-        args = sys.argv[:]
-        args.remove('--profile')
-        sys.exit(profile_main(args))
-    elif '--lsprof' in sys.argv:
-        args = sys.argv[:]
-        args.remove('--lsprof')
-        sys.exit(lsprofile_main(args))
-    else:
-        sys.exit(main(sys.argv))
+    sys.exit(main(sys.argv))
 
 
 class InterWeave(InterVersionedFile):
